@@ -1,7 +1,7 @@
 #ifndef GPIPE_COMMON___TABULAR_FORMAT__HPP
 #define GPIPE_COMMON___TABULAR_FORMAT__HPP
 
-/*  $Id: tabular_fmt.hpp 647188 2022-03-22 16:40:15Z ivanov $
+/*  $Id: tabular_fmt.hpp 654056 2022-08-08 15:05:05Z dicuccio $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -624,6 +624,50 @@ private:
     string m_Text;
 };
 
+/////////////////////////////////////////////////////////////////////////////
+///
+/// formatter for Shannon's entropy
+
+class CTabularFormatter_Entropy : public CTabularFormatter::IFormatter
+{
+public:
+    CTabularFormatter_Entropy(size_t row)
+        : m_Row(row)
+    {
+    }
+
+    void PrintHelpText(CNcbiOstream& ostr) const;
+    void PrintHeader(CNcbiOstream& ostr) const;
+    void Print(CNcbiOstream& ostr,
+               const objects::CSeq_align& align);
+
+private:
+    size_t m_Row;
+};
+
+
+/////////////////////////////////////////////////////////////////////////////
+///
+/// formatter for BLAST seg %
+
+class CTabularFormatter_SegPercent : public CTabularFormatter::IFormatter
+{
+public:
+    CTabularFormatter_SegPercent(size_t row)
+        : m_Row(row)
+    {
+    }
+
+    void PrintHelpText(CNcbiOstream& ostr) const;
+    void PrintHeader(CNcbiOstream& ostr) const;
+    void Print(CNcbiOstream& ostr,
+               const objects::CSeq_align& align);
+
+private:
+    size_t m_Row;
+};
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 class CTabularFormatter_AlignLengthRatio : public CTabularFormatter::IFormatter
