@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_HTTP_SESSION__HPP
 #define CONNECT___NCBI_HTTP_SESSION__HPP
 
-/* $Id: ncbi_http_session.hpp 638349 2021-09-29 11:35:24Z grichenk $
+/* $Id: ncbi_http_session.hpp 651854 2022-06-27 04:30:34Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -47,20 +47,16 @@
 BEGIN_NCBI_SCOPE
 
 
-// Forward declarations for shortcut functions.
-class CHttpResponse;
-class CTlsCertCredentials;
-
 // Default retries value.
 struct SGetHttpDefaultRetries
 {
     unsigned short operator()(void) const;
 };
 
-
 /// Nullable retries for CHttpRequest
 typedef CNullable<unsigned short, SGetHttpDefaultRetries> THttpRetries;
 
+
 class NCBI_XCONNECT_EXPORT CHttpHeaders : public CObject
 {
 public:
@@ -152,8 +148,8 @@ public:
     virtual ~CHttpHeaders(void) {}
 
 private:
-    friend class CHttpResponse;
     friend class CHttpRequest;
+    friend class CHttpResponse;
 
     // Prohibit copying headers.
     CHttpHeaders(const CHttpHeaders&);
@@ -208,6 +204,10 @@ private:
 };
 
 
+class CHttpResponse;
+class CTlsCertCredentials;
+
+
 /// CHttpSession and CHttpRequest parameters.
 class NCBI_XCONNECT_EXPORT CHttpParam
 {
@@ -332,8 +332,6 @@ CHttpResponse g_HttpPut(const CUrl&         url,
                         const CTimeout&     timeout = CTimeout(CTimeout::eDefault),
                         THttpRetries        retries = null);
 
-class CHttpSession_Base;
-
 
 /// Interface for custom form data providers.
 class NCBI_XCONNECT_EXPORT CFormDataProvider_Base : public CObject
@@ -468,6 +466,10 @@ public:
 };
 
 
+class CHttpRequest;
+class CHttpSession_Base;
+
+
 /// HTTP response
 class NCBI_XCONNECT_EXPORT CHttpResponse : public CObject
 {
@@ -650,7 +652,7 @@ private:
 
     // Open connection, initialize response.
     void x_InitConnection(bool use_form_data);
-    void x_InitConnection2(shared_ptr<iostream> stream, bool is_service);
+    void x_InitConnection2(shared_ptr<iostream> stream);
 
     bool x_CanSendData(void) const;
 
@@ -674,27 +676,26 @@ private:
                          unsigned int  failure_count);
 
     CRef<CHttpSession_Base> m_Session;
-    CUrl                m_Url;
-    bool                m_IsService;
-    EReqMethod          m_Method;
-    CRef<CHttpHeaders>  m_Headers;
-    CRef<CHttpFormData> m_FormData;
-    shared_ptr<iostream> m_Stream;
-    CRef<CHttpResponse> m_Response; // current response or null
-    CTimeout            m_Timeout;
-    THttpRetries        m_Retries;
-    CTimeout            m_Deadline;
-    ESwitch             m_RetryProcessing;
+    CUrl                    m_Url;
+    EReqMethod              m_Method;
+    CRef<CHttpHeaders>      m_Headers;
+    CRef<CHttpFormData>     m_FormData;
+    shared_ptr<iostream>    m_Stream;
+    CRef<CHttpResponse>     m_Response; // current response or null
+    CTimeout                m_Timeout;
+    THttpRetries            m_Retries;
+    CTimeout                m_Deadline;
+    ESwitch                 m_RetryProcessing;
     CRef<CAdjustUrlCallback_Base> m_AdjustUrl;
     // Store credentials so that they are not destroyed while request is being executed.
     shared_ptr<CTlsCertCredentials> m_Credentials;
-    CHttpProxy          m_Proxy;
+    CHttpProxy              m_Proxy;
 };
 
 
 /// HTTP session class, holding common data for multiple requests.
 class NCBI_XCONNECT_EXPORT CHttpSession_Base : public CObject,
-                                          virtual protected CConnIniter
+                                               virtual protected CConnIniter
 {
 public:
     /// Supported request methods, proxy for EReqMethod.
