@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBI_URL__HPP
 #define CORELIB___NCBI_URL__HPP
 
-/*  $Id: ncbi_url.hpp 613672 2020-08-11 16:34:37Z grichenk $
+/*  $Id: ncbi_url.hpp 651998 2022-06-29 04:27:29Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -169,7 +169,17 @@ private:
 class NCBI_XNCBI_EXPORT CUrlArgs_Parser
 {
 public:
-    CUrlArgs_Parser(void) : m_SemicolonIsNotArgDelimiter(false) {}
+    /// An inverted subset of CCgiRequest::TFlags
+    /// @sa CCgiRequest
+    typedef int TFlags;
+    enum Flags {
+        /// Treat semicolon as query string argument separator
+        fSemicolonIsArgDelimiter    = (1 << 11),
+        /// Enable parsing input as 'indexed' query (RFC3875) when no '=' is present
+        fEnableParsingAsIndex       = (1 << 14),
+    };
+
+    CUrlArgs_Parser(TFlags flags = 0) : m_Flags(flags) {}
     virtual ~CUrlArgs_Parser(void) {}
 
     /// Parse query string, call AddArgument() to store each value.
@@ -181,7 +191,11 @@ public:
     /// Treat semicolon as query string argument separator
     void SetSemicolonIsNotArgDelimiter(bool enable = true)
     {
-        m_SemicolonIsNotArgDelimiter = enable;
+        if (enable) {
+            m_Flags &= ~fSemicolonIsArgDelimiter;
+        } else {
+            m_Flags |= fSemicolonIsArgDelimiter;
+        }
     }
 
 protected:
@@ -206,11 +220,12 @@ protected:
                              const string& name,
                              const string& value,
                              EArgType      arg_type = eArg_Index) = 0;
+
+    TFlags m_Flags;
+
 private:
     void x_SetIndexString(const string& query,
                           const IUrlEncoder& encoder);
-
-    bool m_SemicolonIsNotArgDelimiter;
 };
 
 
@@ -225,11 +240,11 @@ class NCBI_XNCBI_EXPORT CUrlArgs : public CUrlArgs_Parser
 {
 public:
     /// Create an empty arguments set.
-    CUrlArgs(void);
+    CUrlArgs(TFlags flags = 0);
     /// Parse the query string, store the arguments.
-    CUrlArgs(const string& query, NStr::EUrlEncode decode);
+    CUrlArgs(const string& query, NStr::EUrlEncode decode, TFlags flags = 0);
     /// Parse the query string, store the arguments.
-    CUrlArgs(const string& query, const IUrlEncoder* encoder = 0);
+    CUrlArgs(const string& query, const IUrlEncoder* encoder = 0, TFlags flags = 0);
 
     /// Ampersand encoding for composed URLs
     enum EAmpEncoding {
@@ -349,6 +364,9 @@ public:
     ///   @sa CDefaultUrlEncoder
     CUrl(const string& url, const IUrlEncoder* encoder = 0);
 
+    CUrl(const char* url, const IUrlEncoder* encoder = 0)
+        : CUrl(string(url), encoder) {}
+
     /// Parse the URL.
     ///
     /// @param url
@@ -444,7 +462,7 @@ public:
     typedef int TAdjustFlags;
 
     /// Adjust this URL using information from 'other' URL.
-    /// Scheme, host and port are never changed. Other parts can be replaced or merged
+    /// Host and port are never changed. Other parts can be replaced or merged
     /// depending on the flags.
     /// Throw CUrlException if the flags are inconsistent (e.g. both fPath_Replace and fPath_Append are set).
     void Adjust(const CUrl& other, TAdjustFlags flags);
