@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBISTRE__HPP
 #define CORELIB___NCBISTRE__HPP
 
-/*  $Id: ncbistre.hpp 636804 2021-08-31 14:55:09Z gouriano $
+/*  $Id: ncbistre.hpp 655453 2022-09-02 07:15:42Z fukanchi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -117,6 +117,9 @@
 #include <stddef.h>
 
 
+#define HTTP_EOL "\r\n"
+
+
 // (BEGIN_NCBI_SCOPE must be followed by END_NCBI_SCOPE later in this file)
 BEGIN_NCBI_SCOPE
 
@@ -148,7 +151,9 @@ typedef IO_PREFIX::ostream       CNcbiOstream;
 /// Portable alias for iostream.
 typedef IO_PREFIX::iostream      CNcbiIostream;
 
+
 #ifndef NCBI_SHUN_OSTRSTREAM
+
 /// Portable alias for strstreambuf.
 //typedef IO_PREFIX::strstreambuf  CNcbiStrstreambuf;
 
@@ -285,7 +290,8 @@ template class NCBI_XNCBI_EXPORT CNcbistrstream_Base<IO_PREFIX::ostrstream, IOS_
 template class NCBI_XNCBI_EXPORT CNcbistrstream_Base<IO_PREFIX::strstream,  IOS_BASE::in | IOS_BASE::out>;
 #endif
 
-#else
+#else  // NCBI_SHUN_OSTRSTREAM
+
 #if 0
 //typedef IO_PREFIX::stringbuf      CNcbiStrstreambuf;
 typedef IO_PREFIX::istringstream  CNcbiIstrstream;
@@ -310,11 +316,16 @@ public:
     template<class T>
     using enable_if_integral = typename std::enable_if<std::is_integral<T>::value>;
 
+#if 0
     template< typename TInteger, typename = typename enable_if_integral<TInteger>::type >
     [[deprecated("(const char*, Tinteger, ios::openmode) constructor is deprecated, use string argument instead")]]
 	CNcbistrstream_Base(const char *_Ptr, TInteger _Count, IOS_BASE::openmode _Mode = _DefMode)
 		: _Mybase(string(_Ptr, _Count), _Mode) {
 	}
+#else
+    template< typename TInteger, typename = typename enable_if_integral<TInteger>::type >
+	CNcbistrstream_Base(const char *_Ptr, TInteger _Count, IOS_BASE::openmode _Mode = _DefMode) = delete;
+#endif
 #else
     [[deprecated("(const char*, streamsize, ios::openmode) constructor is deprecated, use string argument instead")]]
     CNcbistrstream_Base(const char* s, streamsize n, IOS_BASE::openmode _Mode = _DefMode)
@@ -354,7 +365,9 @@ template class NCBI_XNCBI_EXPORT CNcbistrstream_Base<IO_PREFIX::ostringstream, I
 template class NCBI_XNCBI_EXPORT CNcbistrstream_Base<IO_PREFIX::stringstream,  IOS_BASE::in | IOS_BASE::out>;
 #endif
 #endif
-#endif
+
+#endif  // NCBI_SHUN_OSTRSTREAM
+
 
 /// Portable alias for filebuf.
 typedef IO_PREFIX::filebuf       CNcbiFilebuf;
@@ -612,20 +625,35 @@ extern CNcbiIstream& NcbiGetlineEOL(CNcbiIstream& is, string& str,
 /// checking for errors after doing a "close()" (if "os" is an ofstream, for
 /// example).  For instance, uploading into the Toolkit FTP stream must be
 /// finalized with a read for the byte count delivered;  otherwise, it may not
-///  work correctly.
+/// work correctly.
 /// @sa
-///   CConn_IOStream
+///   CConn_IOStream, NcbiStreamCopyStart, NcbiStreamCopyThrow
 NCBI_XNCBI_EXPORT
 extern bool NcbiStreamCopy(CNcbiOstream& os, CNcbiIstream& is);
 
 
 /// Same as NcbiStreamCopy() but throws an CCoreException when copy fails.
 /// @sa
-///   NcbiStreamCopy, CCoreException
+///   NcbiStreamCopy, NcbiStreamCopyHead, CCoreException
 NCBI_XNCBI_EXPORT
 extern void NcbiStreamCopyThrow(CNcbiOstream& os, CNcbiIstream& is);
 
 
+/// Similar to NcbiStreamCopyThrow() but copies (and consumes) only the
+/// specified number of bytes (and bypasses operator<< altogether).  In
+/// the event of write failure, the number of bytes consumed may fall
+/// anywhere between the number of bytes written and the target count,
+/// inclusive.  If the input stream has less data than requested, copies
+/// what it can, sets that stream's eof and fail flags, and returns
+/// normally; if anything else goes wrong, throws an exception in
+/// addition to setting appropriate flags (and copying what it can).
+/// @sa
+///   NcbiStreamCopy, NcbiStreamCopyThrow, CCoreException
+NCBI_XNCBI_EXPORT
+extern void NcbiStreamCopyHead(CNcbiOstream& os, CNcbiIstream& is,
+                               string::size_type count);
+
+
 /// Input the entire contents of an istream into a string (NULL causes drain).
 /// "is" gets its failbit set if nothing was extracted from it;  and gets its
 /// eofbit (w/o failbit) set if the stream has reached an EOF condition.
