@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_READER___SEQDBCOMMON__HPP
 #define OBJTOOLS_BLAST_SEQDB_READER___SEQDBCOMMON__HPP
 
-/*  $Id: seqdbcommon.hpp 650083 2022-05-16 17:20:21Z ivanov $
+/*  $Id: seqdbcommon.hpp 651939 2022-06-28 12:33:29Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -394,6 +394,10 @@ public:
         return ! Empty();
     }
 
+    bool HasIdFilters() const
+    {
+        return (GetNumGis() || GetNumSis() || GetNumTis());
+    }
     /// Specify the correct OID for a GI.
     ///
     /// When SeqDB translates a GI into an OID, this method is called
