@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_WRITER___WRITEDB__HPP
 #define OBJTOOLS_BLAST_SEQDB_WRITER___WRITEDB__HPP
 
-/*  $Id: writedb.hpp 650083 2022-05-16 17:20:21Z ivanov $
+/*  $Id: writedb.hpp 651607 2022-06-21 14:07:14Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -697,6 +697,14 @@ void CWriteDB_ConsolidateAliasFiles(const list<string>& alias_files,
 NCBI_XOBJWRITE_EXPORT
 void CWriteDB_ConsolidateAliasFiles(bool delete_source_alias_files = false);
 
+
+NCBI_XOBJWRITE_EXPORT
+void CWriteDB_CreateOidMaskDB(const string& input_db,
+							  const string & output_db,
+                              CWriteDB::ESeqType seq_type,
+                              int oid_mask_type,
+                              const string & title = string());
+
 END_NCBI_SCOPE
 
 #endif // OBJTOOLS_BLAST_SEQDB_WRITER___WRITEDB__HPP
