@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___CONTEXT__HPP
 #define OBJTOOLS_FORMAT___CONTEXT__HPP
 
-/*  $Id: context.hpp 642718 2021-12-27 19:04:47Z fukanchi $
+/*  $Id: context.hpp 656070 2022-09-14 12:55:32Z fukanchi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -229,6 +229,15 @@ public:
     };
     typedef Int8 TUnverified;
     TUnverified GetUnverifiedType(void) const;
+    enum FUnreviewed {
+        fUnreviewed_None = 0, // for convenience
+
+        fUnreviewed_Unannotated = 1 << 0
+        // if you add more here, make sure
+        // to update CFlatGatherer::x_UnreviewedComment
+    };
+    typedef Int8 TUnreviewed;
+    TUnreviewed GetUnreviewedType(void) const;
     bool ShowAnnotCommentAsCOMMENT() const;
 
     bool IsHup(void) const { return m_IsHup; }  // !!! should move to global?
@@ -368,6 +377,7 @@ private:
     mutable bool m_IsCrossKingdom;
     mutable bool m_UsePDBCompoundForComment;
     TUnverified m_fUnverified;
+    TUnreviewed m_fUnreviewed;
     mutable bool m_ShowAnnotCommentAsCOMMENT;
     mutable bool m_ShowAnnotCommentAsCOMMENT_checked;
 
@@ -656,6 +666,12 @@ CBioseqContext::TUnverified CBioseqContext::GetUnverifiedType(void) const
     return m_fUnverified;
 }
 
+inline
+CBioseqContext::TUnreviewed CBioseqContext::GetUnreviewedType(void) const
+{
+    return m_fUnreviewed;
+}
+
 inline
 bool CBioseqContext::UsingSeqEntryIndex(void) const
 {
