@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___FLAT_FILE_CONFIG__HPP
 #define OBJTOOLS_FORMAT___FLAT_FILE_CONFIG__HPP
 
-/*  $Id: flat_file_config.hpp 650337 2022-05-23 12:38:09Z ivanov $
+/*  $Id: flat_file_config.hpp 650159 2022-05-18 12:17:58Z drozdov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -676,7 +676,7 @@ public:
     // check if the given section is shown
     bool IsShownGenbankBlock(FGenbankBlocks fTGenbankBlocksMask) const
     {
-        return (m_fGenbankBlocks & fTGenbankBlocksMask) == fTGenbankBlocksMask;
+        return m_fGenbankBlocks & fTGenbankBlocksMask;
     }
 
     // set the given section to be shown
