@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS__PUBSEQ_GATEWAY__PSG_CLIENT_HPP
 #define OBJTOOLS__PUBSEQ_GATEWAY__PSG_CLIENT_HPP
 
-/*  $Id: psg_client.hpp 642693 2021-12-27 13:58:40Z fukanchi $
+/*  $Id: psg_client.hpp 649484 2022-05-04 16:46:38Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -97,6 +97,14 @@ struct SPSG_UserArgs : unordered_map<string, set<string>>
 class CPSG_Request
 {
 public:
+    enum EType {
+        eBiodata,
+        eResolve,
+        eBlob,
+        eNamedAnnotInfo,
+        eChunk,
+    };
+
     /// Get the user-provided context
     template<typename TUserContext>
     shared_ptr<TUserContext> GetUserContext() const
@@ -109,15 +117,24 @@ public:
     void SetRequestContext(CRef<CRequestContext> request_context);
 
     /// Get request type
-    string GetType() const { return x_GetType(); }
+    /// @{
+    /// A temporary struct to migrate from a string type to an enum one (EType)
+    struct SType
+    {
+        operator EType() const { return m_Type; }
+        NCBI_DEPRECATED operator string() const; ///< @deprecated use EType instead
+        NCBI_DEPRECATED friend ostream& operator<<(ostream& os, const SType& type); ///< @warning Output will be changed after migration
+    private:
+        SType(EType type) : m_Type(type) {}
+        EType m_Type;
+        friend class CPSG_Request;
+    };
+    SType GetType() const { return x_GetType(); }
+    /// @}
 
     // Get request ID
     string GetId() const { return x_GetId(); }
 
-    /// Set hops
-    /// @deprecated use SetUserArgs instead
-    NCBI_DEPRECATED void SetHops(unsigned hops) { m_Hops = hops; }
-
     /// Set arbitrary URL arguments to add to this request.
     /// @code
     /// request->SetUserArgs("param1=value1&param1=value2&param2=value3");
@@ -137,13 +154,12 @@ protected:
     virtual ~CPSG_Request() = default;
 
 private:
-    virtual string x_GetType() const = 0;
+    virtual EType x_GetType() const = 0;
     virtual string x_GetId() const = 0;
     virtual void x_GetAbsPathRef(ostream&) const = 0;
 
     shared_ptr<void> m_UserContext;
     CRef<CRequestContext> m_RequestContext;
-    unsigned m_Hops = 0;
     SPSG_UserArgs m_UserArgs;
 
     friend class CPSG_Queue;
@@ -318,16 +334,11 @@ public:
     /// Set substitution policy for version-less primary seq-ids
     void SetAccSubstitution(EPSG_AccSubstitution acc_substitution) { m_AccSubstitution = acc_substitution; }
 
-    /// Enable/disable auto blob skipping on server for this request
-    /// @deprecated use SetResendTimeout instead
-    NCBI_DEPRECATED
-    void SetAutoBlobSkipping(bool auto_blob_skipping) { m_AutoBlobSkipping = auto_blob_skipping ? eOn : eOff; }
-
     /// Set resend timeout
     void SetResendTimeout(CTimeout resend_timeout) { m_ResendTimeout = move(resend_timeout); }
 
 private:
-    string x_GetType() const override { return "biodata"; }
+    EType x_GetType() const override { return eBiodata; }
     string x_GetId() const override { return GetBioId().Repr(); }
     void x_GetAbsPathRef(ostream&) const override;
 
@@ -335,7 +346,6 @@ private:
     EIncludeData  m_IncludeData = EIncludeData::eDefault;
     TExcludeTSEs  m_ExcludeTSEs;
     EPSG_AccSubstitution m_AccSubstitution = EPSG_AccSubstitution::Default;
-    ESwitch m_AutoBlobSkipping = ESwitch::eDefault;
     CTimeout m_ResendTimeout = CTimeout::eDefault;
 };
 
@@ -383,7 +393,7 @@ public:
     void SetAccSubstitution(EPSG_AccSubstitution acc_substitution) { m_AccSubstitution = acc_substitution; }
 
 private:
-    string x_GetType() const override { return "resolve"; }
+    EType x_GetType() const override { return eResolve; }
     string x_GetId() const override { return GetBioId().Repr(); }
     void x_GetAbsPathRef(ostream&) const override;
 
@@ -417,7 +427,7 @@ public:
     EIncludeData GetIncludeData() const { return m_IncludeData; }
 
 private:
-    string x_GetType() const override { return "blob"; }
+    EType x_GetType() const override { return eBlob; }
     string x_GetId() const override { return GetBlobId().Repr(); }
     void x_GetAbsPathRef(ostream&) const override;
 
@@ -463,7 +473,7 @@ public:
     EIncludeData GetIncludeData() const { return m_IncludeData; }
 
 private:
-    string x_GetType() const override { return "annot"; }
+    EType x_GetType() const override { return eNamedAnnotInfo; }
     string x_GetId() const override { return GetBioId().Repr(); }
     void x_GetAbsPathRef(ostream&) const override;
 
@@ -491,7 +501,7 @@ public:
     const CPSG_ChunkId& GetChunkId() const { return m_ChunkId; }
 
 private:
-    string x_GetType() const override { return "chunk"; }
+    EType x_GetType() const override { return eChunk; }
     string x_GetId() const override { return GetChunkId().Repr(); }
     void x_GetAbsPathRef(ostream&) const override;
 
@@ -683,8 +693,9 @@ public:
         eUnknown,    // Skipped for unknown reason
     };
 
-    /// Get blob ID
-    const CPSG_BlobId& GetId() const { return m_Id; }
+    /// Get data ID
+    template <class TDataId = CPSG_DataId>
+    const TDataId* GetId() const { return dynamic_cast<const TDataId*>(m_Id.get()); }
 
     // Get reason for blob skipping
     EReason GetReason() const { return m_Reason; }
@@ -696,9 +707,9 @@ public:
     const TSeconds& GetTimeUntilResend() const { return m_TimeUntilResend; }
 
 private:
-    CPSG_SkippedBlob(CPSG_BlobId id, EReason reason, TSeconds sent_seconds_ago, TSeconds time_until_resend);
+    CPSG_SkippedBlob(unique_ptr<CPSG_DataId> id, EReason reason, TSeconds sent_seconds_ago, TSeconds time_until_resend);
 
-    CPSG_BlobId m_Id;
+    unique_ptr<CPSG_DataId> m_Id;
     EReason     m_Reason;
     TSeconds    m_SentSecondsAgo;
     TSeconds    m_TimeUntilResend;
@@ -844,6 +855,34 @@ private:
 
 
 
+/// Processor event
+
+class CPSG_Processor : public CPSG_ReplyItem
+{
+public:
+    enum EProgressStatus {
+        eStart,
+        eDone,
+        eNotFound,
+        eCanceled,
+        eTimeout,
+        eError,
+        eUnknown,
+    };
+
+    /// Get progress status
+    EProgressStatus GetProgressStatus() const { return m_ProgressStatus; }
+
+private:
+    CPSG_Processor(EProgressStatus progress_status);
+
+    EProgressStatus m_ProgressStatus;
+
+    friend class CPSG_Reply;
+};
+
+
+
 /// PSG reply -- corresponds to a PSG request. It is used to retrieve data 
 /// (accession resolution; bio-sequence; annotation blobs) from the storage.
 /// 
