@@ -1,6 +1,6 @@
 #ifndef SRA__READER__SRA__VDBREAD__HPP
 #define SRA__READER__SRA__VDBREAD__HPP
-/*  $Id: vdbread.hpp 629397 2021-04-15 14:17:16Z vasilche $
+/*  $Id: vdbread.hpp 653133 2022-07-21 14:50:05Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -238,6 +238,14 @@ public:
     // Commit configuration changes into config file
     void CommitConfig() const;
 
+    // Update VDB request context from Corelib's one
+    class CRequestContextUpdater {
+    public:
+        CRequestContextUpdater();
+        ~CRequestContextUpdater();
+        CRequestContextUpdater(const CRequestContextUpdater&) = delete;
+    };
+
 protected:
     void x_Init(void);
 
@@ -489,6 +497,8 @@ public:
             Init(cursor, element_bit_size, name, backup_name, missing);
         }
 
+    bool IsStatic(const CVDBCursor& cursor) const;
+    
     void Reset(void)
         {
             m_Index = kInvalidIndex;
