@@ -1,6 +1,6 @@
 #ifndef SRA__READER__SRA__WGSREAD__HPP
 #define SRA__READER__SRA__WGSREAD__HPP
-/*  $Id: wgsread.hpp 644751 2022-02-02 14:40:43Z ivanov $
+/*  $Id: wgsread.hpp 652960 2022-07-19 18:44:47Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -91,6 +91,7 @@ enum {
     NCBI_gb_state_eWGSGenBankMigrated = 6
 };
 
+typedef int32_t NCBI_WGS_hash;
 
 BEGIN_NCBI_NAMESPACE;
 
@@ -399,6 +400,15 @@ public:
         return !m_ReplacedBy.empty();
     }
 
+    bool HasCommonTaxId(void) const
+    {
+        return m_HasCommonTaxId;
+    }
+    TTaxId GetCommonTaxId(void) const
+    {
+        return m_CommonTaxId;
+    }
+    
 protected:
     friend class CWGSSeqIterator;
     friend class CWGSScaffoldIterator;
@@ -593,6 +603,7 @@ private:
     CSeq_inst::TMol m_ContigMolType;
     bool m_IsSetMasterDescr;
     bool m_HasNoDefaultGnlId;
+    bool m_HasCommonTaxId;
     EFeatLocIdType m_FeatLocIdType;
     CRef<CSeq_entry> m_MasterEntry;
     TMasterDescr m_MasterDescr;
@@ -600,6 +611,7 @@ private:
     NCBI_gb_state m_ProjectGBState;
     CSeq_id::E_Choice m_SeqIdType;
     string m_ReplacedBy;
+    TTaxId m_CommonTaxId;
 };
 DECLARE_SAFE_FLAGS(CWGSDb_Impl::EGnlIdFlags);
 DECLARE_SAFE_FLAGS(CWGSDb_Impl::EAllowRowType);
@@ -945,8 +957,9 @@ public:
     bool HasTaxId(void) const;
     TTaxId GetTaxId(void) const;
 
+    typedef NCBI_WGS_hash THash;
     bool HasSeqHash(void) const;
-    int GetSeqHash(void) const;
+    THash GetSeqHash(void) const;
 
     TVDBRowIdRange GetLocFeatRowIdRange(void) const;
 
@@ -1385,6 +1398,13 @@ public:
     CTempString GetProteinName(void) const;
     CTempString GetProductName(void) const;
 
+    bool HasTaxId(void) const;
+    TTaxId GetTaxId(void) const;
+    
+    typedef NCBI_WGS_hash THash;
+    bool HasSeqHash(void) const;
+    THash GetSeqHash(void) const;
+    
     TVDBRowIdRange GetLocFeatRowIdRange(void) const;
     size_t GetProductFeatCount(void) const;
     TVDBRowId GetProductFeatRowId(size_t index) const;
