@@ -1087,6 +1087,27 @@ void convert_bv2sv(SV& sv, const BV& bv)
     bit.flush();
 }
 
+#if 0
+/**
+    Get RSS on
+    @internal
+ */
+size_t getCurrentRSS( )
+{
+    long rss = 0L;
+    FILE* fp = NULL;
+    if ( (fp = fopen( "/proc/self/statm", "r" )) == NULL )
+        return (size_t)0L;      /* Can't open? */
+    if ( fscanf( fp, "%*s%ld", &rss ) != 1 )
+    {
+        fclose( fp );
+        return (size_t)0L;      /* Can't read? */
+    }
+    fclose( fp );
+    return (size_t)rss * (size_t)sysconf( _SC_PAGESIZE);
+}
+#endif
+
 
 } // namespace
 
