@@ -881,7 +881,7 @@ public:
     
     /*! \brief syncronize internal structures, build fast access index
     */
-    void sync(bool /*force*/) {}
+    void sync(bool /*force*/) { this->sync_ro(); }
     
     
     /*!
@@ -1066,11 +1066,10 @@ protected:
 
 protected:
     template<class V, class SV> friend class rsc_sparse_vector;
-    template<class SVect> friend class sparse_vector_scanner;
+    template<class SVect, unsigned S_FACTOR> friend class sparse_vector_scanner;
     template<class SVect> friend class sparse_vector_serializer;
     template<class SVect> friend class sparse_vector_deserializer;
 
-
 };
 
 
