@@ -1,7 +1,7 @@
 #ifndef UTIL_COMPRESS__COMPRESS__HPP
 #define UTIL_COMPRESS__COMPRESS__HPP
 
-/*  $Id: compress.hpp 612744 2020-07-27 14:26:09Z ivanov $
+/*  $Id: compress.hpp 656790 2022-09-28 12:23:44Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,6 +32,36 @@
  *
  */
 
+/// @file compress.hpp
+///
+/// @warning
+///   Compression API have support for some compression libraies. Toolkit checks
+///   existing compressions libraries on a configuration stage. If system version 
+///   of some compression library is not found, for some libraries like BZIP22, ZLIB, 
+///   GZIP we have an embedded alternatives, that will be used automatically. 
+///   But for some others, like LZO and ZSTD, we dont have such options.
+///   The configure define next macros, that can be used to put specific library
+///   related code to corresponding #if/#endif block to avoid compilation errors
+///   on some platforms that miss a specific compression library:
+/// 
+///   - HAVE_LIBZ       // for ZLIB and GZIP
+///   - HAVE_LIBBZ2     // for BZIP2
+///   - HAVE_LIBLZO     // for LZO
+///   - HAVE_LIBZSTD    // for ZSTD
+/// 
+///   #if defined(HAVE_LIB***)
+///       // use selected library related code here
+///   #else
+///      // some backup code, or error reporting
+///   #endif
+/// 
+///   Theoretically, you need a check for LZO and ZSTD only.
+/// 
+///   This is related to a library-specific classes only. Many other common compession 
+///   classes/interfaces can be used without such guards, like ICompression or streams,
+///   for example.
+
+
 #include <corelib/ncbistd.hpp>
 #include <corelib/version_api.hpp>
 
@@ -44,11 +74,13 @@
 
 BEGIN_NCBI_SCOPE
 
-
-/// Default compression I/O stream buffer size.
-const streamsize kCompressionDefaultBufSize = 16*1024;
-
 /// Macro to report errors in compression API.
+///
+/// Error codes for ERR_COMPRESS and OMPRESS_HANDLE_EXCEPTIONS are really
+/// a subcodes and current maximum  value is defined in 'include/util/error_codes.hpp':
+///     NCBI_DEFINE_ERRCODE_X(Util_Compress, 210, max);
+/// For new values use 'max'+1 and update it there.
+///
 #define ERR_COMPRESS(subcode, message) ERR_POST_X(subcode, Warning << message)
 
 /// Macro to catch and handle exceptions (from streams in the destructor)
@@ -75,6 +107,9 @@ const streamsize kCompressionDefaultBufSize = 16*1024;
         }                                                             \
     }                                                                 \
 
+/// Default compression I/O stream buffer size.
+const streamsize kCompressionDefaultBufSize = 16*1024;
+
 
 // Forward declaration
 class CCompressionFile;
@@ -93,12 +128,19 @@ public:
     ///
     /// It is in range [0..9]. Increase of level might mean better compression
     /// and usualy greater time of compression. Usualy 1 gives best speed,
-    /// 9 gives best compression, 0 gives no compression at all.
+    /// 9 gives best compression, 0 gives no compression at all (if supported).
     /// eDefault value requests a compromise between speed and compression
     /// (according to developers of the corresponding compression algorithm).
+    /// @note
+    ///   Each compression library/algorithm have its own set of compression levels,
+    ///   than not usually fit into [0..9] range. But for convenience of use and 
+    ///   easier switching we prowide conversion between levels, whehe "lowest"
+    ///   is a always a minimem allowed compression level, and "best" is a maximum.
+    ///   All other approximately fits in between.
+    /// 
     enum ELevel {
         eLevel_Default       = -1,  // default
-        eLevel_NoCompression =  0,  // just store data
+        eLevel_NoCompression =  0,  // just store data (if supported, or eLevel_Lowest)
         eLevel_Lowest        =  1,
         eLevel_VeryLow       =  2,
         eLevel_Low           =  3,
@@ -139,6 +181,19 @@ public:
     virtual TFlags GetFlags(void) const = 0;
     virtual void   SetFlags(TFlags flags) = 0;
 
+#if 0
+    /// Algorithm implememntation feature
+    /// Please don't change values  
+    enum EFeature {
+        f_NoCompressionLevel = 1,   ///< No compression method (copy "as is")
+        f_Dictionary         = 2,   ///< Dictionaries
+        f_EstimateCompressionBufferSize
+    };
+    /// Check if specified feature have support by a current compression
+    /// algorithm implememntation.
+    virtual bool HaveSupport(EFeature feature) = 0;
+#endif
+
     //
     // Utility functions 
     //
@@ -163,14 +218,18 @@ public:
     // (De)compress file "src_file" and put result to file "dst_file".
     // Return TRUE on success, FALSE on error.
     virtual bool CompressFile(
-        const string&     src_file,
-        const string&     dst_file,
-        size_t            buf_size = kCompressionDefaultBufSize
+        const string& src_file,
+        const string& dst_file,
+        size_t        file_io_bufsize         = kCompressionDefaultBufSize,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
     ) = 0;
     virtual bool DecompressFile(
-        const string&     src_file,
-        const string&     dst_file, 
-        size_t            buf_size = kCompressionDefaultBufSize
+        const string& src_file,
+        const string& dst_file, 
+        size_t        file_io_bufsize         = kCompressionDefaultBufSize,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
     ) = 0;
 };
 
@@ -208,18 +267,31 @@ public:
     /// Set flags.
     virtual void   SetFlags(TFlags flags);
 
+    /// Structure to get information about recommended buffer sizes for file/stream I/O
+    /// to tune up a (de)compression performance.
+    /// @sa GetRecommendedBufferSizes
+    struct SRecommendedBufferSizes {
+        size_t  compression_in;     ///< compression: recommended size for input buffer
+        size_t  compression_out;    ///< compression: recommended size for output buffer
+        size_t  decompression_in;   ///< decompression: recommended size for input buffer
+        size_t  decompression_out;  ///< decompression: recommended size for output buffer
+
+        // Round up buffer size to a value divisible by 'precision'
+        size_t RoundUp(size_t value, size_t precision);
+    };
+
 protected:
     // Universal file compression/decompression functions.
     // Return TRUE on success, FALSE on error.
     virtual bool x_CompressFile(
         const string&     src_file,
         CCompressionFile& dst_file,
-        size_t            buf_size = kCompressionDefaultBufSize
+        size_t            file_io_bufsize = kCompressionDefaultBufSize
     );
     virtual bool x_DecompressFile(
         CCompressionFile& src_file,
         const string&     dst_file,
-        size_t            buf_size = kCompressionDefaultBufSize
+        size_t            file_io_bufsize = kCompressionDefaultBufSize
     );
 
     // Set last action error/status code and description
@@ -233,7 +305,7 @@ protected:
         eMode_Decompress,      ///< Generic decompression
         eMode_TransparentRead  ///< Transparent read, the data is uncompressed
     };
-    ///< Decompress mode (Decompress/TransparentRead/Unknown).
+    /// Decompress mode (Decompress/TransparentRead/Unknown).
     EDecompressMode m_DecompressMode;
 
 private:
@@ -272,12 +344,16 @@ public:
 
     // 'ctors
     CCompressionFile(void);
-    CCompressionFile(const string& path, EMode mode); 
+    CCompressionFile(const string& path, EMode mode, 
+                     size_t compression_in_bufsize  = 0,
+                     size_t compression_out_bufsize = 0); 
     virtual ~CCompressionFile(void);
 
     /// Opens a compressed file for reading or writing.
     /// Return NULL if error has been occurred.
-    virtual bool Open(const string& path, EMode mode) = 0; 
+    virtual bool Open(const string& path, EMode mode, 
+                      size_t compression_in_bufsize  = 0,
+                      size_t compression_out_bufsize = 0) = 0; 
 
     /// Read up to "len" uncompressed bytes from the compressed file "file"
     /// into the buffer "buf". Return the number of bytes actually read
@@ -410,7 +486,7 @@ protected:
 private:
     size_t  m_ProcessedSize;  //< The number of processed bytes
     size_t  m_OutputSize;     //< The number of output bytes
-    bool    m_Busy;           //< Is true if compressor is ready to begin next session
+    bool    m_Busy;           //< TRUE if compressor is busy and not ready to begin next session
     // Friend classes
     friend class CCompressionStream;
     friend class CCompressionStreambuf;
@@ -509,6 +585,15 @@ public:
 //
 //===========================================================================
 
+inline
+size_t CCompression::SRecommendedBufferSizes::RoundUp(size_t value, size_t precision)
+{
+    if ( precision <= 1 ) {
+        return value;
+    }
+    return size_t(value / precision) * precision + ((value % precision > 0) ? precision : 0);
+}
+
 inline
 void CCompressionProcessor::Reset(void)
 {
@@ -558,6 +643,7 @@ size_t CCompressionProcessor::GetOutputSize(void)
 }
 
 
+
 END_NCBI_SCOPE
 
 
