@@ -1,7 +1,7 @@
 #ifndef UTIL_COMPRESS__STREAM_UTIL__HPP
 #define UTIL_COMPRESS__STREAM_UTIL__HPP
 
-/*  $Id: stream_util.hpp 438799 2014-06-20 13:06:09Z gotvyans $
+/*  $Id: stream_util.hpp 656213 2022-09-16 15:37:21Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -44,12 +44,14 @@
 ///     MCompress_Zip,      MDecompress_Zip
 ///     MCompress_GZipFile, MDecompress_GZipFile,
 ///                         MDecompress_ConcatenatedGZipFile
+///     MCompress_Zstd,     MDecompress_Zstd
 
 
 #include <util/compress/stream.hpp>
 #include <util/compress/bzip2.hpp>
-#include <util/compress/zlib.hpp>
 #include <util/compress/lzo.hpp>
+#include <util/compress/zlib.hpp>
+#include <util/compress/zstd.hpp>
 
 
 /** @addtogroup CompressionStreams
@@ -69,13 +71,13 @@ BEGIN_NCBI_SCOPE
 class NCBI_XUTIL_EXPORT CCompressStream
 {
 public:
-    /// Compression/decompression method.
+    /// Compression/decompression methods.
     /// @note
-    ///   Don't mix up "eNone" with compression level. In this mode data
-    ///   will be copied "as is", without parsing or writing any header
-    ///   or footer. On the other hand, CCompress::eLevel_NoCompression
-    ///   use uncompressed data, but reading/writing compression format
-    ///   is always correspond to EMethod.
+    ///   Don't mix up "eNone" with CCompress::eLevel_NoCompression 
+    ///   compression level. In "eNone" mode data will be copied "as is", 
+    ///   without parsing or writing any header or footer. 
+    ///   CCompress::eLevel_NoCompression use no-compression for a choosen 
+    ///   compression format and can have extra data necessary to be valid,
     /// @note
     ///   You can use "eNone" to allow transparent reading/writing data
     ///   from/to an underlying stream using compression streams, but we
@@ -83,14 +85,32 @@ public:
     ///   to the original input/output stream if you know in advance that
     ///   data is uncompressed. 
     enum EMethod {
-        eNone,                ///< no compression method (copy "as is")
-        eBZip2,               ///< BZIP2
-        eLZO,                 ///< LZO (LZO1X)
-        eZip,                 ///< ZLIB (raw zip data / DEFLATE method)
-        eGZipFile,            ///< .gz file (including concatenated files)
-        eConcatenatedGZipFile ///< Synonym for eGZipFile (for backward compatibility)
+        eNone,                 ///< no compression method (copy "as is")
+        eBZip2,                ///< BZIP2
+        eLZO,                  ///< LZO (LZO1X)
+        eZip,                  ///< ZLIB (raw zip data / DEFLATE method)
+        eGZipFile,             ///< .gz file (including concatenated files)
+        eConcatenatedGZipFile, ///< Synonym for eGZipFile (for backward compatibility)
+        eZstd                  ///< ZStandard (raw zstd data)
     };
 
+    /// Check if specified compression method is supported on a current platform.
+    ///  
+    /// Compression streams can be created for any method. It ignores the fact
+    /// that some compression library can be missed on a current platform, 
+    /// that lead to a throwing an exception on usage of the newly created stream.
+    /// It is recommended to guard library-specific code by HAVE_LIB* macros
+    /// (see comments at the beginning of "compress.hpp"), at least for eLZO and eZstd. 
+    /// But if this is inconvenient for some reason you can use this method 
+    /// to do a check at runtime.
+    ///  
+    /// @param method
+    ///   Method to check.
+    /// @return
+    ///   Return TRUE if selected compression method is supported, FALSE otherwise.
+    /// 
+    static bool HaveSupport(EMethod method);
+
     /// Default algorithm-specific compression/decompression flags.
     /// @sa TFlags, EMethod
     enum EDefaultFlags {
@@ -126,9 +146,14 @@ public:
     ///   but they can be overridden by using this parameter.
     /// @param level
     ///   Compression level.
+    /// @param own_istream
+    ///   If set to eTakeOwnership then the 'stream' will be owned by
+    ///   CCompressIStream and automatically deleted when necessary.
+    ///
     CCompressIStream(CNcbiIstream& stream, EMethod method, 
                      ICompression::TFlags flags = fDefault,
-                     ICompression::ELevel level = ICompression::eLevel_Default);
+                     ICompression::ELevel level = ICompression::eLevel_Default,
+                     ENcbiOwnership own_istream = eNoOwnership);
 };
 
 
@@ -162,6 +187,10 @@ public:
     ///   but they can be overridden by using this parameter.
     /// @param level
     ///   Compression level.
+    /// @param own_ostream
+    ///   If set to eTakeOwnership then the 'stream' will be owned by
+    ///   CCompressOStream and automatically deleted when necessary.
+    ///
     CCompressOStream(CNcbiOstream& stream, EMethod method, 
                      ICompression::TFlags flags = fDefault,
                      ICompression::ELevel level = ICompression::eLevel_Default,
@@ -194,9 +223,13 @@ public:
     /// @param flags
     ///   By default, predefined algorithm-specific flags will be used,
     ///   but they can be overridden by using this parameter.
+    /// @param own_istream
+    ///   If set to eTakeOwnership then the 'stream' will be owned by
+    ///   CDecompressIStream and automatically deleted when necessary.
+    ///
     CDecompressIStream(CNcbiIstream& stream, EMethod method, 
                        ICompression::TFlags flags = fDefault,
-                       ENcbiOwnership own_instream = eNoOwnership);
+                       ENcbiOwnership own_istream = eNoOwnership);
 };
 
 
@@ -228,18 +261,22 @@ public:
     /// @param flags
     ///   By default, predefined algorithm-specific flags will be used,
     ///   but they can be overridden by using this parameter.
+    /// @param own_istream
+    ///   If set to eTakeOwnership then the 'stream' will be owned by
+    ///   CDecompressOStream and automatically deleted when necessary.
+    ///
     CDecompressOStream(CNcbiOstream& stream, EMethod method, 
-                       ICompression::TFlags flags = fDefault);
+                       ICompression::TFlags flags = fDefault,
+                       ENcbiOwnership own_ostream = eNoOwnership);
 };
 
 
 
-
 /// Auxiliary function to get manipulator error
 template <class T>  
 string g_GetManipulatorError(T& stream)
 {
-    int    status; 
+    int status; 
     string description;
     if (stream.GetError(status, description)) {
         return description + " (errcode = " + NStr::IntToString(status) + ")";
@@ -439,11 +476,13 @@ class MCompress_Proxy_BZip2      {};
 class MCompress_Proxy_LZO        {};
 class MCompress_Proxy_Zip        {};
 class MCompress_Proxy_GZipFile   {};
+class MCompress_Proxy_Zstd       {};
 class MDecompress_Proxy_BZip2    {};
 class MDecompress_Proxy_LZO      {};
 class MDecompress_Proxy_Zip      {};
 class MDecompress_Proxy_GZipFile {};
 class MDecompress_Proxy_ConcatenatedGZipFile {};
+class MDecompress_Proxy_Zstd     {};
 
 
 /// Manipulator definitions.
@@ -452,11 +491,13 @@ class MDecompress_Proxy_ConcatenatedGZipFile {};
 #define  MCompress_LZO                     MCompress_Proxy_LZO()
 #define  MCompress_Zip                     MCompress_Proxy_Zip()
 #define  MCompress_GZipFile                MCompress_Proxy_GZipFile()
+#define  MCompress_Zstd                    MCompress_Proxy_Zstd()
 #define  MDecompress_BZip2                 MDecompress_Proxy_BZip2()
 #define  MDecompress_LZO                   MDecompress_Proxy_LZO()
 #define  MDecompress_Zip                   MDecompress_Proxy_Zip()
 #define  MDecompress_GZipFile              MDecompress_Proxy_GZipFile()
 #define  MDecompress_ConcatenatedGZipFile  MDecompress_Proxy_ConcatenatedGZipFile()
+#define  MDecompress_Zstd                  MDecompress_Proxy_Zstd()
 
 
 // When you pass an object of type M[Dec|C]ompress_Proxy_* to an
@@ -465,6 +506,8 @@ class MDecompress_Proxy_ConcatenatedGZipFile {};
 // >> and <<. This will process the next object and then return 
 // the stream to continue processing as normal.
 
+// --- compression ---
+
 inline
 TCompressOProxy operator<<(ostream& os, MCompress_Proxy_BZip2 const& /*obj*/)
 {
@@ -513,6 +556,20 @@ TCompressIProxy operator>>(istream& is, MCompress_Proxy_GZipFile const& /*obj*/)
     return TCompressIProxy(is, CCompressStream::eGZipFile);
 }
 
+inline
+TCompressOProxy operator<<(ostream& os, MCompress_Proxy_Zstd const& /*obj*/)
+{
+    return TCompressOProxy(os, CCompressStream::eZstd);
+}
+
+inline
+TCompressIProxy operator>>(istream& is, MCompress_Proxy_Zstd const& /*obj*/)
+{
+    return TCompressIProxy(is, CCompressStream::eZstd);
+}
+
+// --- decompression ---
+
 inline
 TDecompressOProxy operator<<(ostream& os, MDecompress_Proxy_BZip2 const& /*obj*/)
 {
@@ -573,6 +630,18 @@ TDecompressIProxy operator>>(istream& is, MDecompress_Proxy_ConcatenatedGZipFile
     return TDecompressIProxy(is, CCompressStream::eConcatenatedGZipFile);
 }
 
+inline
+TDecompressOProxy operator<<(ostream& os, MDecompress_Proxy_Zstd const& /*obj*/)
+{
+    return TDecompressOProxy(os, CCompressStream::eZstd);
+}
+
+inline
+TDecompressIProxy operator>>(istream& is, MDecompress_Proxy_Zstd const& /*obj*/)
+{
+    return TDecompressIProxy(is, CCompressStream::eZstd);
+}
+
 
 /* @} */
 
