@@ -1,4 +1,4 @@
-/*  $Id: seq_masker.cpp 621823 2020-12-17 12:53:02Z grichenk $
+/*  $Id: seq_masker.cpp 657016 2022-10-03 12:18:27Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -87,7 +87,11 @@ CSeqMasker::CSeqMasker( const string & lstat_name,
                         Uint1 tmin_count,
                         bool arg_discontig,
                         Uint4 arg_pattern,
-                        bool arg_use_ba )
+                        bool arg_use_ba,
+                        double min_pct,
+                        double extend_pct,
+                        double thres_pct,
+                        double max_pct )
     : ustat( CSeqMaskerIstatFactory::create( lstat_name,
                                              arg_cutoff_score,
                                              arg_textend,
@@ -95,7 +99,11 @@ CSeqMasker::CSeqMasker( const string & lstat_name,
                                              arg_set_max_score,
                                              arg_min_score,
                                              arg_set_min_score,
-                                             arg_use_ba ) ),
+                                             arg_use_ba,
+                                             min_pct,
+                                             extend_pct,
+                                             thres_pct,
+                                             max_pct ) ),
       score( NULL ), score_p3( NULL ), trigger_score( NULL ),
       window_size( arg_window_size ), window_step( arg_window_step ),
       unit_step( arg_unit_step ),
