@@ -1,4 +1,4 @@
-/*  $Id: ncbicgi.cpp 638779 2021-10-06 11:13:00Z grichenk $
+/*  $Id: ncbicgi.cpp 648660 2022-04-19 17:37:17Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -42,7 +42,6 @@
 #include <corelib/ncbi_strings.h>
 
 #include <cgi/cgi_exception.hpp>
-#include <cgi/ncbicgi.hpp>
 #include <cgi/cgi_serial.hpp>
 #include <cgi/cgi_session.hpp>
 #include <cgi/error_codes.hpp>
@@ -50,6 +49,8 @@
 #include <util/checksum.hpp>
 #include <util/ncbi_url.hpp>
 
+#include "cgi_impl.hpp"
+
 #include <algorithm>
 
 #include <stdio.h>
@@ -962,30 +963,12 @@ static void s_AddEntry(TCgiEntries& entries, const string& name,
 }
 
 
-class CCgiEntries_Parser : public CUrlArgs_Parser
-{
-public:
-    CCgiEntries_Parser(TCgiEntries* entries,
-                       TCgiIndexes* indexes,
-                       bool indexes_as_entries);
-protected:
-    virtual void AddArgument(unsigned int position,
-                             const string& name,
-                             const string& value,
-                             EArgType arg_type);
-private:
-    TCgiEntries* m_Entries;
-    TCgiIndexes* m_Indexes;
-    bool         m_IndexesAsEntries;
-};
-
-
 CCgiEntries_Parser::CCgiEntries_Parser(TCgiEntries* entries,
                                        TCgiIndexes* indexes,
-                                       bool indexes_as_entries)
-    : m_Entries(entries),
-      m_Indexes(indexes),
-      m_IndexesAsEntries(indexes_as_entries  ||  !indexes)
+                                       CCgiRequest::TFlags flags)
+    : CUrlArgs_Parser(CCgiRequestTFlagsToTFlags(flags & ~(indexes ? 0 : CCgiRequest::fIndexesNotEntries))),
+      m_Entries(entries),
+      m_Indexes(indexes)
 {
     return;
 }
@@ -997,7 +980,7 @@ void CCgiEntries_Parser::AddArgument(unsigned int position,
                                      EArgType arg_type)
 {
     if (m_Entries  &&
-        (arg_type == eArg_Value  ||  m_IndexesAsEntries)) {
+        (arg_type == eArg_Value  ||  !(TFlagsToCCgiRequestTFlags(m_Flags) & CCgiRequest::fIndexesNotEntries))) {
         m_Entries->insert(TCgiEntries::value_type(
             name, CCgiEntry(value, kEmptyStr, position, kEmptyStr)));
     }
@@ -1327,11 +1310,7 @@ void CCgiRequest::x_ProcessQueryString(TFlags flags, const CNcbiArguments* args)
         }
 
         if ( query_string ) {
-            CCgiEntries_Parser parser(&m_Entries, &m_Indexes,
-                (flags & fIndexesNotEntries) == 0);
-            if (flags & fSemicolonIsNotArgDelimiter) {
-                parser.SetSemicolonIsNotArgDelimiter(true);
-            }
+            CCgiEntries_Parser parser(&m_Entries, &m_Indexes, flags);
             parser.SetQueryString(*query_string);
         }
     }
@@ -1538,7 +1517,7 @@ void CCgiRequest::SetInputStream(CNcbiIstream* is, bool own, int fd)
 
 SIZE_TYPE CCgiRequest::ParseEntries(const string& str, TCgiEntries& entries)
 {
-    CCgiEntries_Parser parser(&entries, 0, true);
+    CCgiEntries_Parser parser(&entries, 0, 0);
     try {
         parser.SetQueryString(str);
     }
@@ -1551,7 +1530,7 @@ SIZE_TYPE CCgiRequest::ParseEntries(const string& str, TCgiEntries& entries)
 
 SIZE_TYPE CCgiRequest::ParseIndexes(const string& str, TCgiIndexes& indexes)
 {
-    CCgiEntries_Parser parser(0, &indexes, false);
+    CCgiEntries_Parser parser(0, &indexes, fIndexesNotEntries);
     try {
         parser.SetQueryString(str);
     }
