@@ -1,4 +1,4 @@
-/* $Id: ncbi_connection.c 637768 2021-09-17 16:39:06Z lavr $
+/* $Id: ncbi_connection.c 652083 2022-06-30 15:04:46Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -49,35 +49,35 @@
 
 /* Standard logging message
  */
-#define CONN_LOG_EX(subcode, func_name, level, message, status)            \
-  do {                                                                     \
-      const char* ststr = ((EIO_Status) status != eIO_Success              \
-                           ? IO_StatusStr((EIO_Status) status)             \
-                           : "");                                          \
-      const char* ctype = (conn  &&  conn->meta.get_type                   \
-                           ? conn->meta.get_type(conn->meta.c_get_type)    \
-                           : 0);                                           \
-      char* descr = (conn  &&  conn->meta.descr                            \
-                     ? conn->meta.descr(conn->meta.c_descr)                \
-                     : 0);                                                 \
-      char stbuf[80];                                                      \
-      if ((EIO_Status) status == eIO_Timeout  &&  timeout) {               \
-          assert(timeout != kDefaultTimeout);                              \
-          sprintf(stbuf, "%s[%u.%06u]", ststr,                             \
-                  (unsigned int)(timeout->sec + timeout->usec / 1000000),  \
-                  (unsigned int)               (timeout->usec % 1000000)); \
-          assert(strlen(stbuf) < sizeof(stbuf));                           \
-          ststr = stbuf;                                                   \
-      }                                                                    \
-      CORE_LOGF_X(subcode, level,                                          \
-                  ("[CONN_" #func_name "(%s%s%s)]  %s%s%s",                \
-                   ctype  &&  *ctype ? ctype : "UNDEF",                    \
-                   descr  &&  *descr ? "; "  : "", descr ? descr : "",     \
-                   message,                                                \
-                   ststr  &&  *ststr ? ": "  : "",                         \
-                   ststr             ? ststr : ""));                       \
-      if (descr)                                                           \
-          free(descr);                                                     \
+#define CONN_LOG_EX(subcode, func_name, level, message, status)         \
+  do {                                                                  \
+      const char* ststr = ((EIO_Status) status != eIO_Success           \
+                           ? IO_StatusStr((EIO_Status) status)          \
+                           : "");                                       \
+      const char* ctype = (conn  &&  conn->meta.get_type                \
+                           ? conn->meta.get_type(conn->meta.c_get_type) \
+                           : 0);                                        \
+      char* descr = (conn  &&  conn->meta.descr                         \
+                     ? conn->meta.descr(conn->meta.c_descr)             \
+                     : 0);                                              \
+      char stbuf[80];                                                   \
+      if ((EIO_Status) status == eIO_Timeout  &&  timeout) {            \
+          assert(timeout != kDefaultTimeout);                           \
+          sprintf(stbuf, "%s[%u.%06u]", ststr,                          \
+                  timeout->usec / 1000000 + timeout->sec,               \
+                  timeout->usec % 1000000);                             \
+          assert(strlen(stbuf) < sizeof(stbuf));                        \
+          ststr = stbuf;                                                \
+      }                                                                 \
+      CORE_LOGF_X(subcode, level,                                       \
+                  ("[CONN_" #func_name "(%s%s%s)]  %s%s%s",             \
+                   ctype  &&  *ctype ? ctype : "UNDEF",                 \
+                   descr  &&  *descr ? "; "  : "", descr ? descr : "",  \
+                   message,                                             \
+                   ststr  &&  *ststr ? ": "  : "",                      \
+                   ststr             ? ststr : ""));                    \
+      if (descr)                                                        \
+          free(descr);                                                  \
   } while (0)
 
 #define CONN_LOG(s_c, f_n, lvl, msg)  CONN_LOG_EX(s_c, f_n, lvl, msg, status)
@@ -162,11 +162,12 @@ typedef struct SConnectionTag {
     void*           data;        /* user data pointer                        */
 
     /* "[o|r|w|c]_timeout" is either 0 (kInfiniteTimeout), kDefaultTimeout
-       (to use connector-specific one), or points to "[oo|rr|ww|cc]_timeout" */
-    const STimeout* o_timeout;   /* timeout on open                          */
-    const STimeout* r_timeout;   /* timeout on read                          */
-    const STimeout* w_timeout;   /* timeout on write                         */
-    const STimeout* c_timeout;   /* timeout on close                         */
+       (to use connector-specific one), or points to "[oo|rr|ww|cc]_timeout";
+       timeouts are "as is" (may be not normalized) for performance reasons  */
+    const STimeout*  o_timeout;  /* timeout on open                          */
+    const STimeout*  r_timeout;  /* timeout on read                          */
+    const STimeout*  w_timeout;  /* timeout on write                         */
+    const STimeout*  c_timeout;  /* timeout on close                         */
     STimeout        oo_timeout;  /* storage for "o_timeout"                  */
     STimeout        rr_timeout;  /* storage for "r_timeout"                  */
     STimeout        ww_timeout;  /* storage for "w_timeout"                  */
