@@ -1,4 +1,4 @@
-/*  $Id: ncbi_http2_session.cpp 632077 2021-05-26 15:09:40Z sadyrovr $
+/*  $Id: ncbi_http2_session.cpp 651828 2022-06-25 18:46:23Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -659,7 +659,7 @@ void CHttpSessionImpl2::StartRequest(CHttpSession_Base::EProtocol protocol, CHtt
     unique_ptr<IReaderWriter> rw(new SH2S_ReaderWriter(move(update_response), move(response_queue), move(request)));
     auto stream = make_shared<CRWStream>(rw.release(), 0, nullptr, CRWStreambuf::fOwnAll);
 
-    req.x_InitConnection2(move(stream), /* TODO: */ false);
+    req.x_InitConnection2(move(stream));
 }
 
 bool CHttpSessionImpl2::Downgrade(CHttpResponse& resp, CHttpSession_Base::EProtocol& protocol)
