@@ -1,4 +1,4 @@
-/* $Id: ncbi_namerd.c 641912 2021-12-09 12:14:45Z fukanchi $
+/* $Id: ncbi_namerd.c 641994 2021-12-10 18:42:28Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -322,7 +322,7 @@ static TNCBI_Time x_ParseExpires(const char* expires, time_t utc,
         CORE_LOGF_X(eNSub_Json, eLOG_Error,
                     ("[%s]  Unable to get JSON {\"addrs[" FMT_SIZE_T
                      "].meta.expires\"} value", name, i));
-        return 0;
+        return 0/*failure*/;
     }
 
     memset(&tm_exp, 0, sizeof(tm_exp));
@@ -976,7 +976,7 @@ static int/*bool*/ x_UpdateDtabInArgs(SConnNetInfo* net_info,
     assert(dtablen);
 
     /* Find any existing dtab in the args */
-    verify(arg = ConnNetInfo_GetArgs(net_info));
+    verify((arg = ConnNetInfo_GetArgs(net_info)));
     for ( ;  *arg  &&  *arg != '#';  arg += arglen + !(arg[arglen] != '&')) {
         arglen = strcspn(arg, "&#");
         if (arglen < sizeof(NAMERD_DTAB_ARG))
