@@ -1,4 +1,4 @@
-/* $Id: ncbi_pipe.cpp 650112 2022-05-17 12:21:35Z ivanov $
+/* $Id: ncbi_pipe.cpp 652319 2022-07-06 17:21:14Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,12 +37,12 @@
 #endif /*_FORTIFY_SOURCE*/
 #define  _FORTIFY_SOURCE 0
 #include "ncbi_priv.h"
-#include <connect/error_codes.hpp>
 #include <corelib/ncbiexec.hpp>
 #include <corelib/ncbi_param.hpp>
-#include <connect/ncbi_pipe.hpp>
 #include <corelib/ncbi_system.hpp>
 #include <corelib/stream_utils.hpp>
+#include <connect/error_codes.hpp>
+#include <connect/ncbi_pipe.hpp>
 
 #ifdef NCBI_OS_MSWIN
 
@@ -59,9 +59,6 @@
 #  include <sys/types.h>
 #  include <sys/wait.h>
 
-#  define _STR(s)  #s
-#  define  STR(s)  _STR(s)
-
 #else
 #  error "The CPipe class is supported only on Windows and Unix"
 #endif
@@ -987,7 +984,8 @@ CPipeHandle::CPipeHandle(void)
     static NCBI_PARAM_TYPE(CONN, PIPE_USE_POLL) use_poll_param;
 
     m_UsePoll = use_poll_param.Get();
-    _TRACE("CPipeHandle using poll(): " + NStr::BoolToString(m_UsePoll));
+    ERR_POST_ONCE(Trace << "CPipeHandle using poll(): "
+                  + NStr::BoolToString(m_UsePoll));
 }
 
 
@@ -1685,15 +1683,15 @@ CPipe::TChildPollMask CPipeHandle::x_Poll(CPipe::TChildPollMask mask,
 
     if (m_UsePoll) {
         struct pollfd poll_fds[3] = {
-            { m_ChildStdIn,  POLLOUT | POLLERR },
-            { m_ChildStdOut, POLLIN,           },
-            { m_ChildStdErr, POLLIN,           }
+            { m_ChildStdIn,  POLLOUT },
+            { m_ChildStdOut, POLLIN  },
+            { m_ChildStdErr, POLLIN  }
         };
         int timeout_msec(timeout
                          ? timeout->sec * 1000 + (timeout->usec + 500) / 1000
                          : -1/*infinite*/);
 
-        // Negative FDs OK, poll should ignore them
+        // Negative FDs OK, poll ignores them
         // Check the mask
         if (!(mask & CPipe::fStdIn))
             poll_fds[0].fd = -1;
@@ -1768,10 +1766,8 @@ CPipe::TChildPollMask CPipeHandle::x_Poll(CPipe::TChildPollMask mask,
                 }
             }
             if ((mask & CPipe::fStdOut)  &&  m_ChildStdOut != -1) {
-                if (!rd) {
-                    rd = true;
-                    FD_ZERO(&rfds);
-                }
+                rd = true;
+                FD_ZERO(&rfds);
                 if (m_ChildStdOut < FD_SETSIZE) {
                     FD_SET(m_ChildStdOut, &rfds);
                     FD_SET(m_ChildStdOut, &efds);
@@ -1798,7 +1794,8 @@ CPipe::TChildPollMask CPipeHandle::x_Poll(CPipe::TChildPollMask mask,
             if (max >= FD_SETSIZE) {
                 PIPE_THROW(0,
                            "File descriptor " + NStr::NumericToString(max)
-                           + " too large (" + string(STR(FD_SETSIZE)) + ')');
+                           + " too large (maximum allowed "
+                           + string(NCBI_AS_STRING(FD_SETSIZE)) + ')');
             }
 
             int n = ::select(max + 1,
