@@ -1,4 +1,4 @@
-/* $Id: ncbi_service.c 638176 2021-09-24 15:44:10Z lavr $
+/* $Id: ncbi_service.c 656347 2022-09-19 19:17:00Z fukanchi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -94,6 +94,14 @@ static int/*bool*/ x_tr(char* str, char a, char b, size_t len)
 }
 
 
+/* "service" == the original input service name;
+ * "svc"     == current service name (== "service" at first);
+ * "ismask"  = 1 if "service" may contain any wildcards;
+ * "*isfast" = 1 on input if not to do environment scan;
+ * "*isfast" = 1 on output if the service was substituted with itself (but may
+ *               be different case);  otherwise, "*isfast" == 0.  This is used
+ *               (only) in namerd searches, which are case-sensitive.
+ */
 static char* x_ServiceName(unsigned int depth,
                            const char* service, const char* svc,
                            int/*bool*/ ismask, int* /*bool*/isfast)
@@ -103,6 +111,7 @@ static char* x_ServiceName(unsigned int depth,
 
     assert(isfast);
     assert(!svc == !service);
+    assert(depth  ||  svc == service);
     assert(sizeof(buf) > sizeof(REG_CONN_SERVICE_NAME));
     if (!svc  ||  (!ismask  &&  (!*svc  ||  strpbrk(svc, "?*[")))
         ||  (len = strlen(svc)) >= sizeof(buf)-sizeof(REG_CONN_SERVICE_NAME)
@@ -150,7 +159,7 @@ static char* x_ServiceName(unsigned int depth,
                     return x_ServiceName(depth, service, s, ismask, isfast);
                 CORE_LOGF_X(8, eLOG_Error,
                             ("[%s]  Maximal service name recursion"
-                             " depth reached: %u", service, depth));
+                             " depth exceeded: %u", service, depth));
                 return 0/*failure*/;
             } else
                 svc = s, *isfast = 1/*true*/;
@@ -539,6 +548,8 @@ static int/*bool*/ x_ConsistencyCheck(SERV_ITER iter, const SSERV_Info* info)
                        info, infostr ? infostr : "<NULL>"));
             RETURN(0/*failure*/);
         }
+        /* for the case of n/w delay */
+        iter->time = (TNCBI_Time) time(0);
         if (info->time > iter->time + LBSM_DEFAULT_TIME) {
             CORE_LOGF(eLOG_Critical,
                       ("[%s]  Excessive expiration (%u) @%p:\n%s", iter->name,
@@ -1060,31 +1071,7 @@ char* SERV_Print(SERV_ITER iter, const SConnNetInfo* net_info, int but_last)
         return 0;
     }
     if (iter) {
-        TSERV_TypeOnly t, types
-            = iter->types & (TSERV_TypeOnly)(~fSERV_Stateless);
-        /* Accepted server types */
-        buflen = sizeof(kAcceptedServerTypes) - 1;
-        memcpy(buffer, kAcceptedServerTypes, buflen);
-        for (t = 1;  t;  t = (TSERV_TypeOnly)(t << 1)) {
-            if (types & t) {
-                const char* name = SERV_TypeStr((ESERV_Type) t);
-                size_t namelen = strlen(name);
-                if (!namelen  ||  buflen + 1 + namelen + 2 >= sizeof(buffer))
-                    break;
-                buffer[buflen++] = ' ';
-                memcpy(buffer + buflen, name, namelen);
-                buflen += namelen;
-            } else if (types < t)
-                break;
-        }
-        if (buffer[buflen - 1] != ':') {
-            strcpy(&buffer[buflen], "\r\n");
-            assert(strlen(buffer) == buflen+2  &&  buflen+2 < sizeof(buffer));
-            if (!BUF_Write(&buf, buffer, buflen + 2)) {
-                BUF_Destroy(buf);
-                return 0;
-            }
-        }
+        TSERV_TypeOnly types;
         if (iter->external) {
             /* External */
             if (!BUF_Write(&buf, kNcbiExternal, sizeof(kNcbiExternal)-1)) {
@@ -1092,6 +1079,34 @@ char* SERV_Print(SERV_ITER iter, const SConnNetInfo* net_info, int but_last)
                 return 0;
             }
         }
+        if ((types = (iter->types & fSERV_All)) != fSERV_Any) {
+            /* Accepted server types */
+            TSERV_TypeOnly t;
+            buflen = 0;
+            for (t = 1;  t;  t <<= 1) {
+                if (types & t) {
+                    const char* name = SERV_TypeStr((ESERV_Type) t);
+                    size_t namelen = strlen(name);
+                    if (!namelen)
+                        continue;
+                    if (buflen + namelen + (1 + 2) > sizeof(buffer))
+                        break;
+                    buffer[buflen++] = ' ';
+                    memcpy(buffer + buflen, name, namelen);
+                    buflen += namelen;
+                } else if (types < t)
+                    break;
+            }
+            if (buflen) {
+                memcpy(buffer + buflen, "\r\n", 2);
+                if (!BUF_Write(&buf, kAcceptedServerTypes,
+                               sizeof(kAcceptedServerTypes) - 1)
+                    ||  !BUF_Write(&buf, buffer, buflen + 2)) {
+                    BUF_Destroy(buf);
+                    return 0;
+                }
+            }
+        }
         if (types & fSERV_Firewall) {
             /* Firewall */
             EFWMode mode
@@ -1112,7 +1127,7 @@ char* SERV_Print(SERV_ITER iter, const SConnNetInfo* net_info, int but_last)
             buffer[buflen++] = ' ';
             buflen = (size_t)(strcpy(NCBI_simple_ftoa(buffer + buflen,
                                                       iter->pref * 100.0, 2),
-                                     "\r\n") - buffer) + 2/*"\r\n"*/;
+                                     "%\r\n") - buffer) + 3/*"%\r\n"*/;
             if (!BUF_Write(&buf, kPreference, sizeof(kPreference) - 1)  ||
                 !BUF_Write(&buf, buffer, buflen)) {
                 BUF_Destroy(buf);
