@@ -1,4 +1,4 @@
-/* $Id: ncbi_socket_cxx.cpp 626137 2021-02-24 02:58:16Z lavr $
+/* $Id: ncbi_socket_cxx.cpp 652321 2022-07-06 17:22:27Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -357,7 +357,7 @@ EIO_Status CSocket::Read(void*          buf,
 
 EIO_Status CSocket::ReadLine(string& str)
 {
-    str.erase();
+    str.clear();
     if ( !m_Socket )
         return eIO_Closed;
     EIO_Status status;
@@ -365,7 +365,7 @@ EIO_Status CSocket::ReadLine(string& str)
     size_t size;
     do {
         status = SOCK_ReadLine(m_Socket, buf, sizeof(buf), &size);
-        if (!size)
+        if ( !size )
             break;
         str.append(buf, size);
     } while (status == eIO_Success  &&  size == sizeof(buf));
@@ -608,9 +608,8 @@ EIO_Status CSocketAPI::Poll(vector<SPoll>&  polls,
                 CListeningSocket* ls = dynamic_cast<CListeningSocket*>(p);
                 if (!ls) {
                     CTrigger* tr = dynamic_cast<CTrigger*>(p);
-                    x_polls[i].poll = POLLABLE_FromTRIGGER(tr
-                                                           ? tr->GetTRIGGER()
-                                                           : 0);
+                    x_polls[i].poll
+                        = tr ? POLLABLE_FromTRIGGER(tr->GetTRIGGER()) : 0;
                 } else
                     x_polls[i].poll = POLLABLE_FromLSOCK(ls->GetLSOCK());
                 polls[i].m_REvent = eIO_Open;
@@ -642,7 +641,7 @@ EIO_Status CSocketAPI::Poll(vector<SPoll>&  polls,
             polls[i].m_REvent = x_polls[i].revent;
     }
 
-    if (n_ready)
+    if ( n_ready )
         *n_ready = xx_ready + x_ready;
 
     delete[] x_polls;
