@@ -1,4 +1,4 @@
-/*  $Id: ncbi_uv_nghttp2.cpp 632212 2021-05-27 15:35:27Z sadyrovr $
+/*  $Id: ncbi_uv_nghttp2.cpp 653927 2022-08-05 15:20:25Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -202,24 +202,16 @@ SUv_Tcp::SUv_Tcp(uv_loop_t *l, const SSocketAddress& address, size_t rd_buf_size
 
 int SUv_Tcp::Write()
 {
+    if (m_State == eClosing) {
+        m_State = eRestarting;
+    }
+
     if (m_State == eClosed) {
-        auto rv = uv_tcp_init(m_Loop, this);
+        auto rv = Connect();
 
         if (rv < 0) {
-            NCBI_UV_TCP_TRACE(this << " init failed: " << SUvNgHttp2_Error::LibuvStr(rv));
             return rv;
         }
-
-        rv = m_Connect(this, s_OnConnect);
-
-        if (rv < 0) {
-            NCBI_UV_TCP_TRACE(this << " pre-connect failed: " << SUvNgHttp2_Error::LibuvStr(rv));
-            Close();
-            return rv;
-        }
-
-        NCBI_UV_TCP_TRACE(this << " connecting");
-        m_State = eConnecting;
     }
 
     if (m_State == eConnected) {
@@ -237,7 +229,33 @@ int SUv_Tcp::Write()
     return 0;
 }
 
-void SUv_Tcp::Close()
+// uv_tcp_close_reset was added in libuv v1.32.0
+#if UV_VERSION_HEX < 0x12000
+bool SUv_Tcp::CloseReset(ECloseType)
+{
+    return false;
+}
+#else
+bool SUv_Tcp::CloseReset(ECloseType close_type)
+{
+    if (close_type == eNormalClose) {
+        return false;
+    }
+
+    auto rv = uv_tcp_close_reset(this, s_OnClose);
+
+    if (rv < 0) {
+        NCBI_UV_TCP_TRACE(this << " close reset failed: " << SUvNgHttp2_Error::LibuvStr(rv));
+        m_State = eClosed;
+    } else {
+        NCBI_UV_TCP_TRACE(this << " close resetting");
+    }
+
+    return true;
+}
+#endif
+
+void SUv_Tcp::Close(ECloseType close_type)
 {
     if (m_State == eConnected) {
         auto rv = uv_read_stop(reinterpret_cast<uv_stream_t*>(this));
@@ -251,15 +269,45 @@ void SUv_Tcp::Close()
 
     m_Write.Reset();
 
-    if ((m_State != eClosing) && (m_State != eClosed)) {
-        NCBI_UV_TCP_TRACE(this << " closing");
-        m_State = eClosing;
-        SUv_Handle<uv_tcp_t>::Close();
+    if ((m_State == eClosing) || (m_State == eRestarting)) {
+        NCBI_UV_TCP_TRACE(this << " already closing");
+
+    } else if (m_State == eClosed) {
+        NCBI_UV_TCP_TRACE(this << " already closed");
+
     } else {
-        NCBI_UV_TCP_TRACE(this << " already closing/closed");
+        m_State = eClosing;
+
+        if (!CloseReset(close_type)) {
+            SUv_Handle<uv_tcp_t>::Close();
+            NCBI_UV_TCP_TRACE(this << " closing");
+        }
     }
 }
 
+int SUv_Tcp::Connect()
+{
+    auto rv = uv_tcp_init(m_Loop, this);
+
+    if (rv < 0) {
+        NCBI_UV_TCP_TRACE(this << " init failed: " << SUvNgHttp2_Error::LibuvStr(rv));
+        return rv;
+    }
+
+    rv = m_Connect(this, s_OnConnect);
+
+    if (rv < 0) {
+        NCBI_UV_TCP_TRACE(this << " pre-connect failed: " << SUvNgHttp2_Error::LibuvStr(rv));
+        Close();
+        return rv;
+    }
+
+    NCBI_UV_TCP_TRACE(this << " connecting");
+    m_State = eConnecting;
+
+    return 0;
+}
+
 void SUv_Tcp::OnConnect(uv_connect_t*, int status)
 {
     if (status >= 0) {
@@ -289,6 +337,7 @@ void SUv_Tcp::OnConnect(uv_connect_t*, int status)
 
 void SUv_Tcp::OnAlloc(uv_handle_t*, size_t suggested_size, uv_buf_t* buf)
 {
+    NCBI_UV_TCP_TRACE(this << " alloc: " << suggested_size);
     m_ReadBuffer.resize(suggested_size);
     buf->base = m_ReadBuffer.data();
     buf->len = static_cast<decltype(buf->len)>(m_ReadBuffer.size());
@@ -322,7 +371,10 @@ void SUv_Tcp::OnWrite(uv_write_t* req, int status)
 void SUv_Tcp::OnClose(uv_handle_t*)
 {
     NCBI_UV_TCP_TRACE(this << " closed");
-    m_State = eClosed;
+
+    if (exchange(m_State, eClosed) == eRestarting) {
+        Connect();
+    }
 }
 
 struct SUvNgHttp2_UserAgentImpl : string
@@ -375,7 +427,7 @@ SNgHttp2_Session::SNgHttp2_Session(void* user_data, uint32_t max_streams,
         nghttp2_on_data_chunk_recv_callback on_data,
         nghttp2_on_stream_close_callback    on_stream_close,
         nghttp2_on_header_callback          on_header,
-        nghttp2_error_callback              on_error,
+        nghttp2_error_callback2             on_error,
         nghttp2_on_frame_recv_callback      on_frame_recv) :
     m_UserData(user_data),
     m_OnData(on_data),
@@ -398,7 +450,7 @@ int SNgHttp2_Session::Init()
     nghttp2_session_callbacks_set_on_data_chunk_recv_callback(callbacks, m_OnData);
     nghttp2_session_callbacks_set_on_stream_close_callback(   callbacks, m_OnStreamClose);
     nghttp2_session_callbacks_set_on_header_callback(         callbacks, m_OnHeader);
-    nghttp2_session_callbacks_set_error_callback(             callbacks, m_OnError);
+    nghttp2_session_callbacks_set_error_callback2(            callbacks, m_OnError);
     if (m_OnFrameRecv) nghttp2_session_callbacks_set_on_frame_recv_callback(callbacks, m_OnFrameRecv);
 
     nghttp2_session_client_new(&m_Session, callbacks, m_UserData);
@@ -414,9 +466,9 @@ int SNgHttp2_Session::Init()
         return x_DelOnError(rv);
     }
 
-    NCBI_NGHTTP2_SESSION_TRACE(this << " initialized");
     auto max_streams = nghttp2_session_get_remote_settings(m_Session, NGHTTP2_SETTINGS_MAX_CONCURRENT_STREAMS);
     m_MaxStreams.first = min(max_streams, m_MaxStreams.second);
+    NCBI_NGHTTP2_SESSION_TRACE(this << " initialized: " << m_MaxStreams.first);
     return 0;
 }
 
@@ -858,7 +910,7 @@ bool SUvNgHttp2_SessionBase::Send()
         Reset(SUvNgHttp2_Error::FromNgHttp2(send_rv, "on send"));
 
     } else if (send_rv == SNgHttp2_Session::eWantsClose) {
-        Reset("nghttp2 asked to drop connection");
+        Reset("nghttp2 asked to drop connection", SUv_Tcp::eNormalClose);
 
     } else {
         auto tls_rv = m_Tls->Write();
@@ -877,6 +929,12 @@ bool SUvNgHttp2_SessionBase::Send()
     return false;
 }
 
+int SUvNgHttp2_SessionBase::OnError(nghttp2_session*, int lib_error_code, const char* msg, size_t len)
+{
+    NCBI_UVNGHTTP2_SESSION_TRACE(this << " error: " << SUvNgHttp2_Error::NgHttp2Str(lib_error_code) << ". " << CTempString(msg, len));
+    return 0;
+}
+
 void SUvNgHttp2_SessionBase::OnConnect(int status)
 {
     NCBI_UVNGHTTP2_SESSION_TRACE(this << " connected: " << status);
@@ -912,7 +970,7 @@ void SUvNgHttp2_SessionBase::OnRead(const char* buf, ssize_t nread)
         if (read_rv == 0) {
             m_Session.Del();
             m_Tls->Close();
-            m_Tcp.Close();
+            m_Tcp.Close(SUv_Tcp::eNormalClose);
 
         } else if (s_WantReadOrWrite(read_rv)) {
             if (nread == 0) break;
@@ -942,12 +1000,12 @@ void SUvNgHttp2_SessionBase::OnRead(const char* buf, ssize_t nread)
     Send();
 }
 
-void SUvNgHttp2_SessionBase::Reset(SUvNgHttp2_Error error)
+void SUvNgHttp2_SessionBase::Reset(SUvNgHttp2_Error error, SUv_Tcp::ECloseType close_type)
 {
     NCBI_UVNGHTTP2_SESSION_TRACE(this << " resetting with " << error);
     m_Session.Del();
     m_Tls->Close();
-    m_Tcp.Close();
+    m_Tcp.Close(close_type);
     OnReset(move(error));
 }
 
