@@ -1,4 +1,4 @@
-/*  $Id: netservice_api.cpp 637852 2021-09-20 17:06:16Z sadyrovr $
+/*  $Id: netservice_api.cpp 650424 2022-05-24 15:26:21Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -182,7 +182,7 @@ bool SNetServiceIterator_OmitPenalized::Next()
 }
 
 DEFINE_STATIC_FAST_MUTEX(s_RndLock);
-static CRandom s_RandomIteratorGen((CRandom::TValue) time(NULL));
+static struct R : CRandom { R() { Randomize(); } } s_RandomIteratorGen;
 
 static CRandom::TValue
 s_GetRand(CRandom::TValue max_value)
