@@ -1,4 +1,4 @@
-/*  $Id: ncbi_url.cpp 620642 2020-11-25 17:54:32Z lavr $
+/*  $Id: ncbi_url.cpp 648663 2022-04-19 17:56:47Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,6 +37,10 @@
 
 BEGIN_NCBI_SCOPE
 
+NCBI_PARAM_DECL(bool, CUrl, enable_parsing_as_index);
+NCBI_PARAM_DEF(bool, CUrl, enable_parsing_as_index, false);
+typedef NCBI_PARAM_TYPE(CUrl, enable_parsing_as_index) TCUrlEnableParsingAsIndex;
+
 //////////////////////////////////////////////////////////////////////////////
 //
 // CUrlArgs_Parser
@@ -105,7 +109,7 @@ void CUrlArgs_Parser::SetQueryString(const string& query,
 
     // If no '=' present in the parsed string then try to parse it as ISINDEX
     // RFC3875
-    if (query.find("=") == NPOS) {
+    if (((m_Flags & fEnableParsingAsIndex) || TCUrlEnableParsingAsIndex::GetDefault()) && (query.find("=") == NPOS)) {
         x_SetIndexString(query, *encoder);
         return;
     }
@@ -113,7 +117,7 @@ void CUrlArgs_Parser::SetQueryString(const string& query,
     // Parse into entries
     string mid_seps = "=&";
     string end_seps = "&";
-    if (!m_SemicolonIsNotArgDelimiter)
+    if (m_Flags & fSemicolonIsArgDelimiter)
     {
         mid_seps += ';';
         end_seps += ';';
@@ -129,7 +133,7 @@ void CUrlArgs_Parser::SetQueryString(const string& query,
             continue;
         }
         // Alternative separator - ';'
-        else if (!m_SemicolonIsNotArgDelimiter  &&  query[beg] == ';')
+        else if ((m_Flags & fSemicolonIsArgDelimiter)  &&  query[beg] == ';')
         {
             ++beg;
             continue;
@@ -176,24 +180,27 @@ void CUrlArgs_Parser::SetQueryString(const string& query,
 // CUrlArgs
 //
 
-CUrlArgs::CUrlArgs(void)
-    : m_Case(NStr::eNocase),
+CUrlArgs::CUrlArgs(TFlags flags)
+    : CUrlArgs_Parser(flags),
+      m_Case(NStr::eNocase),
       m_IsIndex(false)
 {
     return;
 }
 
 
-CUrlArgs::CUrlArgs(const string& query, NStr::EUrlEncode decode)
-    : m_Case(NStr::eNocase),
+CUrlArgs::CUrlArgs(const string& query, NStr::EUrlEncode decode, TFlags flags)
+    : CUrlArgs_Parser(flags),
+      m_Case(NStr::eNocase),
       m_IsIndex(false)
 {
     SetQueryString(query, decode);
 }
 
 
-CUrlArgs::CUrlArgs(const string& query, const IUrlEncoder* encoder)
-    : m_Case(NStr::eNocase),
+CUrlArgs::CUrlArgs(const string& query, const IUrlEncoder* encoder, TFlags flags)
+    : CUrlArgs_Parser(flags),
+      m_Case(NStr::eNocase),
       m_IsIndex(false)
 {
     SetQueryString(query, encoder);
