@@ -1,4 +1,4 @@
-/*  $Id: ncbiapp.cpp 635548 2021-08-05 15:14:16Z gouriano $
+/*  $Id: ncbiapp.cpp 655216 2022-08-30 11:41:00Z fukanchi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -76,6 +76,7 @@ extern const char* s_ArgFullVersion;
 extern const char* s_ArgFullVersionXml;
 extern const char* s_ArgFullVersionJson;
 extern const char* s_ArgDryRun;
+extern const char* s_ArgDelimiter;
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -805,6 +806,9 @@ int CNcbiApplicationAPI::AppMain
             if ( !argv[i] ) {
                 continue;
             }
+            if ( NStr::strcmp(argv[i], s_ArgDelimiter) == 0 ) {
+                break;
+            }
             if ( NStr::strcmp(argv[i], s_ArgLogFile) == 0 ) {
                 if (!argv[++i]) {
                     continue;
@@ -883,10 +887,18 @@ int CNcbiApplicationAPI::AppMain
         const char** v = new const char*[argc];
         v[0] = argv[0];
         int real_arg_index = 1;
+        bool skip = false;
         for (int i = 1;  i < argc;  i++) {
             if ( !argv[i] ) {
                 continue;
             }
+            if ( NStr::strcmp(argv[i], s_ArgDelimiter) == 0 ) {
+                skip = true;
+            }
+            if (skip) {
+                v[real_arg_index++] = argv[i];
+                continue;
+            }
             // Log file - ignore if diag is eDS_User - the user wants to
             // take care about logging.
             if ( diag != eDS_User  &&
@@ -961,7 +973,8 @@ int CNcbiApplicationAPI::AppMain
                 conf = conf_arg;
             }
             else {
-                ERR_POST_X(23, Critical << "Registry file \"" << conf_arg << "\" not found");
+                ERR_POST_X(23, Critical << "Registry file \"" << conf_arg << "\" not found, aborting.");
+                Abort();
             }
         }
     }
