@@ -1,4 +1,4 @@
-/*  $Id: dbapi_conn_factory.cpp 630272 2021-04-29 15:55:00Z ucko $
+/*  $Id: dbapi_conn_factory.cpp 655273 2022-08-31 11:12:21Z fukanchi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -581,6 +581,9 @@ CDBConnectionFactory::DispatchServerName(
             }
 
             if (need_exclude) {
+                if (balancer.NotEmpty()) {
+                    balancer->LocallyPenalize(ctx.last_tried);
+                }
                 // Server might be temporarily unavailable ...
                 // Check conn_status ...
                 if (ctx.conn_status == IConnValidator::eTempInvalidConn) {
