@@ -1,4 +1,4 @@
-/*  $Id: dbapi_impl_context.cpp 630228 2021-04-28 20:30:25Z ucko $
+/*  $Id: dbapi_impl_context.cpp 653775 2022-08-03 14:42:32Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -958,6 +958,12 @@ CDriverContext::ReadDBConfParams(const string&  service_name,
         params->pool_max_conn_use
             = reg.Get(section_name, "conn_pool_max_conn_use");
     }
+    if (reg.HasEntry(section_name, "log_minor_messages",
+                     IRegistry::fCountCleared)) {
+        params->flags += SDBConfParams::fLogMinorMsgsSet;
+        params->log_minor_messages
+            = reg.Get(section_name, "log_minor_messages");
+    }
     if (reg.HasEntry(section_name, "args", IRegistry::fCountCleared)) {
         params->flags += SDBConfParams::fArgsSet;
         params->args = reg.Get(section_name, "args");
@@ -1185,6 +1191,21 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
         else if (params.GetParam("continue_after_raiserror") == "default") {
             act_params.SetParam("continue_after_raiserror", "false");
         }
+        if (conf_params.IsLogMinorMessagesSet()) {
+            if (conf_params.log_minor_messages.empty()) {
+                act_params.SetParam("log_minor_messages", "false");
+            }
+            else {
+                act_params.SetParam
+                    ("log_minor_messages", 
+                     NStr::BoolToString(
+                         NStr::StringToBool(
+                             conf_params.log_minor_messages)));
+            }
+        }
+        else if (params.GetParam("log_minor_messages") == "default") {
+            act_params.SetParam("log_minor_messages", "false");
+        }
 
         s_TransformLoginData(server_name, user_name, db_name, password);
         act_params.SetServerName(server_name);
