@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR___SEQ_ID_TREE__HPP
 #define OBJECTS_OBJMGR___SEQ_ID_TREE__HPP
 
-/*  $Id: seq_id_tree.hpp 628257 2021-03-26 17:49:22Z vasilche $
+/*  $Id: seq_id_tree.hpp 656854 2022-09-29 12:18:47Z fukanchi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -39,6 +39,7 @@
 #include <corelib/ncbi_limits.hpp>
 
 #include <objects/general/Date.hpp>
+#include <objects/general/Date_std.hpp>
 #include <objects/general/Dbtag.hpp>
 #include <objects/general/Object_id.hpp>
 
@@ -467,6 +468,8 @@ public:
     
     virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
     
+    int CompareOrdered(const CSeq_id_Info& other, const CSeq_id_Handle& h_this, const CSeq_id_Handle& h_other) const override;
+
 private:
     TKey m_Key;
 };
@@ -788,6 +791,8 @@ public:
     
     virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
     
+    int CompareOrdered(const CSeq_id_Info& other, const CSeq_id_Handle& h_this, const CSeq_id_Handle& h_other) const override;
+
 private:
     TKey m_Key;
 };
@@ -860,7 +865,11 @@ public:
     
     virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
     
+    int CompareOrdered(const CSeq_id_Info& other, const CSeq_id_Handle& h_this, const CSeq_id_Handle& h_other) const override;
+
 private:
+    string x_GetStr(TPacked param) const;
+
     TKey m_Key;
 };
 
@@ -1001,6 +1010,40 @@ private:
 // e_PDB tree
 
 
+class CSeq_id_PDB_Info : public CSeq_id_Info {
+public:
+    CSeq_id_PDB_Info(const CConstRef<CSeq_id>& seq_id, CSeq_id_Mapper* mapper);
+    
+    CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const override;
+    
+    static pair<CConstRef<CSeq_id>, TVariant> Normalize(const CSeq_id& seq_id);
+
+private:
+    static TVariant x_NormalizeDate(const CDate_std& date_std);
+    
+    // variant contains bits describing if chain fields are set,
+    // and normalized date fields - year, month, day, hour, minute, and second
+
+    // base constants
+    static const int kSecondBits = 6; // 0-63, 63 means not set
+    static const int kMinuteBits = 6; // 0-63, 63 means not set
+    static const int kHourBits = 5; // 0-31, 31 means not set
+    static const int kDayBits = 5; // 1-31, 0 means not set
+    static const int kMonthBits = 4; // 1-12, 0 means not set
+    static const int kYearBits = 12; // 1-4095
+
+    // derived constants
+    static const int kNoChainOffset = 0;
+    static const int kNoChain_idOffset = kNoChainOffset + 1;
+    static const int kSecondOffset = kNoChain_idOffset + 1;
+    static const int kMinuteOffset = kSecondOffset + kSecondBits;
+    static const int kHourOffset = kMinuteOffset + kMinuteBits;
+    static const int kDayOffset = kHourOffset + kHourBits;
+    static const int kMonthOffset = kDayOffset + kDayBits;
+    static const int kYearOffset = kMonthOffset + kMonthBits;
+};
+
+
 class CSeq_id_PDB_Tree : public CSeq_id_Which_Tree
 {
 public:
