@@ -1,4 +1,4 @@
-/*  $Id: seqdbimpl.cpp 644209 2022-01-24 13:33:29Z ivanov $
+/*  $Id: seqdbimpl.cpp 651939 2022-06-28 12:33:29Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -121,7 +121,6 @@ CSeqDBImpl::CSeqDBImpl(const string       & db_name_list,
 
             // This is a whole-database scan; it's always done in
             // approximate length mode.
-
             x_ScanTotals(true, & m_NumSeqs, & m_TotalLength,
                                & m_MaxLength, & m_MinLength, locked);
 
@@ -138,6 +137,9 @@ CSeqDBImpl::CSeqDBImpl(const string       & db_name_list,
         }
         m_NumSeqsStats     = x_GetNumSeqsStats();
         m_TotalLengthStats = x_GetTotalLengthStats();
+
+        LOG_POST(Info << "Num of Seqs: " << m_NumSeqs);
+        LOG_POST(Info << "Total Length: " << m_TotalLength);
     }
     catch(CSeqDBException & e) {
         m_UserGiList.Reset();
@@ -2620,6 +2622,14 @@ void CSeqDBImpl::SetVolsMemBit(int mbit)
     }
 }
 
+void CSeqDBImpl::SetVolsOidMaskType(int oid_mask_type)
+{
+    int nvols = m_VolSet.GetNumVols();
+    for (int vol = 0; vol < nvols; ++vol) {
+        m_VolSet.GetVolNonConst(vol)->SetOidMaskType(oid_mask_type);
+    }
+}
+
 void CSeqDBImpl::DebugDump(CDebugDumpContext ddc, unsigned int depth) const
 {
     ddc.SetFrame("CSeqDBImpl");
