@@ -1,4 +1,4 @@
-/*  $Id: gbnative.cpp 644848 2022-02-03 19:35:13Z ivanov $
+/*  $Id: gbnative.cpp 651743 2022-06-23 17:45:58Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -130,7 +130,7 @@ public:
     virtual void GetLoadedBlob_ids(const CSeq_id_Handle& idh,
                                    TLoadedBlob_ids& blob_ids) const override;
 
-    virtual operator CInitMutexPool&(void) { return GetMutexPool(); }
+    virtual operator CInitMutexPool&(void) override { return GetMutexPool(); }
 
     CInitMutexPool& GetMutexPool(void) { return m_Loader->m_MutexPool; }
 
