@@ -1,4 +1,4 @@
-/*  $Id: request_result.cpp 607120 2020-04-30 12:47:09Z grichenk $
+/*  $Id: request_result.cpp 654053 2022-08-08 14:53:18Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -247,6 +247,18 @@ void CFixedBlob_ids::SetNotFound(void)
 }
 
 
+CFixedBlob_ids CFixedBlob_ids::Filter(const SAnnotSelector* sel) const
+{
+    TList infos;
+    for ( auto& blob_info : Get() ) {
+        if ( blob_info.Matches(fBlobHasExtAnnot|fBlobHasNamedAnnot, sel) ) {
+            infos.push_back(blob_info);
+        }
+    }
+    return CFixedBlob_ids(eTakeOwnership, infos);
+}
+
+
 CNcbiOstream& operator<<(CNcbiOstream& out, const CFixedBlob_ids& ids)
 {
     const char* sep = "( ";
@@ -565,6 +577,7 @@ END_LOCAL_NAMESPACE;
 
 
 CLoadLockSetter::CLoadLockSetter(CLoadLockBlob& lock)
+    : m_AllowIncompleteLoading(false)
 {
     x_Init(lock, lock.GetSelectedChunkId());
 }
@@ -572,6 +585,7 @@ CLoadLockSetter::CLoadLockSetter(CLoadLockBlob& lock)
 
 CLoadLockSetter::CLoadLockSetter(CLoadLockBlob& lock,
                                  TChunkId chunk_id)
+    : m_AllowIncompleteLoading(false)
 {
     x_Init(lock, chunk_id);
 }
@@ -580,7 +594,8 @@ CLoadLockSetter::CLoadLockSetter(CLoadLockBlob& lock,
 CLoadLockSetter::CLoadLockSetter(CReaderRequestResult& result,
                                  const CBlob_id& blob_id,
                                  TChunkId chunk_id)
-    : TParent(result.GetLoadLockBlob(blob_id))
+    : TParent(result.GetLoadLockBlob(blob_id)),
+      m_AllowIncompleteLoading(false)
 {
     x_ObtainTSE_LoadLock(result, blob_id);
     if ( chunk_id != kMain_ChunkId ) {
@@ -615,7 +630,7 @@ void CLoadLockSetter::x_Init(CLoadLockBlob& lock, TChunkId chunk_id)
 
 CLoadLockSetter::~CLoadLockSetter(void)
 {
-    if ( !IsLoaded() ) {
+    if ( !m_AllowIncompleteLoading && !IsLoaded() ) {
         ERR_POST("Incomplete loading");
     }
 }
@@ -632,6 +647,12 @@ bool CLoadLockSetter::IsLoaded(void) const
 }
 
 
+void CLoadLockSetter::AllowIncompleteLoading()
+{
+    m_AllowIncompleteLoading = true;
+}
+
+
 void CLoadLockSetter::SetLoaded(void)
 {
     _ASSERT(!IsLoaded());
@@ -1039,6 +1060,33 @@ CLoadLockBlobIds::CLoadLockBlobIds(CReaderRequestResult& result,
 }
 
 
+bool CLoadLockBlobIds::SetLoadedBlob_ids(const SAnnotSelector* sel,
+                                         const TData& blob_ids)
+{
+    if ( !sel || sel->IsIncludedNamedAnnotAccession("NA*") ) {
+        // no filtering
+        return SetLoadedBlob_ids(blob_ids);
+    }
+    else {
+        return SetLoadedBlob_ids(blob_ids.Filter(sel));
+    }
+}
+
+
+bool CLoadLockBlobIds::SetLoadedBlob_ids(const SAnnotSelector* sel,
+                                         const TData& blob_ids,
+                                         TExpirationTime expiration)
+{
+    if ( !sel || sel->IsIncludedNamedAnnotAccession("NA*") ) {
+        // no filtering
+        return SetLoadedBlob_ids(blob_ids, expiration);
+    }
+    else {
+        return SetLoadedBlob_ids(blob_ids.Filter(sel), expiration);
+    }
+}
+
+
 CLoadLockBlobState::CLoadLockBlobState(CReaderRequestResult& result,
                                        const CBlob_id& id)
     : TParent(result.GetLoadLockBlobState(id))
