@@ -1,4 +1,4 @@
-/*  $Id: comment_item.cpp 643890 2022-01-19 18:03:38Z ivanov $
+/*  $Id: comment_item.cpp 657594 2022-10-13 13:00:55Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -73,7 +73,7 @@ BEGIN_SCOPE(objects)
 
 
 // static variables initialization
-bool CCommentItem::sm_FirstComment = true;
+// bool CCommentItem::sm_FirstComment = true;
 
 static const string kRefSeq = "REFSEQ";
 static const string kRefSeqInformation = "REFSEQ INFORMATION";
@@ -90,7 +90,7 @@ CCommentItem::CCommentItem(CBioseqContext& ctx, bool need_period) :
     m_First(false),
     m_NeedPeriod(need_period)
 {
-    swap(m_First, sm_FirstComment);
+    // swap(m_First, sm_FirstComment);
 }
 
 IFlatItem::EItem CCommentItem::GetItemType(void) const
@@ -112,7 +112,7 @@ CCommentItem::CCommentItem
     if (! ctx.Config().IsFormatGBSeq() && ! ctx.Config().IsFormatINSDSeq()) {
         ExpandTildes(m_Comment.back(), eTilde_comment);
     }
-    swap(m_First, sm_FirstComment);
+    // swap(m_First, sm_FirstComment);
     if ( obj != 0 ) {
         x_SetObject(*obj);
     }
@@ -125,7 +125,7 @@ CCommentItem::CCommentItem(const CSeqdesc&  desc, CBioseqContext& ctx) :
     m_First(false),
     m_NeedPeriod(true)
 {
-    swap(m_First, sm_FirstComment);
+    // swap(m_First, sm_FirstComment);
     x_SetObject(desc);
     x_GatherInfo(ctx);
     if ( x_IsCommentEmpty() ) {
@@ -140,7 +140,7 @@ CCommentItem::CCommentItem(const CSeq_feat& feat, CBioseqContext& ctx) :
     m_First(false),
     m_NeedPeriod(true)
 {
-    swap(m_First, sm_FirstComment);
+    // swap(m_First, sm_FirstComment);
     x_SetObject(feat);
     x_GatherInfo(ctx);
     NON_CONST_ITERATE( list<string>, it, m_Comment ) {
@@ -157,7 +157,7 @@ CCommentItem::CCommentItem(const CUser_object & userObject, CBioseqContext& ctx)
     m_First(false),
     m_NeedPeriod(true)
 {
-    swap(m_First, sm_FirstComment);
+    // swap(m_First, sm_FirstComment);
     x_SetObject(userObject);
     x_GatherInfo(ctx);
     if ( x_IsCommentEmpty() ) {
@@ -723,7 +723,7 @@ string CCommentItem::GetStringForRefTrack(const CBioseqContext& ctx, const CUser
             } else {
                 oss << " NCBI contigs are derived from assembled genomic sequence data.~"
                     << "Also see:~"
-                    << "    Documentation of NCBI's Annotation Process~ ";
+                    << "    Documentation of NCBI's Annotation Process ";
             }
         }
         break;
@@ -1289,7 +1289,7 @@ string CCommentItem::GetStringForModelEvidance(const CBioseqContext& ctx, const
         "Documentation" );
 
     text << ".~Also see:~"
-        << "    " << documentation_str << " of NCBI's Annotation Process~    ";
+        << "    " << documentation_str << " of NCBI's Annotation Process    ";
 
     return CNcbiOstrstreamToString(text);
 }
@@ -1510,10 +1510,6 @@ string CCommentItem::GetStringForBaseMod(CBioseqContext& ctx)
         return CNcbiOstrstreamToString(str);
     }
 
-    if( ! sm_FirstComment ) {
-        str << "\n";
-    }
-
     if ( numBases == 1 ) {
         str << "This genome has a ";
         if( bHtml ) {
@@ -1626,7 +1622,7 @@ string s_HtmlizeStructuredCommentData( const bool is_html, const string &label_s
                << data_str
                << "\">" << data_str << "</a>";
         return CNcbiOstrstreamToString(result);
-    } else if ( NStr::Equal (label_str, "Annotation Name") && NStr::Equal (provider, "NCBI") ) {
+    } else if ( NStr::Equal (label_str, "Annotation Name") && ( NStr::Equal (provider, "NCBI") || NStr::Equal (provider, "NCBI RefSeq") ) ) {
         string fst;
         string snd;
         if (NStr::Find(data_str, "Updated Annotation Release") != NPOS) {
@@ -1639,7 +1635,7 @@ string s_HtmlizeStructuredCommentData( const bool is_html, const string &label_s
                << snd
                << "\">" << data_str << "</a>";
         return CNcbiOstrstreamToString(result);
-    } else if ( NStr::Equal (label_str, "Annotation Version") && NStr::Equal (provider, "NCBI") && NStr::Equal (status, "Full annotation") && (! has_name) ) {
+    } else if ( NStr::Equal (label_str, "Annotation Version") && ( NStr::Equal (provider, "NCBI") || NStr::Equal (provider, "NCBI RefSeq") ) && NStr::Equal (status, "Full annotation") && (! has_name) ) {
         string fst;
         string snd;
         NStr::Replace( data_str, " Annotation Release ", "/", fst );
@@ -1760,9 +1756,11 @@ void s_GetStrForStructuredComment(
     }
     out_prefix_len = (longest_label_len + 4); // "+4" because we add " :: " after the prefix
 
+    /*
     if( ! is_first ) {
         out_lines.push_back( "\n" );
     }
+    */
     out_lines.push_back( prefix );
     out_lines.back().append( "\n" );
 
@@ -1916,7 +1914,7 @@ void CCommentItem::x_GatherUserObjInfo(const CUser_object& userObject )
 void CCommentItem::x_SetSkip(void)
 {
     CFlatItem::x_SetSkip();
-    swap(m_First, sm_FirstComment);
+    // swap(m_First, sm_FirstComment);
 }
 
 
@@ -2112,7 +2110,7 @@ void CGenomeAnnotComment::x_GatherInfo(CBioseqContext& ctx)
     } else {
         text << "NCBI contigs are derived from assembled genomic sequence data."
              << "~Also see:~"
-             << "    Documentation of NCBI's Annotation Process~ ";
+             << "    Documentation of NCBI's Annotation Process ";
     }
 
     /// add our assembly info
