@@ -1,4 +1,4 @@
-/*  $Id: context.cpp 642717 2021-12-27 19:04:38Z fukanchi $
+/*  $Id: context.cpp 656070 2022-09-14 12:55:32Z fukanchi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -122,6 +122,7 @@ CBioseqContext::CBioseqContext
     m_IsCrossKingdom(false),
     m_UsePDBCompoundForComment(false),
     m_fUnverified(fUnverified_None),
+    m_fUnreviewed(fUnreviewed_None),
     m_ShowAnnotCommentAsCOMMENT(false),
     m_ShowAnnotCommentAsCOMMENT_checked(false),
     m_FFCtx(ffctx),
@@ -185,6 +186,7 @@ CBioseqContext::CBioseqContext
     m_IsCrossKingdom(false),
     m_UsePDBCompoundForComment(false),
     m_fUnverified(fUnverified_None),
+    m_fUnreviewed(fUnreviewed_None),
     m_ShowAnnotCommentAsCOMMENT(false),
     m_ShowAnnotCommentAsCOMMENT_checked(false),
     m_FFCtx(ffctx),
@@ -656,6 +658,10 @@ void CBioseqContext::x_SetDataFromUserObjects(void)
                 if( m_fUnverified == fUnverified_None ) {
                     m_fUnverified = fUnverified_SequenceOrAnnotation;
                 }
+            } else if (utype == CUser_object::eObjectType_Unreviewed) {
+                if (uo.IsUnreviewedUnannotated()) {
+                    m_fUnreviewed |= fUnreviewed_Unannotated;
+                }
             } else if ( utype == CUser_object::eObjectType_FileTrack ) {
                 x_SetFiletrackURL(uo);
             } else if ( NStr::EqualNocase(uo.GetType().GetStr(), "AuthorizedAccess") ) {
