@@ -1,4 +1,4 @@
-/*  $Id: genbank_formatter.cpp 643889 2022-01-19 18:03:29Z ivanov $
+/*  $Id: genbank_formatter.cpp 657594 2022-10-13 13:00:55Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1363,10 +1363,11 @@ void CGenbankFormatter::FormatComment
             TryToSanitizeHtml(*comment_it);
         }
 
-        if (!is_first) {
-            Wrap(l, kEmptyStr, *comment_it, eSubp, bHtml, internalIndent);
+        string& comm = *comment_it;
+        if (is_first) {
+            Wrap(l, "COMMENT", comm, ePara, bHtml, internalIndent);
         } else {
-            Wrap(l, "COMMENT", *comment_it, ePara, bHtml, internalIndent);
+            Wrap(l, kEmptyStr, comm, eSubp, bHtml, internalIndent);
         }
 
         // Sometimes Wrap gets overzealous and wraps us right after the "::"
@@ -1381,7 +1382,24 @@ void CGenbankFormatter::FormatComment
     //    if ( bHtml ) {
     //        s_FixLineBrokenWeblinks( l );
     //    }
-    text_os.AddParagraph(l, comment.GetObject());
+
+    list<string> x;
+    ITERATE(list<string>, line, l) {
+        const string& txt = *line;
+        if (NStr::Find(txt, "COMMENT") != NPOS) {
+            x.push_back(txt);
+        } else if (NStr::Find(txt, "START##") != NPOS) {
+            x.push_back("            \n" + txt);
+        } else if (NStr::Find(txt, "this sequence version replaced") != NPOS ||
+                   NStr::Find(txt, "this sequence was replaced by") != NPOS ||
+                   NStr::Find(txt, "this project was updated") != NPOS) {
+            x.push_back("            \n" + txt);
+        } else {
+            x.push_back(txt);
+        }
+    }
+
+    text_os.AddParagraph(x, comment.GetObject());
 
     text_os.Flush();
 }
