@@ -1,4 +1,4 @@
-/*  $Id: load_blob.cpp 649495 2022-05-04 18:03:39Z ivanov $
+/*  $Id: load_blob.cpp 651942 2022-06-28 14:10:54Z saprykin $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,6 +33,7 @@
 
 #include <ncbi_pch.hpp>
 
+#include <objtools/pubseq_gateway/impl/cassandra/blob_storage.hpp>
 #include <objtools/pubseq_gateway/impl/cassandra/blob_task/load_blob.hpp>
 #include <objtools/pubseq_gateway/impl/cassandra/cass_blob_op.hpp>
 #include <objtools/pubseq_gateway/impl/cassandra/cass_driver.hpp>
@@ -48,68 +49,21 @@
 BEGIN_IDBLOB_SCOPE
 USING_NCBI_SCOPE;
 
-CCassBlobTaskLoadBlob::CCassBlobTaskLoadBlob(
-    unsigned int op_timeout_ms,
-    unsigned int max_retries,
-    shared_ptr<CCassConnection> conn,
-    const string & keyspace,
-    CBlobRecord::TSatKey sat_key,
-    bool load_chunks,
-    TDataErrorCallback data_error_cb
-)
-    : CCassBlobTaskLoadBlob(
-        conn,
-        keyspace,
-        sat_key,
-        kAnyModified,
-        load_chunks,
-        data_error_cb
-    )
-{
-    SetMaxRetries(max_retries);
+void CCassBlobTaskLoadBlob::InitBlobChunkDataQuery(
+    CCassQuery* query, string const& keyspace, CBlobRecord const& blob, int32_t chunk_no
+) {
+    if (query != nullptr) {
+        const char* const chunk_table_name = blob.GetFlag(EBlobFlags::eBigBlobSchema)
+             ? SBlobStorageConstants::kChunkTableBig
+             : SBlobStorageConstants::kChunkTableDefault;
+        query->SetSQL("SELECT data FROM " + keyspace + "." + chunk_table_name
+            + " WHERE sat_key = ? AND last_modified = ? AND chunk_no = ?", 3);
+        query->BindInt32(0, blob.GetKey());
+        query->BindInt64(1, blob.GetModified());
+        query->BindInt32(2, chunk_no);
+    }
 }
 
-CCassBlobTaskLoadBlob::CCassBlobTaskLoadBlob(
-    unsigned int op_timeout_ms,
-    unsigned int max_retries,
-    shared_ptr<CCassConnection> conn,
-    const string & keyspace,
-    CBlobRecord::TSatKey sat_key,
-    CBlobRecord::TTimestamp modified,
-    bool load_chunks,
-    TDataErrorCallback data_error_cb
-)
-    : CCassBlobWaiter(
-        op_timeout_ms, conn, keyspace, sat_key, true,
-        max_retries, move(data_error_cb)
-      )
-    , m_Blob(make_unique<CBlobRecord>(sat_key))
-    , m_Modified(modified)
-    , m_LoadChunks(load_chunks)
-{
-    m_Blob->SetModified(modified);
-}
-
-CCassBlobTaskLoadBlob::CCassBlobTaskLoadBlob(
-    unsigned int op_timeout_ms,
-    unsigned int max_retries,
-    shared_ptr<CCassConnection> conn,
-    const string & keyspace,
-    unique_ptr<CBlobRecord> blob_record,
-    bool load_chunks,
-    TDataErrorCallback data_error_cb
-)
-    : CCassBlobWaiter(
-        op_timeout_ms, conn, keyspace, blob_record->GetKey(), true,
-        max_retries, move(data_error_cb)
-      )
-    , m_Blob(move(blob_record))
-    , m_Modified(m_Blob->GetModified())
-    , m_LoadChunks(load_chunks)
-    , m_PropsFound(true)
-    , m_ExplicitBlob(true)
-{}
-
 CCassBlobTaskLoadBlob::CCassBlobTaskLoadBlob(
     shared_ptr<CCassConnection> conn,
     const string & keyspace,
@@ -156,6 +110,7 @@ CCassBlobTaskLoadBlob::CCassBlobTaskLoadBlob(
     , m_Modified(m_Blob->GetModified())
     , m_LoadChunks(load_chunks)
     , m_PropsFound(true)
+    , m_ExplicitBlob(true)
 {
 }
 
@@ -458,12 +413,7 @@ void CCassBlobTaskLoadBlob::x_RequestChunksAhead()
 
 void CCassBlobTaskLoadBlob::x_RequestChunk(CCassQuery& qry, int32_t chunk_no)
 {
-    string sql = "SELECT data FROM " + GetKeySpace() +
-        ".blob_chunk WHERE sat_key = ? AND last_modified = ? AND chunk_no = ?";
-    qry.SetSQL(sql, 3);
-    qry.BindInt32(0, GetKey());
-    qry.BindInt64(1, m_Blob->GetModified());
-    qry.BindInt32(2, chunk_no);
+    InitBlobChunkDataQuery(&qry, GetKeySpace(), *m_Blob, chunk_no);
     {
         auto DataReadyCb3 = m_DataReadyCb3.lock();
         if (DataReadyCb3) {
