@@ -1,4 +1,4 @@
-/*  $Id: cass_driver.cpp 642741 2021-12-28 15:17:53Z fukanchi $
+/*  $Id: cass_driver.cpp 653362 2022-07-26 16:21:14Z saprykin $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -49,8 +49,9 @@
 #include <utility>
 #include <vector>
 
-#include "corelib/ncbitime.hpp"
-#include "corelib/ncbistr.hpp"
+#include <corelib/ncbiapp.hpp>
+#include <corelib/ncbitime.hpp>
+#include <corelib/ncbistr.hpp>
 
 BEGIN_IDBLOB_SCOPE
 USING_NCBI_SCOPE;
@@ -94,6 +95,57 @@ BEGIN_SCOPE()
         }
         return CASS_LOG_ERROR;
     }
+
+    void SetDebugInformation(CassCluster* cluster)
+    {
+        auto app = CNcbiApplication::Instance();
+        if (app) {
+            auto name = app->GetProgramDisplayName();
+            cass_cluster_set_application_name_n(cluster, name.c_str(), name.size());
+            auto version = app->GetFullVersion().GetVersionInfo().Print();
+            if (!version.empty()) {
+                cass_cluster_set_application_version_n(cluster, version.c_str(), version.size());
+            }
+        }
+    }
+
+    using TSchemaMeta = const CassSchemaMeta;
+    using TTableMeta = const CassTableMeta;
+    auto GetMetaPointer(CassSession *m_session)
+    {
+        if (!m_session) {
+            RAISE_DB_ERROR(eSeqFailed, "invalid sequence of operations, driver is not connected");
+        }
+        unique_ptr<TSchemaMeta, function<void(TSchemaMeta*)>> schema_meta(
+            cass_session_get_schema_meta(m_session),
+            [](TSchemaMeta* meta)-> void {
+                cass_schema_meta_free(meta);
+            }
+        );
+        if (!schema_meta) {
+            NCBI_THROW(CCassandraException, eFatal, "Cluster metadata is not resolved");
+        }
+        return schema_meta;
+    }
+
+    TTableMeta* GetTableMetaPointer(TSchemaMeta* schema_meta, string const& keyspace, string const& table)
+    {
+        const CassKeyspaceMeta* keyspace_meta = cass_schema_meta_keyspace_by_name_n(
+            schema_meta, keyspace.c_str(), keyspace.size()
+        );
+        if (!keyspace_meta) {
+            NCBI_THROW(CCassandraException, eNotFound, "Keyspace not found");
+        }
+
+        TTableMeta* table_meta = cass_keyspace_meta_table_by_name_n(
+            keyspace_meta, table.c_str(), table.size()
+        );
+        if (!table_meta) {
+            NCBI_THROW(CCassandraException, eNotFound, "Table not found");
+        }
+
+        return table_meta;
+    }
 END_SCOPE()
 
 bool CCassConnection::m_LoggingInitialized = false;
@@ -159,11 +211,6 @@ void CCassConnection::UpdateLogging(void)
     }
 }
 
-unsigned int CCassConnection::QryTimeout() const
-{
-    return m_qtimeoutms;
-}
-
 unsigned int CCassConnection::QryTimeoutRetryMs() const
 {
     return m_qtimeout_retry_ms;
@@ -179,12 +226,6 @@ unsigned int CCassConnection::QryTimeoutMs() const
     return m_qtimeoutms;
 }
 
-void CCassConnection::SetRtLimits(unsigned int  numThreadsIo, unsigned int  numConnPerHost,
-                 unsigned int /*maxConnPerHost*/)
-{
-    SetRtLimits(numThreadsIo, numConnPerHost);
-}
-
 void CCassConnection::SetRtLimits(unsigned int numThreadsIo, unsigned int numConnPerHost)
 {
     m_numThreadsIo = numThreadsIo;
@@ -292,6 +333,7 @@ void CCassConnection::Connect()
 
     UpdateLogging();
     m_cluster = cass_cluster_new();
+    SetDebugInformation(m_cluster);
 
     cass_cluster_set_connect_timeout(m_cluster, m_ctimeoutms);
     cass_cluster_set_contact_points(m_cluster, m_hostlist.c_str());
@@ -499,92 +541,90 @@ shared_ptr<CCassQuery> CCassConnection::NewQuery()
     return rv;
 }
 
-void CCassConnection::getTokenRanges(TTokenRanges &ranges)
-{
-    GetTokenRanges(ranges);
-}
-
 void CCassConnection::GetTokenRanges(TTokenRanges &ranges)
 {
-    set<TTokenValue> cluster_tokens;
+    vector<TTokenValue> cluster_tokens;
     map<string, vector<TTokenValue>> peer_tokens;
+    string rpc_address, datacenter, schema, host_id;
 
-    auto query = NewQuery();
-    query->SetSQL("select data_center, schema_version, host_id, tokens "
-                  "from system.local", 0);
-    query->Query(CassConsistency::CASS_CONSISTENCY_LOCAL_ONE, false, false);
-
-    string host_id,  datacenter, schema;
-    vector<string> tokens;
-
-    query->NextRow();
-    query->FieldGetStrValue(0, datacenter);
-    query->FieldGetStrValue(1, schema);
-    query->FieldGetStrValue(2, host_id);
-    query->FieldGetSetValues(3, tokens);
-    auto itr = peer_tokens.insert(make_pair(host_id,vector<TTokenValue>())).first;
-    for(const auto & item: tokens) {
-        TTokenValue value = strtol(item.c_str(), nullptr, 10);
-        itr->second.push_back(value);
-        cluster_tokens.insert(value);
-    }
-
-    ERR_POST(Info << "GET_TOKEN_MAP: Schema version is " << schema);
-    ERR_POST(Info << "GET_TOKEN_MAP: datacenter is " << datacenter);
-    ERR_POST(Info << "GET_TOKEN_MAP: host_id is " << host_id);
-
-    unsigned int query_host_count = 0;
-    unsigned int retries = 0;
-    set<string> query_hosts;
-    while (retries < 3 && (query_host_count == 0 || peer_tokens.size() == query_host_count)) {
-        retries++;
-        if (query_host_count != 0) {
-            ERR_POST(Info << "GET_TOKEN_MAP: Host_id count is too small. "
-                "Retrying system.peers fetch. " << retries);
+    // Fetch tokens and schema version from any local host
+    {
+        vector<string> tokens;
+        auto query = NewQuery();
+        query->SetSQL("SELECT data_center, schema_version, rpc_address, host_id, tokens FROM system.local", 0);
+        query->Query(CassConsistency::CASS_CONSISTENCY_LOCAL_ONE, false, false);
+        query->NextRow();
+        query->FieldGetStrValue(0, datacenter);
+        query->FieldGetStrValue(1, schema);
+        query->FieldGetStrValue(2, rpc_address);
+        query->FieldGetStrValue(3, host_id);
+        query->FieldGetSetValues(4, tokens);
+        auto itr = peer_tokens.insert(make_pair(host_id, vector<TTokenValue>())).first;
+        for(const auto & item: tokens) {
+            TTokenValue value = strtol(item.c_str(), nullptr, 10);
+            itr->second.push_back(value);
+            cluster_tokens.push_back(value);
         }
 
-        query_host_count = 0;
-        query_hosts.clear();
-        query->SetSQL("select host_id, data_center, schema_version, tokens from system.peers", 0);
+        ERR_POST(Trace << "GET_TOKEN_MAP: Schema version is " << schema);
+        ERR_POST(Trace << "GET_TOKEN_MAP: datacenter is " << datacenter);
+        ERR_POST(Trace << "GET_TOKEN_MAP: host_id is " << host_id);
+        ERR_POST(Trace << "GET_TOKEN_MAP: rpc_address is " << rpc_address);
+    }
+
+    // Fetch peer tokens from the same local host
+    {
+        size_t peers_count{0};
+        auto query = NewQuery();
+        // We have to query the same host to get complete tokens distribution
+        query->SetHost(rpc_address);
+        query->SetSQL("SELECT data_center, schema_version, host_id, tokens FROM system.peers", 0);
         query->Query(CassConsistency::CASS_CONSISTENCY_LOCAL_ONE, false, false);
         while (query->NextRow() == ar_dataready) {
             string peer_host_id, peer_dc, peer_schema;
             vector<string> tokens;
-            query->FieldGetStrValue(1, peer_dc);
-            query->FieldGetStrValue(2, peer_schema);
+            query->FieldGetStrValue(0, peer_dc);
+            query->FieldGetStrValue(1, peer_schema);
             if (datacenter == peer_dc && schema == peer_schema) {
-                query->FieldGetStrValue(0, peer_host_id);
-                if (query_hosts.find(peer_host_id) == query_hosts.end()) {
-                    query_hosts.insert(peer_host_id);
-                    query_host_count++;
-                }
+                ++peers_count;
+                query->FieldGetStrValue(2, peer_host_id);
                 query->FieldGetSetValues(3, tokens);
-                ERR_POST(Info << "GET_TOKEN_MAP: host is " << peer_host_id);
-                ERR_POST(Info << "GET_TOKEN_MAP: tokens " << tokens.size());
+                ERR_POST(Trace << "GET_TOKEN_MAP: host is '" << peer_host_id
+                    << "'; tokens count - " << tokens.size());
                 auto itr = peer_tokens.find(peer_host_id);
                 if (itr == peer_tokens.end()) {
                     itr = peer_tokens.insert(make_pair(peer_host_id, vector<TTokenValue>())).first;
                     for (const auto & item : tokens) {
                         TTokenValue value = strtol(item.c_str(), nullptr, 10);
                         itr->second.push_back(value);
-                        cluster_tokens.insert(value);
+                        cluster_tokens.push_back(value);
                     }
                 }
             }
         }
-        ERR_POST(Info << "GET_TOKEN_MAP: PEERS HOST COUNT IS " << peer_tokens.size());
-        ERR_POST(Info << "GET_TOKEN_MAP: QUERY HOST COUNT IS " << query_host_count);
+        ERR_POST(Trace << "GET_TOKEN_MAP: TOTAL HOST COUNT IS " << peer_tokens.size());
+        if (peer_tokens.size() != peers_count + 1) {
+            NCBI_THROW(CCassandraException, eFatal, "Wrong count of peers while resolving cluster tokens");
+        }
     }
 
-    for(const auto& token : cluster_tokens) {
-        ERR_POST(Trace << "GET_TOKEN_MAP: \ttoken " << token);
+    // Sort and validate
+    {
+        auto token_count = cluster_tokens.size();
+        sort(cluster_tokens.begin(), cluster_tokens.end());
+        cluster_tokens.erase(unique(cluster_tokens.begin(), cluster_tokens.end()), cluster_tokens.end());
+        if (token_count != cluster_tokens.size()) {
+            NCBI_THROW(CCassandraException, eFatal, "Duplicate values resolved for cluster tokens");
+        }
     }
 
-    ERR_POST(Info << "GET_TOKEN_MAP: tokens size " << cluster_tokens.size());
+    // Format and return
+    ERR_POST(Trace << "GET_TOKEN_MAP: tokens count " << cluster_tokens.size());
+    ranges.clear();
     ranges.reserve(cluster_tokens.size() + 1);
     TTokenValue lower_bound = numeric_limits<TTokenValue>::min();
     for (int64_t token : cluster_tokens) {
-        ERR_POST(Trace << "GET_TOKEN_MAP: token " << token << " : " << token - lower_bound);
+        //ERR_POST(Trace << "GET_TOKEN_MAP: range " << lower_bound << " : " << token << "; distance is " << token - lower_bound);
         ranges.push_back(make_pair(lower_bound, token));
         lower_bound = token;
     }
@@ -668,38 +708,12 @@ string CCassConnection::GetDatacenterName()
 
 vector<string> CCassConnection::GetPartitionKeyColumnNames(string const & keyspace, string const & table) const
 {
-    if (!m_session) {
-        RAISE_DB_ERROR(eSeqFailed, "invalid sequence of operations, driver is not connected");
-    }
-    using TMeta = const CassSchemaMeta;
-    unique_ptr<TMeta, function<void(TMeta*)>> schema_meta(
-        cass_session_get_schema_meta(m_session),
-        [](TMeta* meta)-> void {
-            cass_schema_meta_free(meta);
-        }
-    );
-    if (!schema_meta) {
-        NCBI_THROW(CCassandraException, eFatal, "Cluster metadata is not resolved");
-    }
-
-    const CassKeyspaceMeta* keyspace_meta = cass_schema_meta_keyspace_by_name_n(
-        schema_meta.get(), keyspace.c_str(), keyspace.size()
-    );
-    if (!keyspace_meta) {
-        NCBI_THROW(CCassandraException, eNotFound, "Keyspace not found");
-    }
-
-    const CassTableMeta* table_meta = cass_keyspace_meta_table_by_name_n(
-        keyspace_meta, table.c_str(), table.size()
-    );
-    if (!table_meta) {
-        NCBI_THROW(CCassandraException, eNotFound, "Table not found");
-    }
-
-    size_t partition_key_count = cass_table_meta_partition_key_count(table_meta);
+    auto schema_meta = GetMetaPointer(m_session);
+    auto table_meta = GetTableMetaPointer(schema_meta.get(), keyspace, table);
+    size_t key_count = cass_table_meta_partition_key_count(table_meta);
     vector<string> result;
-    result.reserve(partition_key_count);
-    for (size_t i = 0; i < partition_key_count; ++i) {
+    result.reserve(key_count);
+    for (size_t i = 0; i < key_count; ++i) {
         const CassColumnMeta* column_meta = cass_table_meta_partition_key(table_meta, i);
         if (column_meta) {
             const char* name;
@@ -711,6 +725,25 @@ vector<string> CCassConnection::GetPartitionKeyColumnNames(string const & keyspa
     return result;
 }
 
+vector<string> CCassConnection::GetClusteringKeyColumnNames(string const & keyspace, string const & table) const
+{
+    auto schema_meta = GetMetaPointer(m_session);
+    auto table_meta = GetTableMetaPointer(schema_meta.get(), keyspace, table);
+    size_t key_count = cass_table_meta_clustering_key_count(table_meta);
+    vector<string> result;
+    result.reserve(key_count);
+    for (size_t i = 0; i < key_count; ++i) {
+        const CassColumnMeta* column_meta = cass_table_meta_clustering_key(table_meta, i);
+        if (column_meta) {
+            const char* name;
+            size_t name_length;
+            cass_column_meta_name(column_meta, &name, &name_length);
+            result.emplace_back(name, name_length);
+        }
+    }
+    return result;
+}
+
 const CassPrepared * CCassConnection::Prepare(const string &  sql)
 {
     const CassPrepared * rv = nullptr;
@@ -1141,7 +1174,8 @@ void CCassQuery::Query(CassConsistency  c, bool  run_async,
     }
 
     if (!m_execution_host.empty()) {
-        cass_statement_set_host_n(m_statement, m_execution_host.c_str(), m_execution_host.size(), m_connection->m_port);
+        int port = m_connection->m_port ? m_connection->m_port : 9042;
+        cass_statement_set_host_n(m_statement, m_execution_host.c_str(), m_execution_host.size(), port);
     }
 
     try {
