@@ -1,4 +1,4 @@
-/*  $Id: nannot_fetch.cpp 642742 2021-12-28 15:18:08Z fukanchi $
+/*  $Id: nannot_fetch.cpp 653723 2022-08-02 16:00:49Z saprykin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -50,6 +50,7 @@
 #include <objects/seqsplit/ID2S_Seq_annot_Info.hpp>
 
 #include <objtools/pubseq_gateway/impl/cassandra/nannot_task/fetch.hpp>
+#include <objtools/pubseq_gateway/impl/cassandra/nannot/filter.hpp>
 #include <objtools/pubseq_gateway/impl/cassandra/cass_driver.hpp>
 #include <objtools/pubseq_gateway/impl/cassandra/cass_factory.hpp>
 
@@ -87,6 +88,8 @@ class CNAnnotTaskFetchTest
     static shared_ptr<CCassConnection> m_Connection;
 
     string m_KeyspaceName{"nannotg2"};
+    string m_FrozenKeyspaceName{"psg_test_sat_41"};
+    string m_NewNannotSchemaKeyspace{"psg_test_sat_52"};
 };
 
 class CCassNAnnotTaskFetchWithTimeout : public CCassNAnnotTaskFetch
@@ -212,6 +215,7 @@ TEST_F(CNAnnotTaskFetchTest, SingleRetrievalVector) {
                 EXPECT_EQ(entry.GetAnnotName(), "NA000122202.1");
                 EXPECT_EQ(entry.GetStart(), 9853);
                 EXPECT_EQ(entry.GetStop(), 9858);
+                EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
             }
             return true;
         },
@@ -242,6 +246,7 @@ TEST_F(CNAnnotTaskFetchTest, SingleRetrievalTempString) {
                 EXPECT_EQ(entry.GetAnnotName(), "NA000122202.1");
                 EXPECT_EQ(entry.GetStart(), 9853);
                 EXPECT_EQ(entry.GetStop(), 9858);
+                EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
             }
             return true;
         },
@@ -269,6 +274,7 @@ TEST_F(CNAnnotTaskFetchTest, MultipleRetrieval) {
                     EXPECT_EQ(entry.GetStart(), 9853);
                     EXPECT_EQ(entry.GetStop(), 9858);
                     EXPECT_EQ(entry.GetSatKey(), 888);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 2:
@@ -276,6 +282,7 @@ TEST_F(CNAnnotTaskFetchTest, MultipleRetrieval) {
                     EXPECT_EQ(entry.GetStart(), 2506);
                     EXPECT_EQ(entry.GetStop(), 8119);
                     EXPECT_EQ(entry.GetSatKey(), 19347);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 3:
@@ -283,6 +290,7 @@ TEST_F(CNAnnotTaskFetchTest, MultipleRetrieval) {
                     EXPECT_EQ(entry.GetStart(), 640);
                     EXPECT_EQ(entry.GetStop(), 5865);
                     EXPECT_EQ(entry.GetSatKey(), 39419);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 4:
@@ -290,6 +298,7 @@ TEST_F(CNAnnotTaskFetchTest, MultipleRetrieval) {
                     EXPECT_EQ(entry.GetStart(), 992);
                     EXPECT_EQ(entry.GetStop(), 1445);
                     EXPECT_EQ(entry.GetSatKey(), 58472);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return false;
                 case 5:
@@ -324,6 +333,7 @@ TEST_F(CNAnnotTaskFetchTest, MultipleRetrievalWithTimeout) {
                     EXPECT_EQ(entry.GetStart(), 9853);
                     EXPECT_EQ(entry.GetStop(), 9858);
                     EXPECT_EQ(entry.GetSatKey(), 888);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 2:
@@ -331,6 +341,7 @@ TEST_F(CNAnnotTaskFetchTest, MultipleRetrievalWithTimeout) {
                     EXPECT_EQ(entry.GetStart(), 2506);
                     EXPECT_EQ(entry.GetStop(), 8119);
                     EXPECT_EQ(entry.GetSatKey(), 19347);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 3:
@@ -338,6 +349,7 @@ TEST_F(CNAnnotTaskFetchTest, MultipleRetrievalWithTimeout) {
                     EXPECT_EQ(entry.GetStart(), 640);
                     EXPECT_EQ(entry.GetStop(), 5865);
                     EXPECT_EQ(entry.GetSatKey(), 39419);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 4:
@@ -345,6 +357,7 @@ TEST_F(CNAnnotTaskFetchTest, MultipleRetrievalWithTimeout) {
                     EXPECT_EQ(entry.GetStart(), 992);
                     EXPECT_EQ(entry.GetStop(), 1445);
                     EXPECT_EQ(entry.GetSatKey(), 58472);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return false;
                 case 5:
@@ -385,6 +398,7 @@ TEST_F(CNAnnotTaskFetchTest, ListRetrievalWithTimeout) {
                     EXPECT_EQ(entry.GetStart(), 9853);
                     EXPECT_EQ(entry.GetStop(), 9858);
                     EXPECT_EQ(entry.GetSatKey(), 888);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 2:
@@ -392,6 +406,7 @@ TEST_F(CNAnnotTaskFetchTest, ListRetrievalWithTimeout) {
                     EXPECT_EQ(entry.GetStart(), 2506);
                     EXPECT_EQ(entry.GetStop(), 8119);
                     EXPECT_EQ(entry.GetSatKey(), 19347);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 3:
@@ -399,6 +414,7 @@ TEST_F(CNAnnotTaskFetchTest, ListRetrievalWithTimeout) {
                     EXPECT_EQ(entry.GetStart(), 640);
                     EXPECT_EQ(entry.GetStop(), 5865);
                     EXPECT_EQ(entry.GetSatKey(), 39419);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 4:
@@ -439,6 +455,7 @@ TEST_F(CNAnnotTaskFetchTest, ListRetrievalWithTimeoutOnEOF) {
                     EXPECT_EQ(entry.GetStart(), 9853);
                     EXPECT_EQ(entry.GetStop(), 9858);
                     EXPECT_EQ(entry.GetSatKey(), 888);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 2:
@@ -446,6 +463,7 @@ TEST_F(CNAnnotTaskFetchTest, ListRetrievalWithTimeoutOnEOF) {
                     EXPECT_EQ(entry.GetStart(), 2506);
                     EXPECT_EQ(entry.GetStop(), 8119);
                     EXPECT_EQ(entry.GetSatKey(), 19347);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 3:
@@ -453,6 +471,7 @@ TEST_F(CNAnnotTaskFetchTest, ListRetrievalWithTimeoutOnEOF) {
                     EXPECT_EQ(entry.GetStart(), 640);
                     EXPECT_EQ(entry.GetStop(), 5865);
                     EXPECT_EQ(entry.GetSatKey(), 39419);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 4:
@@ -495,6 +514,7 @@ TEST_F(CNAnnotTaskFetchTest, ListRetrievalWithCancel) {
                     EXPECT_EQ(entry.GetStart(), 9853);
                     EXPECT_EQ(entry.GetStop(), 9858);
                     EXPECT_EQ(entry.GetSatKey(), 888);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 2:
@@ -502,6 +522,7 @@ TEST_F(CNAnnotTaskFetchTest, ListRetrievalWithCancel) {
                     EXPECT_EQ(entry.GetStart(), 2506);
                     EXPECT_EQ(entry.GetStop(), 8119);
                     EXPECT_EQ(entry.GetSatKey(), 19347);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 default:
@@ -543,12 +564,13 @@ TEST_F(CNAnnotTaskFetchTest, RetrievalWithSeqAnnotInfo) {
                     EXPECT_EQ(entry.GetStart(), 6616);
                     EXPECT_EQ(entry.GetStop(), 41589);
                     EXPECT_EQ(entry.GetSatKey(), 500);
-                    EXPECT_EQ(entry.GetAnnotInfoModified(), 1567710575270);
+                    EXPECT_EQ(entry.GetAnnotInfoModified(), 1652276051850);
                     EXPECT_FALSE(entry.GetSeqAnnotInfo().empty());
                     EXPECT_EQ(gGetSeqAnnotInfoString(&entry),
                         "ID2S-Seq-annot-Info ::= {name \"NA000122202.1\",feat {{type 13}},seq-loc gi-interval {gi 1142972004,start 6616,length 34974}}\n"
                         "ID2S-Seq-annot-Info ::= {name \"NA000122202.1@@10\",graph NULL,seq-loc gi-interval {gi 1142972004,start 6610,length 34980}}\n"
                         "ID2S-Seq-annot-Info ::= {name \"NA000122202.1@@100\",graph NULL,seq-loc gi-interval {gi 1142972004,start 6610,length 35000}}\n");
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 2:
@@ -585,13 +607,14 @@ TEST_F(CNAnnotTaskFetchTest, RetrievalWithSeqAnnotInfo2) {
                     EXPECT_EQ(entry.GetStart(), 14153);
                     EXPECT_EQ(entry.GetStop(), 77991);
                     EXPECT_EQ(entry.GetSatKey(), 26395559);
-                    EXPECT_EQ(entry.GetAnnotInfoModified(), 1567919439853);
+                    EXPECT_EQ(entry.GetAnnotInfoModified(), 1652726171060);
                     EXPECT_FALSE(entry.GetSeqAnnotInfo().empty());
                     EXPECT_EQ(gGetSeqAnnotInfoString(&entry),
                         "ID2S-Seq-annot-Info ::= {name \"NA000156740.1\",feat {{type 8,subtypes {56}}},seq-loc gi-interval {gi 1385009444,start 14153,length 63839}}\n"
                         "ID2S-Seq-annot-Info ::= {name \"NA000156740.1@@10\",graph NULL,seq-loc gi-interval {gi 1385009444,start 14150,length 63850}}\n"
                         "ID2S-Seq-annot-Info ::= {name \"NA000156740.1@@100\",graph NULL,seq-loc gi-interval {gi 1385009444,start 14150,length 63900}}\n"
                     );
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
                     EXPECT_FALSE(last);
                     return true;
                 case 2:
@@ -616,4 +639,158 @@ TEST_F(CNAnnotTaskFetchTest, RetrievalWithSeqAnnotInfo2) {
     EXPECT_EQ(call_count, 2UL);
 }
 
+TEST_F(CNAnnotTaskFetchTest, RetrievalWithDeadRecords) {
+    size_t call_count = 0;
+    CCassNAnnotTaskFetch fetch(
+        m_Connection, m_FrozenKeyspaceName,
+        "NW_019824444", 1, 10,
+        [&call_count](CNAnnotRecord && entry, bool last) -> bool {
+            switch(++call_count) {
+                case 1:
+                    EXPECT_EQ(entry.GetAnnotName(), "NA000888889.1");
+                    EXPECT_EQ(entry.GetStart(), 777);
+                    EXPECT_EQ(entry.GetStop(), 1234);
+                    EXPECT_EQ(entry.GetSatKey(), 222222222);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
+                    EXPECT_FALSE(last);
+                    return true;
+                case 2:
+                    EXPECT_TRUE(last);
+                    return false;
+                default:
+                    EXPECT_TRUE(false) << "Callback should not be called " << call_count << " times.";
+            }
+            return true;
+        },
+        [](CRequestStatus::ECode status, int code, EDiagSev severity, const string & message) {
+            EXPECT_TRUE(false) << "Error callback called during the test (status - "
+                << status << ", code - " << code << ", message - '" << message << "')";
+        }
+    );
+    bool done = fetch.Wait();
+    while (!done) {
+        usleep(100);
+        done = fetch.Wait();
+    }
+}
+
+TEST_F(CNAnnotTaskFetchTest, RetrievalWithSatKeyInPrimary) {
+    size_t call_count = 0;
+    CCassNAnnotTaskFetch fetch(
+        m_Connection, m_NewNannotSchemaKeyspace,
+        "NC_062543", 1, 10, vector<string>({"NA000353807.1"}),
+        [&call_count](CNAnnotRecord && entry, bool last) -> bool {
+            switch(++call_count) {
+                case 1:
+                    EXPECT_EQ(entry.GetAnnotName(), "NA000353807.1");
+                    EXPECT_EQ(entry.GetStart(), 447);
+                    EXPECT_EQ(entry.GetStop(), 12505360);
+                    EXPECT_EQ(entry.GetSatKey(), 8754056);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
+                    EXPECT_FALSE(last);
+                    return true;
+                case 2:
+                    EXPECT_EQ(entry.GetAnnotName(), "NA000353807.1");
+                    EXPECT_EQ(entry.GetStart(), 11751284);
+                    EXPECT_EQ(entry.GetStop(), 16972309);
+                    EXPECT_EQ(entry.GetSatKey(), 8754057);
+                    EXPECT_EQ(entry.GetState(), CNAnnotRecord::eStateAlive);
+                    EXPECT_FALSE(last);
+                    return true;
+                case 3:
+                    EXPECT_TRUE(last);
+                    return false;
+                default:
+                    EXPECT_TRUE(false) << "Callback should not be called " << call_count << " times.";
+            }
+            return true;
+        },
+        [](CRequestStatus::ECode status, int code, EDiagSev severity, const string & message) {
+            EXPECT_TRUE(false) << "Error callback called during the test (status - "
+                << status << ", code - " << code << ", message - '" << message << "')";
+        }
+    );
+    bool done = fetch.Wait();
+    while (!done) {
+        usleep(100);
+        done = fetch.Wait();
+    }
+}
+
+TEST_F(CNAnnotTaskFetchTest, DuplicateFilterTest)
+{
+    CNAnnotFilter filter;
+
+    CNAnnotRecord record;
+    record.SetAccession("NC_062543");
+    record.SetVersion(1);
+    record.SetSeqIdType(10);
+    record.SetAnnotName("NA000353807.1");
+
+    auto copy = record;
+    copy.SetSatKey(1);
+    filter.Store(3, move(copy));
+
+    copy = record;
+    copy.SetSatKey(2);
+    filter.Store(3, move(copy));
+
+    copy = record;
+    copy.SetSatKey(3);
+    filter.Store(3, move(copy));
+
+    copy = record;
+    copy.SetSatKey(4);
+    filter.Store(2, move(copy));
+
+    copy = record;
+    record.SetAnnotName("NA000353808.1");
+    copy.SetSatKey(5);
+    filter.Store(2, move(copy));
+
+    size_t index{0};
+    filter.Consume(
+        [&index]
+        (int32_t sat, CNAnnotRecord&& entry)
+        {
+            switch(++index) {
+                case 1:
+                    EXPECT_EQ(sat, 3);
+                    EXPECT_EQ(entry.GetAccession(), "NC_062543");
+                    EXPECT_EQ(entry.GetVersion(), 1);
+                    EXPECT_EQ(entry.GetSeqIdType(), 10);
+                    EXPECT_EQ(entry.GetAnnotName(), "NA000353807.1");
+                    EXPECT_EQ(entry.GetSatKey(), 1);
+                    break;
+                case 2:
+                    EXPECT_EQ(sat, 3);
+                    EXPECT_EQ(entry.GetAccession(), "NC_062543");
+                    EXPECT_EQ(entry.GetVersion(), 1);
+                    EXPECT_EQ(entry.GetSeqIdType(), 10);
+                    EXPECT_EQ(entry.GetAnnotName(), "NA000353807.1");
+                    EXPECT_EQ(entry.GetSatKey(), 2);
+                    break;
+                case 3:
+                    EXPECT_EQ(sat, 3);
+                    EXPECT_EQ(entry.GetAccession(), "NC_062543");
+                    EXPECT_EQ(entry.GetVersion(), 1);
+                    EXPECT_EQ(entry.GetSeqIdType(), 10);
+                    EXPECT_EQ(entry.GetAnnotName(), "NA000353807.1");
+                    EXPECT_EQ(entry.GetSatKey(), 3);
+                    break;
+                case 4:
+                    EXPECT_EQ(sat, 2);
+                    EXPECT_EQ(entry.GetAccession(), "NC_062543");
+                    EXPECT_EQ(entry.GetVersion(), 1);
+                    EXPECT_EQ(entry.GetSeqIdType(), 10);
+                    EXPECT_EQ(entry.GetAnnotName(), "NA000353808.1");
+                    EXPECT_EQ(entry.GetSatKey(), 5);
+                    break;
+                default:
+                    EXPECT_TRUE(false) << "Callback should not be called " << index << " times.";
+            }
+        }
+     );
+}
+
 }  // namespace
