@@ -1,4 +1,4 @@
-/*  $Id: psg_client.cpp 642693 2021-12-27 13:58:40Z fukanchi $
+/*  $Id: psg_client.cpp 649484 2022-05-04 16:46:38Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -70,8 +70,9 @@ const char* CPSG_Exception::GetErrCodeString(void) const
 }
 
 
-SPSG_BlobReader::SPSG_BlobReader(SPSG_Reply::SItem::TTS& src)
-    : m_Src(src)
+SPSG_BlobReader::SPSG_BlobReader(SPSG_Reply::SItem::TTS& src, TStats stats)
+    : m_Src(src),
+      m_Stats(stats)
 {
 }
 
@@ -161,13 +162,40 @@ void SPSG_BlobReader::CheckForNewChunks()
     if (m_Data.size() < chunks.size()) m_Data.resize(chunks.size());
 
     for (size_t i = 0; i < chunks.size(); ++i) {
-        if (!chunks[i].empty()) {
+        if (auto size = chunks[i].size()) {
             m_Data[i].swap(chunks[i]);
+            if (auto stats = m_Stats.second.lock()) stats->AddData(m_Stats.first, SPSG_Stats::eRead, size);
         }
     }
 }
 
 
+const char* s_GetRequestTypeName(CPSG_Request::EType type)
+{
+    switch (type) {
+        case CPSG_Request::eBiodata:        return "biodata";
+        case CPSG_Request::eResolve:        return "resolve";
+        case CPSG_Request::eBlob:           return "blob";
+        case CPSG_Request::eNamedAnnotInfo: return "annot";
+        case CPSG_Request::eChunk:          return "chunk";
+    }
+
+    // Should not happen
+    _TROUBLE;
+    return "unknown";
+}
+
+CPSG_Request::SType::operator string() const
+{
+    return s_GetRequestTypeName(m_Type);
+}
+
+ostream& operator<<(ostream& os, const CPSG_Request::SType& type)
+{
+    return os << s_GetRequestTypeName(type.m_Type);
+}
+
+
 static_assert(is_nothrow_move_constructible<CPSG_BioId>::value, "CPSG_BioId move constructor must be noexcept");
 
 string CPSG_BioId::Repr() const
@@ -199,33 +227,6 @@ string CPSG_BlobId::Repr() const
     return m_LastModified.IsNull() ? m_Id : m_Id + '~' + to_string(m_LastModified.GetValue());
 }
 
-unique_ptr<CPSG_DataId> s_GetDataId(const SPSG_Args& args)
-{
-    try {
-        const auto& blob_id = args.GetValue("blob_id");
-
-        if (blob_id.empty()) {
-            auto id2_chunk = NStr::StringToNumeric<int>(args.GetValue("id2_chunk"));
-            return unique_ptr<CPSG_DataId>(new CPSG_ChunkId(id2_chunk, args.GetValue("id2_info")));
-        }
-
-        CPSG_BlobId::TLastModified last_modified;
-        const auto& last_modified_str = args.GetValue("last_modified");
-
-        if (last_modified_str.empty()) {
-            return unique_ptr<CPSG_DataId>(new CPSG_BlobId(blob_id));
-        }
-
-        last_modified = NStr::StringToNumeric<Int8>(last_modified_str);
-        return unique_ptr<CPSG_DataId>(new CPSG_BlobId(blob_id, move(last_modified)));
-    }
-    catch (...) {
-        NCBI_THROW_FMT(CPSG_Exception, eServerError,
-                "Both blob_id[+last_modified] and id2_chunk+id2_info pairs are missing/corrupted in server response: " <<
-                args.GetQueryString(CUrlArgs::eAmp_Char));
-    }
-}
-
 CPSG_BlobId s_GetBlobId(const CJsonNode& data)
 {
     CPSG_BlobId::TLastModified last_modified;
@@ -261,8 +262,93 @@ ostream& operator<<(ostream& os, const CPSG_ChunkId& chunk_id)
 }
 
 
+struct SDataId
+{
+    enum ETypePriority { eBlobIdPriority, eChunkIdPriority };
+
+    SDataId(const SPSG_Args& args) : m_Args(args) {}
+
+    template <ETypePriority = eBlobIdPriority>
+    bool HasBlobId() const { return !m_Args.GetValue<SPSG_Args::eBlobId>().get().empty(); }
+
+    template <ETypePriority type_priority = eBlobIdPriority>
+    static unique_ptr<CPSG_DataId> Get(SDataId data_id);
+
+    template <class TRequestedId = CPSG_DataId>
+    unique_ptr<CPSG_DataId> Get(shared_ptr<SPSG_Stats>& stats);
+
+private:
+    template <class TRequestedId> unique_ptr<TRequestedId> x_Get() const;
+
+    template <class TRequestedId, class TAllowedId>
+    unique_ptr<TRequestedId> Get() const;
+
+    const SPSG_Args& m_Args;
+};
+
+template <>
+bool SDataId::HasBlobId<SDataId::eChunkIdPriority>() const
+{
+    return m_Args.GetValue<SPSG_Args::eId2Chunk>().get().empty();
+}
+
+template <>
+unique_ptr<CPSG_BlobId> SDataId::x_Get<CPSG_BlobId>() const
+{
+    CPSG_BlobId::TLastModified last_modified;
+    const auto& blob_id = m_Args.GetValue<SPSG_Args::eBlobId>();
+    const auto& last_modified_str = m_Args.GetValue("last_modified");
+
+    if (last_modified_str.empty()) {
+        return make_unique<CPSG_BlobId>(blob_id);
+    }
+
+    last_modified = NStr::StringToNumeric<Int8>(last_modified_str);
+    return make_unique<CPSG_BlobId>(blob_id, move(last_modified));
+}
+
+template <>
+unique_ptr<CPSG_ChunkId> SDataId::x_Get<CPSG_ChunkId>() const
+{
+    auto id2_chunk = NStr::StringToNumeric<int>(m_Args.GetValue<SPSG_Args::eId2Chunk>().get());
+    return make_unique<CPSG_ChunkId>(id2_chunk, m_Args.GetValue("id2_info"));
+}
+
+template <class TRequestedId, class TAllowedId>
+unique_ptr<TRequestedId> SDataId::Get() const
+{
+    try {
+        return x_Get<TAllowedId>();
+    }
+    catch (...) {
+        NCBI_THROW_FMT(CPSG_Exception, eServerError,
+                "Both blob_id[+last_modified] and id2_chunk+id2_info pairs are missing/corrupted in server response: " <<
+                m_Args.GetQueryString(CUrlArgs::eAmp_Char));
+    }
+}
+
+template <SDataId::ETypePriority type_priority>
+unique_ptr<CPSG_DataId> SDataId::Get(SDataId data_id)
+{
+    return data_id.HasBlobId<type_priority>() ? data_id.Get<CPSG_DataId, CPSG_BlobId>() : data_id.Get<CPSG_DataId, CPSG_ChunkId>();
+}
+
+template <class TRequestedId>
+unique_ptr<CPSG_DataId> SDataId::Get(shared_ptr<SPSG_Stats>& stats)
+{
+    auto id = Get<TRequestedId, TRequestedId>();
+    if (stats) stats->AddId(*id);
+    return id;
+}
+
+template <>
+unique_ptr<CPSG_DataId> SDataId::Get<CPSG_DataId>(shared_ptr<SPSG_Stats>& stats)
+{
+    return HasBlobId() ? Get<CPSG_BlobId>(stats) : Get<CPSG_ChunkId>(stats);
+}
+
 template <class TReplyItem>
-TReplyItem* CPSG_Reply::SImpl::CreateImpl(TReplyItem* item, const vector<SPSG_Chunk>& chunks)
+CPSG_ReplyItem* CPSG_Reply::SImpl::CreateImpl(TReplyItem* item, const vector<SPSG_Chunk>& chunks)
 {
     if (chunks.empty()) return item;
 
@@ -272,69 +358,12 @@ TReplyItem* CPSG_Reply::SImpl::CreateImpl(TReplyItem* item, const vector<SPSG_Ch
     return rv.release();
 }
 
-struct SItemTypeAndReason : pair<CPSG_ReplyItem::EType, CPSG_SkippedBlob::EReason>
+CPSG_ReplyItem* CPSG_Reply::SImpl::CreateImpl(SPSG_Reply::SItem::TTS& item_ts, const SPSG_Args& args, shared_ptr<SPSG_Stats>& stats)
 {
-    static SItemTypeAndReason Get(const SPSG_Args& args);
-
-private:
-    using TBase = pair<CPSG_ReplyItem::EType, CPSG_SkippedBlob::EReason>;
-    using TBase::TBase;
-    SItemTypeAndReason(CPSG_ReplyItem::EType type) : TBase(type, CPSG_SkippedBlob::eUnknown) {}
-};
-
-// Item type cannot be determined by "item_type" alone, for blobs "reason" has to be used as well.
-// That's why there are two switch-like groups of 'if' statements in two different methods below.
-SItemTypeAndReason SItemTypeAndReason::Get(const SPSG_Args& args)
-{
-    const auto item_type = args.GetValue("item_type");
-
-    if (item_type == "blob") {
-        const auto reason = args.GetValue("reason");
-
-        if (reason.empty()) {
-            return CPSG_ReplyItem::eBlobData;
-
-        } else if (reason == "excluded") {
-            return { CPSG_ReplyItem::eSkippedBlob, CPSG_SkippedBlob::eExcluded };
-
-        } else if (reason == "inprogress") {
-            return { CPSG_ReplyItem::eSkippedBlob, CPSG_SkippedBlob::eInProgress };
-
-        } else if (reason == "sent") {
-            return { CPSG_ReplyItem::eSkippedBlob, CPSG_SkippedBlob::eSent };
-
-        } else {
-            return { CPSG_ReplyItem::eSkippedBlob, CPSG_SkippedBlob::eUnknown };
-        }
-
-    } else if (item_type == "bioseq_info") {
-        return CPSG_ReplyItem::eBioseqInfo;
-
-    } else if (item_type == "blob_prop") {
-        return CPSG_ReplyItem::eBlobInfo;
-
-    } else if (item_type == "bioseq_na") {
-        return CPSG_ReplyItem::eNamedAnnotInfo;
-
-    } else if (item_type == "public_comment") {
-        return CPSG_ReplyItem::ePublicComment;
-
-    } else if (item_type == "processor") {
-        return CPSG_ReplyItem::eProcessor;
-
-    } else {
-        if (TPSG_FailOnUnknownItems::GetDefault()) {
-            NCBI_THROW_FMT(CPSG_Exception, eServerError, "Received unknown item type: " << item_type);
-        }
-
-        static atomic_bool reported(false);
-
-        if (!reported.exchange(true)) {
-            ERR_POST("Received unknown item type: " << item_type);
-        }
-
-        return CPSG_ReplyItem::eEndOfReply;
-    }
+    SDataId data_id(args);
+    unique_ptr<CPSG_BlobData> blob_data(new CPSG_BlobData(data_id.Get(stats)));
+    blob_data->m_Stream.reset(new SPSG_RStream(item_ts, make_pair(data_id.HasBlobId(), reply->stats)));
+    return blob_data.release();
 }
 
 CPSG_SkippedBlob::TSeconds s_GetSeconds(const SPSG_Args& args, const string& name)
@@ -346,66 +375,147 @@ CPSG_SkippedBlob::TSeconds s_GetSeconds(const SPSG_Args& args, const string& nam
     return NStr::StringToNumeric<double>(value);
 }
 
-shared_ptr<CPSG_ReplyItem> CPSG_Reply::SImpl::Create(SPSG_Reply::SItem::TTS& item_ts)
+CPSG_ReplyItem* CPSG_Reply::SImpl::CreateImpl(CPSG_SkippedBlob::EReason reason, const SPSG_Args& args, shared_ptr<SPSG_Stats>& stats)
 {
-    auto user_reply_locked = user_reply.lock();
+    auto id = SDataId::Get<SDataId::eChunkIdPriority>(args);
+    auto sent_seconds_ago = s_GetSeconds(args, "sent_seconds_ago");
+    auto time_until_resend = s_GetSeconds(args, "time_until_resend");
 
-    assert(user_reply_locked);
+    if (stats) {
+        stats->IncCounter(SPSG_Stats::eSkippedBlob, reason);
 
-    auto item_locked = item_ts.GetLock();
+        if (!sent_seconds_ago.IsNull()) stats->AddTime(SPSG_Stats::eSentSecondsAgo, sent_seconds_ago);
+        if (!time_until_resend.IsNull()) stats->AddTime(SPSG_Stats::eTimeUntilResend, time_until_resend);
+    }
+
+    return new CPSG_SkippedBlob(move(id), reason, move(sent_seconds_ago), move(time_until_resend));
+}
+
+CPSG_Processor::EProgressStatus s_GetProgressStatus(const SPSG_Args& args)
+{
+    const auto& progress = args.GetValue("progress");
+
+    if (progress == "start")      return CPSG_Processor::eStart;
+    if (progress == "done")       return CPSG_Processor::eDone;
+    if (progress == "not_found")  return CPSG_Processor::eNotFound;
+    if (progress == "canceled")   return CPSG_Processor::eCanceled;
+    if (progress == "timeout")    return CPSG_Processor::eTimeout;
+    if (progress == "error")      return CPSG_Processor::eError;
+
+    // Should not happen, new server?
+    return CPSG_Processor::eUnknown;
+}
 
-    item_locked->state.SetReturned();
+CPSG_ReplyItem* CPSG_Reply::SImpl::CreateImpl(SPSG_Reply::SItem::TTS& item_ts, SPSG_Reply::SItem& item, CPSG_ReplyItem::EType type, CPSG_SkippedBlob::EReason reason)
+{
+    auto stats = reply->stats.lock();
+    if (stats) stats->IncCounter(SPSG_Stats::eReplyItem, type);
 
-    auto impl = make_unique<CPSG_ReplyItem::SImpl>(item_ts);
+    auto status = item.state.GetStatus();
+    auto& args = item.args;
+    auto& chunks = item.chunks;
 
-    shared_ptr<CPSG_ReplyItem> rv;
+    if ((status == EPSG_Status::eSuccess) || (status == EPSG_Status::eInProgress)) {
+        switch (type) {
+            case CPSG_ReplyItem::eBlobData:         return CreateImpl(item_ts, args, stats);
+            case CPSG_ReplyItem::eSkippedBlob:      return CreateImpl(reason, args, stats);
+            case CPSG_ReplyItem::eBioseqInfo:       return CreateImpl(new CPSG_BioseqInfo, chunks);
+            case CPSG_ReplyItem::eBlobInfo:         return CreateImpl(new CPSG_BlobInfo(SDataId::Get(args)), chunks);
+            case CPSG_ReplyItem::eNamedAnnotInfo:   return CreateImpl(new CPSG_NamedAnnotInfo(args.GetValue("na")), chunks);
+            case CPSG_ReplyItem::ePublicComment:    return new CPSG_PublicComment(SDataId::Get(args), chunks.empty() ? string() : chunks.front());
+            case CPSG_ReplyItem::eProcessor:        return new CPSG_Processor(s_GetProgressStatus(args));
+            case CPSG_ReplyItem::eEndOfReply:       return nullptr;
+        }
 
-    auto& chunks = item_locked->chunks;
-    auto& args = item_locked->args;
-    auto processor_id = args.GetValue("processor_id");
+        // Should not happen
+        _TROUBLE;
 
-    const auto& state = item_locked->state.GetState();
-    const auto itar = SItemTypeAndReason::Get(args);
+    } else if (type != CPSG_ReplyItem::eEndOfReply) {
+        if (stats) stats->IncCounter(SPSG_Stats::eReplyItemStatus, static_cast<size_t>(status));
+        return new CPSG_ReplyItem(type);
+    }
 
-    if ((state != SPSG_Reply::SState::eSuccess) && (state != SPSG_Reply::SState::eInProgress)) {
-        rv.reset(new CPSG_ReplyItem(itar.first));
+    return nullptr;
+}
 
-    } else if (itar.first == CPSG_ReplyItem::eBlobData) {
-        unique_ptr<CPSG_BlobData> blob_data(new CPSG_BlobData(s_GetDataId(args)));
-        blob_data->m_Stream.reset(new SPSG_RStream(item_ts));
-        rv.reset(blob_data.release());
+struct SItemTypeAndReason : pair<CPSG_ReplyItem::EType, CPSG_SkippedBlob::EReason>
+{
+    static SItemTypeAndReason Get(const SPSG_Args& args);
 
-    } else if (itar.first == CPSG_ReplyItem::eSkippedBlob) {
-        auto data_id = s_GetDataId(args);
-        auto blob_id = move(dynamic_cast<CPSG_BlobId&>(*data_id));
-        auto sent_seconds_ago = s_GetSeconds(args, "sent_seconds_ago");
-        auto time_until_resend = s_GetSeconds(args, "time_until_resend");
-        rv.reset(new CPSG_SkippedBlob(move(blob_id), itar.second, move(sent_seconds_ago), move(time_until_resend)));
+private:
+    using TBase = pair<CPSG_ReplyItem::EType, CPSG_SkippedBlob::EReason>;
+    using TBase::TBase;
+    SItemTypeAndReason(CPSG_ReplyItem::EType type) : TBase(type, CPSG_SkippedBlob::eUnknown) {}
+    static SItemTypeAndReason GetIfBlob(const SPSG_Args& args);
+};
 
-    } else if (itar.first == CPSG_ReplyItem::eBioseqInfo) {
-        rv.reset(CreateImpl(new CPSG_BioseqInfo, chunks));
+// Item type cannot be determined by "item_type" alone, for blobs "reason" has to be used as well.
+SItemTypeAndReason SItemTypeAndReason::GetIfBlob(const SPSG_Args& args)
+{
+    const auto reason = args.GetValue("reason");
 
-    } else if (itar.first == CPSG_ReplyItem::eBlobInfo) {
-        rv.reset(CreateImpl(new CPSG_BlobInfo(s_GetDataId(args)), chunks));
+    if (reason.empty()) {
+        return CPSG_ReplyItem::eBlobData;
 
-    } else if (itar.first == CPSG_ReplyItem::eNamedAnnotInfo) {
-        auto name = args.GetValue("na");
-        rv.reset(CreateImpl(new CPSG_NamedAnnotInfo(name), chunks));
+    } else if (reason == "excluded") {
+        return { CPSG_ReplyItem::eSkippedBlob, CPSG_SkippedBlob::eExcluded };
 
-    } else if (itar.first == CPSG_ReplyItem::ePublicComment) {
-        auto text = chunks.empty() ? string() : chunks.front();
-        rv.reset(new CPSG_PublicComment(s_GetDataId(args), text));
+    } else if (reason == "inprogress") {
+        return { CPSG_ReplyItem::eSkippedBlob, CPSG_SkippedBlob::eInProgress };
 
-    } else if (itar.first == CPSG_ReplyItem::eProcessor) {
-        rv.reset(new CPSG_ReplyItem(CPSG_ReplyItem::eProcessor));
+    } else if (reason == "sent") {
+        return { CPSG_ReplyItem::eSkippedBlob, CPSG_SkippedBlob::eSent };
 
     } else {
-        return rv;
+        return { CPSG_ReplyItem::eSkippedBlob, CPSG_SkippedBlob::eUnknown };
+    }
+}
+
+SItemTypeAndReason SItemTypeAndReason::Get(const SPSG_Args& args)
+{
+    const auto item_type = args.GetValue<SPSG_Args::eItemType>();
+
+    switch (item_type.first) {
+        case SPSG_Args::eBioseqInfo:     return CPSG_ReplyItem::eBioseqInfo;
+        case SPSG_Args::eBlobProp:       return CPSG_ReplyItem::eBlobInfo;
+        case SPSG_Args::eBlob:           return GetIfBlob(args);
+        case SPSG_Args::eReply:          break;
+        case SPSG_Args::eBioseqNa:       return CPSG_ReplyItem::eNamedAnnotInfo;
+        case SPSG_Args::ePublicComment:  return CPSG_ReplyItem::ePublicComment;
+        case SPSG_Args::eProcessor:      return CPSG_ReplyItem::eProcessor;
+        case SPSG_Args::eUnknownItem:    break;
+    }
+
+    if (TPSG_FailOnUnknownItems::GetDefault()) {
+        NCBI_THROW_FMT(CPSG_Exception, eServerError, "Received unknown item type: " << item_type.second.get());
+    }
+
+    static atomic_bool reported(false);
+
+    if (!reported.exchange(true)) {
+        ERR_POST("Received unknown item type: " << item_type.second.get());
+    }
+
+    return CPSG_ReplyItem::eEndOfReply;
+}
+
+shared_ptr<CPSG_ReplyItem> CPSG_Reply::SImpl::Create(SPSG_Reply::SItem::TTS& item_ts)
+{
+    auto item_locked = item_ts.GetLock();
+    item_locked->state.SetReturned();
+
+    const auto& args = item_locked->args;
+    const auto itar = SItemTypeAndReason::Get(args);
+
+    shared_ptr<CPSG_ReplyItem> rv(CreateImpl(item_ts, *item_locked, itar.first, itar.second));
+
+    if (rv) {
+        rv->m_Impl.reset(new CPSG_ReplyItem::SImpl(item_ts));
+        rv->m_Reply = user_reply.lock();
+        rv->m_ProcessorId = args.GetValue("processor_id");
+        _ASSERT(rv->m_Reply);
     }
 
-    rv->m_Impl.reset(impl.release());
-    rv->m_Reply = user_reply_locked;
-    rv->m_ProcessorId = move(processor_id);
     return rv;
 }
 
@@ -618,11 +728,10 @@ string CPSG_Queue::SImpl::x_GetAbsPathRef(shared_ptr<const CPSG_Request> user_re
 {
     static const string other_args(s_GetOtherArgs());
 
+    _ASSERT(user_request);
     ostringstream os;
     user_request->x_GetAbsPathRef(os);
 
-    if (const auto hops = user_request->m_Hops) os << "&hops=" << hops;
-
     os << other_args;
     m_UserArgsBuilder.GetLock()->Build(os, user_request->m_UserArgs);
     return os.str();
@@ -669,7 +778,6 @@ void CPSG_Request_Biodata::x_GetAbsPathRef(ostream& os) const
     }
 
     os << s_GetAccSubstitution(m_AccSubstitution);
-    os << s_GetAutoBlobSkipping(m_AutoBlobSkipping);
 
     if (m_ResendTimeout.IsInfinite()) {
         NCBI_THROW(CPSG_Exception, eParameterMissing, "Infinite resend timeout is not supported");
@@ -740,14 +848,18 @@ shared_ptr<CPSG_Reply> CPSG_Queue::SImpl::SendRequestAndGetReply(shared_ptr<CPSG
 {
     _ASSERT(queue);
 
+    if (!r) {
+        NCBI_THROW(CPSG_Exception, eParameterMissing, "request cannot be empty");
+    }
+
     auto user_request = const_pointer_cast<const CPSG_Request>(r);
     auto& ioc = m_Service.ioc;
     auto& params = ioc.params;
+    auto& stats = ioc.stats;
 
-    auto user_context = params.client_mode == EPSG_PsgClientMode::eOff ?
-        nullptr : user_request->GetUserContext<string>();
+    auto user_context = params.user_request_ids ? user_request->GetUserContext<string>() : nullptr;
     const auto request_id = user_context ? *user_context : ioc.GetNewRequestId();
-    auto reply = make_shared<SPSG_Reply>(move(request_id), params, queue);
+    auto reply = make_shared<SPSG_Reply>(move(request_id), params, queue, stats);
     auto abs_path_ref = x_GetAbsPathRef(user_request);
     const auto& request_context = user_request->m_RequestContext;
 
@@ -756,6 +868,7 @@ shared_ptr<CPSG_Reply> CPSG_Queue::SImpl::SendRequestAndGetReply(shared_ptr<CPSG
     auto request = make_shared<SPSG_Request>(move(abs_path_ref), reply, request_context->Clone(), params);
 
     if (ioc.AddRequest(request, queue->Stopped(), deadline)) {
+        if (stats) stats->IncCounter(SPSG_Stats::eRequest, user_request->GetType());
         shared_ptr<CPSG_Reply> user_reply(new CPSG_Reply);
         user_reply->m_Impl->reply = move(reply);
         user_reply->m_Impl->user_reply = user_reply;
@@ -795,12 +908,10 @@ EPSG_Status s_GetStatus(SPSG_Reply::SItem::TTS& ts, const CDeadline& deadline)
     auto& state = ts->state;
 
     do {
-        switch (state.GetState()) {
-            case SPSG_Reply::SState::eNotFound:   return EPSG_Status::eNotFound;
-            case SPSG_Reply::SState::eForbidden:  return EPSG_Status::eForbidden;
-            case SPSG_Reply::SState::eError:      return EPSG_Status::eError;
-            case SPSG_Reply::SState::eSuccess:    return EPSG_Status::eSuccess;
-            case SPSG_Reply::SState::eInProgress: break;
+        auto status = state.GetStatus();
+
+        if (status != EPSG_Status::eInProgress) {
+            return status;
         }
     }
     while (state.change.WaitUntil(deadline));
@@ -936,9 +1047,9 @@ Uint8 CPSG_BlobInfo::GetNChunks() const
 }
 
 
-CPSG_SkippedBlob::CPSG_SkippedBlob(CPSG_BlobId id, EReason reason, TSeconds sent_seconds_ago, TSeconds time_until_resend) :
+CPSG_SkippedBlob::CPSG_SkippedBlob(unique_ptr<CPSG_DataId> id, EReason reason, TSeconds sent_seconds_ago, TSeconds time_until_resend) :
     CPSG_ReplyItem(eSkippedBlob),
-    m_Id(id),
+    m_Id(move(id)),
     m_Reason(reason),
     m_SentSecondsAgo(move(sent_seconds_ago)),
     m_TimeUntilResend(move(time_until_resend))
@@ -982,7 +1093,7 @@ vector<CPSG_BioId> CPSG_BioseqInfo::GetOtherIds() const
         _ASSERT(request);
 
         NCBI_THROW_FMT(CPSG_Exception, eServerError, "Wrong seq_ids format: '" << seq_ids.Repr() <<
-                "' for " << request->GetType() << " request '" << request->GetId() << '\'');
+                "' for " << s_GetRequestTypeName(request->GetType()) << " request '" << request->GetId() << '\'');
     }
 
     return rv;
@@ -1101,6 +1212,13 @@ CPSG_PublicComment::CPSG_PublicComment(unique_ptr<CPSG_DataId> id, string text)
 }
 
 
+CPSG_Processor::CPSG_Processor(EProgressStatus progress_status) :
+    CPSG_ReplyItem(eProcessor),
+    m_ProgressStatus(progress_status)
+{
+}
+
+
 CPSG_Reply::CPSG_Reply() :
     m_Impl(new SImpl)
 {
