@@ -1,4 +1,4 @@
-/*  $Id: psg_client_transport.cpp 640535 2021-11-12 18:57:53Z sadyrovr $
+/*  $Id: psg_client_transport.cpp 654901 2022-08-24 11:37:46Z fukanchi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -59,6 +59,7 @@ BEGIN_NCBI_SCOPE
 NCBI_PARAM_DEF(unsigned, PSG, rd_buf_size,            64 * 1024);
 NCBI_PARAM_DEF(size_t,   PSG, wr_buf_size,            64 * 1024);
 NCBI_PARAM_DEF(unsigned, PSG, max_concurrent_streams, 200);
+NCBI_PARAM_DEF(unsigned, PSG, max_concurrent_submits, 150);
 NCBI_PARAM_DEF(unsigned, PSG, max_sessions,           40);
 NCBI_PARAM_DEF(unsigned, PSG, num_io,                 6);
 NCBI_PARAM_DEF(unsigned, PSG, reader_timeout,         12);
@@ -66,11 +67,16 @@ NCBI_PARAM_DEF(double,   PSG, rebalance_time,         10.0);
 NCBI_PARAM_DEF(unsigned, PSG, request_timeout,        10);
 NCBI_PARAM_DEF(size_t, PSG, requests_per_io,          1);
 NCBI_PARAM_DEF(unsigned, PSG, request_retries,        2);
+NCBI_PARAM_DEF(unsigned, PSG, refused_stream_retries, 2);
 NCBI_PARAM_DEF(string,   PSG, request_user_args,      "");
+NCBI_PARAM_DEF(bool,     PSG, user_request_ids,       false);
 NCBI_PARAM_DEF(unsigned, PSG, localhost_preference,   1);
 NCBI_PARAM_DEF(bool,     PSG, fail_on_unknown_items,  false);
+NCBI_PARAM_DEF(bool,     PSG, fail_on_unknown_chunks, false);
 NCBI_PARAM_DEF(bool,     PSG, https,                  false);
 NCBI_PARAM_DEF(double,   PSG, no_servers_retry_delay, 1.0);
+NCBI_PARAM_DEF(bool,     PSG, stats,                  false);
+NCBI_PARAM_DEF(double,   PSG, stats_period,           0.0);
 
 NCBI_PARAM_DEF(double,   PSG, throttle_relaxation_period,                  0.0);
 NCBI_PARAM_DEF(unsigned, PSG, throttle_by_consecutive_connection_failures, 0);
@@ -97,12 +103,33 @@ NCBI_PARAM_ENUM_DEF(EPSG_UseCache, PSG, use_cache, EPSG_UseCache::eDefault);
 NCBI_PARAM_ENUM_ARRAY(EPSG_PsgClientMode, PSG, internal_psg_client_mode)
 {
     { "off",         EPSG_PsgClientMode::eOff         },
-    { "interactive", EPSG_PsgClientMode::eInteractive },
     { "performance", EPSG_PsgClientMode::ePerformance },
-    { "io",          EPSG_PsgClientMode::eIo          }
 };
 NCBI_PARAM_ENUM_DEF(EPSG_PsgClientMode, PSG, internal_psg_client_mode, EPSG_PsgClientMode::eOff);
 
+SPSG_ArgsBase::SArg<SPSG_ArgsBase::eItemType>::TType SPSG_ArgsBase::SArg<SPSG_ArgsBase::eItemType>::Get(const string& value)
+{
+    if (value == "bioseq_info")     return { SPSG_ArgsBase::eBioseqInfo,     cref(value) };
+    if (value == "blob_prop")       return { SPSG_ArgsBase::eBlobProp,       cref(value) };
+    if (value == "blob")            return { SPSG_ArgsBase::eBlob,           cref(value) };
+    if (value == "reply")           return { SPSG_ArgsBase::eReply,          cref(value) };
+    if (value == "bioseq_na")       return { SPSG_ArgsBase::eBioseqNa,       cref(value) };
+    if (value == "public_comment")  return { SPSG_ArgsBase::ePublicComment,  cref(value) };
+    if (value == "processor")       return { SPSG_ArgsBase::eProcessor,      cref(value) };
+    if (value.empty())              return { SPSG_ArgsBase::eReply,          cref(value) };
+    return { SPSG_ArgsBase::eUnknownItem, cref(value) };
+};
+
+SPSG_ArgsBase::SArg<SPSG_ArgsBase::eChunkType>::TType SPSG_ArgsBase::SArg<SPSG_ArgsBase::eChunkType>::Get(const string& value)
+{
+    if (value == "meta")              return { SPSG_ArgsBase::eMeta,            cref(value) };
+    if (value == "data")              return { SPSG_ArgsBase::eData,            cref(value) };
+    if (value == "message")           return { SPSG_ArgsBase::eMessage,         cref(value) };
+    if (value == "data_and_meta")     return { SPSG_ArgsBase::eDataAndMeta,     cref(value) };
+    if (value == "message_and_meta")  return { SPSG_ArgsBase::eMessageAndMeta,  cref(value) };
+    return { SPSG_ArgsBase::eUnknownChunk, cref(value) };
+};
+
 struct SContextSetter
 {
     SContextSetter(CRequestContext* context) { CDiagContext::SetRequestContext(context); }
@@ -128,7 +155,7 @@ void SDebugPrintout::Print(const SPSG_Args& args, const SPSG_Chunk& chunk)
     os << args.GetQueryString(CUrlArgs::eAmp_Char) << '\n';
 
     if ((m_Params.debug_printout == EPSG_DebugPrintout::eAll) ||
-            (args.GetValue("item_type") != "blob") || (args.GetValue("chunk_type") != "data")) {
+            (args.GetValue<SPSG_Args::eItemType>().first != SPSG_Args::eBlob) || (args.GetValue<SPSG_Args::eChunkType>().first != SPSG_Args::eData)) {
         os << chunk;
     } else {
         os << "<BINARY DATA OF " << chunk.size() << " BYTES>";
@@ -164,13 +191,343 @@ SDebugPrintout::~SDebugPrintout()
             os << fixed << id << '\t' << ms << '\t' << type << '\t' << thread_id << '\n';
         }
 
-        static mutex cout_mutex;
-        lock_guard<mutex> lock(cout_mutex);
-        cout << os.str();
-        cout.flush();
+        cout << os.str() << flush;
     }
 }
 
+template <>
+struct SPSG_StatsCounters::SGroup<SPSG_StatsCounters::eRequest>
+{
+    using type = CPSG_Request::EType;
+    static constexpr size_t size = CPSG_Request::eChunk + 1;
+    static constexpr auto prefix = "\trequest\ttype=";
+
+    static constexpr array<type, size> values = {
+        CPSG_Request::eBiodata,
+        CPSG_Request::eResolve,
+        CPSG_Request::eBlob,
+        CPSG_Request::eNamedAnnotInfo,
+        CPSG_Request::eChunk,
+    };
+
+    static const char* ValueName(type value)
+    {
+        switch (value) {
+            case CPSG_Request::eBiodata:            return "biodata";
+            case CPSG_Request::eResolve:            return "resolve";
+            case CPSG_Request::eBlob:               return "blob";
+            case CPSG_Request::eNamedAnnotInfo:     return "named_annot_info";
+            case CPSG_Request::eChunk:              return "chunk";
+        }
+
+        // Should not happen
+        _TROUBLE;
+        return "unknown";
+    }
+};
+
+template <>
+struct SPSG_StatsCounters::SGroup<SPSG_StatsCounters::eReplyItem>
+{
+    using type = CPSG_ReplyItem::EType;
+    static constexpr size_t size = CPSG_ReplyItem::eEndOfReply + 1;
+    static constexpr auto prefix = "\treply_item\ttype=";
+
+    static constexpr array<type, size> values = {
+        CPSG_ReplyItem::eBlobData,
+        CPSG_ReplyItem::eBlobInfo,
+        CPSG_ReplyItem::eSkippedBlob,
+        CPSG_ReplyItem::eBioseqInfo,
+        CPSG_ReplyItem::eNamedAnnotInfo,
+        CPSG_ReplyItem::ePublicComment,
+        CPSG_ReplyItem::eProcessor,
+        CPSG_ReplyItem::eEndOfReply,
+    };
+
+    static const char* ValueName(type value)
+    {
+        switch (value) {
+            case CPSG_ReplyItem::eBlobData:             return "blob_data";
+            case CPSG_ReplyItem::eBlobInfo:             return "blob_info";
+            case CPSG_ReplyItem::eSkippedBlob:          return "skipped_blob";
+            case CPSG_ReplyItem::eBioseqInfo:           return "bioseq_info";
+            case CPSG_ReplyItem::eNamedAnnotInfo:       return "named_annot_info";
+            case CPSG_ReplyItem::ePublicComment:        return "public_comment";
+            case CPSG_ReplyItem::eProcessor:            return "processor";
+            case CPSG_ReplyItem::eEndOfReply:           return "end_of_reply";
+        }
+
+        // Should not happen
+        _TROUBLE;
+        return "unknown";
+    }
+};
+
+template <>
+struct SPSG_StatsCounters::SGroup<SPSG_StatsCounters::eSkippedBlob>
+{
+    using type = CPSG_SkippedBlob::EReason;
+    static constexpr size_t size = CPSG_SkippedBlob::eUnknown + 1;
+    static constexpr auto prefix = "\tskipped_blob\treason=";
+
+    static constexpr array<type, size> values = {
+        CPSG_SkippedBlob::eExcluded,
+        CPSG_SkippedBlob::eInProgress,
+        CPSG_SkippedBlob::eSent,
+        CPSG_SkippedBlob::eUnknown,
+    };
+
+    static const char* ValueName(type value)
+    {
+        switch (value) {
+            case CPSG_SkippedBlob::eExcluded:       return "excluded";
+            case CPSG_SkippedBlob::eInProgress:     return "in_progress";
+            case CPSG_SkippedBlob::eSent:           return "sent";
+            case CPSG_SkippedBlob::eUnknown:        return "unknown";
+        }
+
+        // Should not happen
+        _TROUBLE;
+        return "unknown";
+    }
+};
+
+template <>
+struct SPSG_StatsCounters::SGroup<SPSG_StatsCounters::eReplyItemStatus>
+{
+    using type = EPSG_Status;
+    static constexpr size_t size = static_cast<size_t>(EPSG_Status::eError) + 1;
+    static constexpr auto prefix = "\treply_item_status\tstatus=";
+
+    static constexpr array<type, size> values = {
+        EPSG_Status::eSuccess,
+        EPSG_Status::eInProgress,
+        EPSG_Status::eNotFound,
+        EPSG_Status::eCanceled,
+        EPSG_Status::eForbidden,
+        EPSG_Status::eError,
+    };
+
+    static const char* ValueName(type value)
+    {
+        switch (value) {
+            case EPSG_Status::eSuccess:         return "success";
+            case EPSG_Status::eInProgress:      return "in_progress";
+            case EPSG_Status::eNotFound:        return "not_found";
+            case EPSG_Status::eCanceled:        return "canceled";
+            case EPSG_Status::eForbidden:       return "forbidden";
+            case EPSG_Status::eError:           return "error";
+        }
+
+        // Should not happen
+        _TROUBLE;
+        return "unknown";
+    }
+};
+
+template <>
+struct SPSG_StatsCounters::SGroup<SPSG_StatsCounters::eMessage>
+{
+    using type = EDiagSev;
+    static constexpr size_t size = EDiagSev::eDiag_Trace + 1;
+    static constexpr auto prefix = "\tmessage\tseverity=";
+
+    static constexpr array<type, size> values = {
+        EDiagSev::eDiag_Info,
+        EDiagSev::eDiag_Warning,
+        EDiagSev::eDiag_Error,
+        EDiagSev::eDiag_Critical,
+        EDiagSev::eDiag_Fatal,
+        EDiagSev::eDiag_Trace,
+    };
+
+    static const char* ValueName(type value)
+    {
+        switch (value) {
+            case EDiagSev::eDiag_Info:          return "info";
+            case EDiagSev::eDiag_Warning:       return "warning";
+            case EDiagSev::eDiag_Error:         return "error";
+            case EDiagSev::eDiag_Critical:      return "critical";
+            case EDiagSev::eDiag_Fatal:         return "fatal";
+            case EDiagSev::eDiag_Trace:         return "trace";
+        }
+
+        // Should not happen
+        _TROUBLE;
+        return "unknown";
+    }
+};
+
+template <SPSG_Stats::EGroup group>
+void SPSG_StatsCounters::SInit::Func(TData& data)
+{
+    data.emplace_back(SGroup<group>::size);
+
+    for (auto& counter : data.back()) {
+        counter = 0;
+    }
+};
+
+template <SPSG_Stats::EGroup group>
+void SPSG_StatsCounters::SReport::Func(const TData& data, const char* prefix, unsigned report)
+{
+    using TGroup = SGroup<group>;
+
+    _ASSERT(data.size() > group);
+    const auto& g = data[group];
+    _ASSERT(g.size() == TGroup::size);
+
+    for (auto i : TGroup::values) {
+        auto n = g[static_cast<size_t>(i)].load();
+        if (n) ERR_POST(Note << prefix << report << TGroup::prefix << TGroup::ValueName(i) << "&count=" << n);
+    }
+}
+
+SPSG_StatsCounters::SPSG_StatsCounters()
+{
+    Apply<SInit>(eRequest, m_Data);
+}
+
+template <class TWhat, class... TArgs>
+void SPSG_StatsCounters::Apply(EGroup start_with, TArgs&&... args)
+{
+    // This method is always called with start_with == eRequest (so, all cases are run with one call, one by one).
+    // This switch usage however makes compilers warn if any enum value is missing/not handled
+    switch (start_with) {
+        case eRequest:          TWhat::template Func<eRequest>          (forward<TArgs>(args)...);
+        case eReplyItem:        TWhat::template Func<eReplyItem>        (forward<TArgs>(args)...);
+        case eSkippedBlob:      TWhat::template Func<eSkippedBlob>      (forward<TArgs>(args)...);
+        case eReplyItemStatus:  TWhat::template Func<eReplyItemStatus>  (forward<TArgs>(args)...);
+        case eMessage:          TWhat::template Func<eMessage>          (forward<TArgs>(args)...);
+    }
+}
+
+template <class... TArgs>
+void SPSG_StatsCounters::Report(TArgs&&... args)
+{
+    Apply<SReport>(eRequest, m_Data, forward<TArgs>(args)...);
+}
+
+SPSG_StatsAvgTime::SPSG_StatsAvgTime() :
+    m_Data(eTimeUntilResend + 1)
+{
+}
+
+const char* SPSG_StatsAvgTime::GetName(EAvgTime avg_time)
+{
+    switch (avg_time) {
+        case SPSG_Stats::eSentSecondsAgo:   return "sent_seconds_ago";
+        case SPSG_Stats::eTimeUntilResend:  return "time_until_resend";
+    }
+
+    // Should not happen
+    _TROUBLE;
+    return "unknown";
+}
+
+void SPSG_StatsAvgTime::Report(const char* prefix, unsigned report)
+{
+    for (auto i : { eSentSecondsAgo, eTimeUntilResend }) {
+        _ASSERT(m_Data.size() > i);
+        const auto& data = m_Data[i];
+        auto v = data.first.load();
+        auto n = data.second.load();
+        if (n) ERR_POST(Note << prefix << report << '\t' << GetName(i) << "\taverage=" << double(v / n) / milli::den);
+    }
+}
+
+template <class TDataId>
+void SPSG_StatsData::SData<TDataId>::Report(const char* prefix, unsigned report, const char* data_prefix)
+{
+    struct SLess {
+        static auto Tuple(const CPSG_BlobId& id)  { return                       tie(id.GetId(),       id.GetLastModified()); }
+        static auto Tuple(const CPSG_ChunkId& id) { return tuple<int, const string&>(id.GetId2Chunk(), id.GetId2Info());      }
+        bool operator()(const TDataId& lhs, const TDataId& rhs) const { return Tuple(lhs) < Tuple(rhs); }
+    };
+
+    size_t total = 0;
+    map<TDataId, unsigned, SLess> unique_ids;
+
+    if (auto locked = m_Ids.GetLock()) {
+        total = locked->size();
+
+        if (!total) return;
+
+        for (const auto& data_id : *locked) {
+            auto created = unique_ids.emplace(data_id, 1);
+            if (!created.second) ++created.first->second;
+        }
+    }
+
+    ERR_POST(Note << prefix << report << data_prefix << "\ttotal=" << total << "&unique=" << unique_ids.size());
+
+    auto received = m_Received.load();
+    auto read = m_Read.load();
+    if (received) ERR_POST(Note << prefix << report << data_prefix << "_data\treceived=" << received << "&read=" << read);
+
+    map<unsigned, unsigned> group_by_count;
+
+    for (const auto& p : unique_ids) {
+        auto created = group_by_count.emplace(p.second, 1);
+        if (!created.second) ++created.first->second;
+    }
+
+    for (const auto& p : group_by_count) {
+        ERR_POST(Note << prefix << report << data_prefix << "_retrievals\tnumber=" << p.first << "&unique_ids=" << p.second);
+    }
+}
+
+void SPSG_StatsData::Report(const char* prefix, unsigned report)
+{
+    m_Blobs.Report(prefix, report, "\tblob");
+    m_Chunks.Report(prefix, report, "\tchunk");
+    if (auto n = m_TSEs.GetLock()->size()) ERR_POST(Note << prefix << report << "\tchunk_tse\tunique=" << n);
+}
+
+uint64_t s_GetStatsPeriod()
+{
+    return SecondsToMs(TPSG_StatsPeriod::GetDefault());
+}
+
+SPSG_Stats::SPSG_Stats(SPSG_Servers::TTS& servers) :
+    m_Timer(this, s_OnTimer, s_GetStatsPeriod(), s_GetStatsPeriod()),
+    m_Report(0),
+    m_Servers(servers)
+{
+}
+
+void SPSG_Stats::Report()
+{
+    const auto prefix = "PSG_STATS\t";
+    const auto report = ++m_Report;
+
+    SPSG_StatsCounters::Report(prefix, report);
+    SPSG_StatsAvgTime::Report(prefix, report);
+    SPSG_StatsData::Report(prefix, report);
+
+    auto servers_locked = m_Servers.GetLock();
+
+    for (const auto& server : *servers_locked) {
+        auto n = server.stats.load();
+        if (n) ERR_POST(Note << prefix << report << "\tserver\tname=" << server.address.AsString() << "&requests_sent=" << n);
+    }
+}
+
+EPSG_Status SPSG_Reply::SState::GetStatus() const volatile
+{
+    switch (m_State) {
+        case eNotFound:     return EPSG_Status::eNotFound;
+        case eForbidden:    return EPSG_Status::eForbidden;
+        case eError:        return EPSG_Status::eError;
+        case eSuccess:      return EPSG_Status::eSuccess;
+        case eInProgress:   return EPSG_Status::eInProgress;
+    }
+
+    // Should not happen
+    _TROUBLE;
+    return EPSG_Status::eError;
+}
+
 string SPSG_Reply::SState::GetError()
 {
     if (m_Messages.empty()) return {};
@@ -243,13 +600,10 @@ SPSG_Request::SPSG_Request(string p, shared_ptr<SPSG_Reply> r, CRef<CRequestCont
     full_path(move(p)),
     reply(r),
     context(c),
-    m_State(params.client_mode == EPSG_PsgClientMode::eIo ?
-            &SPSG_Request::StateIo : &SPSG_Request::StatePrefix),
-    m_Retries(params.request_retries)
+    m_State(&SPSG_Request::StatePrefix),
+    m_Retries(params.request_retries, params.refused_stream_retries)
 {
     _ASSERT(reply);
-
-    if (params.client_mode == EPSG_PsgClientMode::eIo) AddIo();
 }
 
 bool SPSG_Request::StatePrefix(const char*& data, size_t& len)
@@ -257,7 +611,7 @@ bool SPSG_Request::StatePrefix(const char*& data, size_t& len)
     static const string kPrefix = "\n\nPSG-Reply-Chunk: ";
 
     // No retries after receiving any data
-    m_Retries = 0;
+    m_Retries = make_pair(0u, 0u);
 
     auto& index = m_Buffer.prefix_index;
 
@@ -276,9 +630,11 @@ bool SPSG_Request::StatePrefix(const char*& data, size_t& len)
     }
 
     // Check failed
-    const auto remaining = min(len, kPrefix.size() - index);
-    const auto wrong_prefix(NStr::PrintableString(string(data, remaining)));
-    ERR_POST("Prefix mismatch: " << (index ? "offending part " : "") << '\'' << wrong_prefix << '\'');
+    const auto matched = CTempString(kPrefix, 0, index);
+    const auto different = CTempString(data, min(len, kPrefix.size() - index));
+    stringstream ss;
+    ss << "Protocol error: prefix mismatch, expected '" << kPrefix << "' vs received '" << matched << different << '\'';
+    reply->reply_item.GetLock()->state.AddError(ss.str());
     return false;
 }
 
@@ -295,12 +651,13 @@ bool SPSG_Request::StateArgs(const char*& data, size_t& len)
 
     SPSG_Args args(m_Buffer.args_buffer);
 
-    auto size = args.GetValue("size");
+    const auto& size_str = args.GetValue("size");
+    const auto size = size_str.empty() ? 0ul : stoul(size_str);
 
     m_Buffer.args = move(args);
 
-    if (!size.empty()) {
-        SetStateData(stoul(size));
+    if (size) {
+        SetStateData(size);
     } else {
         SetStatePrefix();
     }
@@ -329,57 +686,40 @@ bool SPSG_Request::StateData(const char*& data, size_t& len)
     return true;
 }
 
-void SPSG_Request::AddIo()
+EDiagSev s_GetSeverity(const string& severity)
 {
-    SPSG_Chunk chunk(1, ' ');
+    if (severity == "error")        return eDiag_Error;
+    if (severity == "warning")      return eDiag_Warning;
+    if (severity == "info")         return eDiag_Info;
+    if (severity == "trace")        return eDiag_Trace;
+    if (severity == "fatal")        return eDiag_Fatal;
+    if (severity == "critical")     return eDiag_Critical;
 
-    SPSG_Reply::SItem::TTS* item_ts = nullptr;
-    auto reply_item_ts = &reply->reply_item;
-
-    if (auto items_locked = reply->items.GetLock()) {
-        auto& items = *items_locked;
-        items.emplace_back();
-        item_ts = &items.back();
-    }
-
-    if (auto item_locked = item_ts->GetLock()) {
-        auto& item = *item_locked;
-        item.chunks.push_back(move(chunk));
-        item.args = SPSG_Args("item_id=1&item_type=blob&chunk_type=meta&blob_id=0&n_chunks=2");
-        item.received = item.expected = 2;
-        item.state.SetNotEmpty();
-    }
-
-    if (auto item_locked = reply_item_ts->GetLock()) {
-        auto& item = *item_locked;
-        item.args = SPSG_Args("item_id=0&item_type=reply&chunk_type=meta&n_chunks=3");
-        item.received = item.expected = 3;
-    }
-
-    reply_item_ts->NotifyOne();
-    item_ts->NotifyOne();
-    reply->queue->CV().NotifyOne();
+    // Should not happen
+    _TROUBLE;
+    return eDiag_Error;
 }
 
 void SPSG_Request::Add()
 {
     SContextSetter setter(context);
 
-    reply->debug_printout << m_Buffer.args << m_Buffer.chunk << endl;
+    auto& args = m_Buffer.args;
+    reply->debug_printout << args << m_Buffer.chunk << endl;
 
-    auto& chunk = m_Buffer.chunk;
-    auto* args = &m_Buffer.args;
+    const auto item_type = args.GetValue<SPSG_Args::eItemType>().first;
+    auto& reply_item_ts = reply->reply_item;
 
-    auto item_type = args->GetValue("item_type");
-    SPSG_Reply::SItem::TTS* item_ts = nullptr;
+    if (item_type == SPSG_Args::eReply) {
+        if (auto item_locked = reply_item_ts.GetLock()) {
+            UpdateItem(item_type, *item_locked, args);
+        }
 
-    const bool is_reply = item_type.empty() || (item_type == "reply");
-
-    if (is_reply) {
-        item_ts = &reply->reply_item;
+        // Item must be unlocked before notifying
+        reply_item_ts.NotifyOne();
 
     } else {
-        if (auto reply_item_locked = reply->reply_item.GetLock()) {
+        if (auto reply_item_locked = reply_item_ts.GetLock()) {
             auto& reply_item = *reply_item_locked;
             ++reply_item.received;
 
@@ -388,106 +728,126 @@ void SPSG_Request::Add()
             }
         }
 
-        auto item_id = args->GetValue("item_id");
+        auto item_id = args.GetValue("item_id");
         auto& item_by_id = m_ItemsByID[item_id];
 
-        if (!item_by_id) {
+        if (item_by_id) {
+            auto item_locked = item_by_id->GetLock();
+            UpdateItem(item_type, *item_locked, args);
+
+        } else {
             if (auto items_locked = reply->items.GetLock()) {
-                auto& items = *items_locked;
-                items.emplace_back();
-                item_by_id = &items.back();
+                items_locked->emplace_back();
+                item_by_id = &items_locked->back();
             }
 
             if (auto item_locked = item_by_id->GetLock()) {
-                auto& item = *item_locked;
-                item.args = move(*args);
-                args = &item.args;
+                item_locked->args = move(args);
+                UpdateItem(item_type, *item_locked, item_locked->args);
             }
 
-            auto reply_item_ts = &reply->reply_item;
-            reply_item_ts->NotifyOne();
+            reply_item_ts.NotifyOne();
         }
 
-        item_ts = item_by_id;
+        // Item must be unlocked before notifying
+        item_by_id->NotifyOne();
     }
 
-    if (auto item_locked = item_ts->GetLock()) {
-        auto& item = *item_locked;
-        ++item.received;
-
-        auto chunk_type = args->GetValue("chunk_type");
-
-        if (chunk_type == "meta") {
-            auto n_chunks = args->GetValue("n_chunks");
-
-            if (!n_chunks.empty()) {
-                auto expected = stoul(n_chunks);
-
-                if (item.expected.Cmp<not_equal_to>(expected)) {
-                    item.state.AddError("Protocol error: contradicting n_chunks");
-                } else {
-                    item.expected = expected;
-                }
-            }
-
-        } else if (chunk_type == "message") {
-            auto severity = args->GetValue("severity");
-
-            if (severity == "warning") {
-                ERR_POST(Warning << chunk);
-            } else if (severity == "info") {
-                ERR_POST(Info << chunk);
-            } else if (severity == "trace") {
-                ERR_POST(Trace << chunk);
-            } else {
-                item.state.AddError(move(chunk));
-                const auto status = NStr::StringToInt(args->GetValue("status"));
-                const auto state = SPSG_Reply::SState::FromRequestStatus(status);
-
-                switch (state) {
-                    case SPSG_Reply::SState::eSuccess:
-                    case SPSG_Reply::SState::eError:
-                        break;
-                    default:
-                        // Any message leads to eError when item completes, so need to set other states early
-                        item.state.SetState(state);
-                }
-            }
-
-        } else if (chunk_type == "data") {
-            auto blob_chunk = args->GetValue("blob_chunk");
-            auto index = blob_chunk.empty() ? 0 : stoul(blob_chunk);
-
-            if (item.chunks.size() <= index) item.chunks.resize(index + 1);
-
-            item.chunks[index] = move(chunk);
-            item.state.SetNotEmpty();
-
-        } else {
-            item.state.AddError("Protocol error: unknown chunk type");
-        }
-
-        if (item.expected.Cmp<less>(item.received)) {
-            item.state.AddError("Protocol error: received more than expected");
-
-            // If item is not a reply itself, add the error to its reply as well
-            if (!is_reply) {
-                reply->reply_item.GetLock()->state.AddError("Protocol error: received more than expected");
-            }
-
-        // Set item complete if received everything. Reply is set complete when stream closes
-        } else if (!is_reply && item.expected.Cmp<equal_to>(item.received)) {
-            item.state.SetComplete();
-        }
-    }
-
-    // Item must be unlocked before notifying
-    item_ts->NotifyOne();
-
     reply->queue->CV().NotifyOne();
     m_Buffer = SBuffer();
 }
 
+void SPSG_Request::UpdateItem(SPSG_Args::EItemType item_type, SPSG_Reply::SItem& item, const SPSG_Args& args)
+{
+    ++item.received;
+
+    auto chunk_type = args.GetValue<SPSG_Args::eChunkType>();
+    auto& chunk = m_Buffer.chunk;
+
+    if (chunk_type.first & SPSG_Args::eMeta) {
+        auto n_chunks = args.GetValue("n_chunks");
+
+        if (!n_chunks.empty()) {
+            auto expected = stoul(n_chunks);
+
+            if (item.expected.Cmp<not_equal_to>(expected)) {
+                item.state.AddError("Protocol error: contradicting n_chunks");
+            } else {
+                item.expected = expected;
+            }
+        }
+
+    } else if (chunk_type.first == SPSG_Args::eUnknownChunk) {
+        static atomic_bool reported(false);
+
+        if (!reported.exchange(true)) {
+            ERR_POST("Received unknown chunk type: " << chunk_type.second.get());
+        }
+
+        if (TPSG_FailOnUnknownChunks::GetDefault()) {
+            item.state.AddError("Protocol error: unknown chunk type '" + chunk_type.second + '\'');
+        }
+    }
+
+    if (chunk_type.first & SPSG_Args::eMessage) {
+        auto severity = s_GetSeverity(args.GetValue("severity"));
+
+        if (severity == eDiag_Warning) {
+            ERR_POST(Warning << chunk);
+        } else if (severity == eDiag_Info) {
+            ERR_POST(Info << chunk);
+        } else if (severity == eDiag_Trace) {
+            ERR_POST(Trace << chunk);
+        } else {
+            item.state.AddError(move(chunk));
+            const auto status = NStr::StringToInt(args.GetValue("status"));
+            const auto state = SPSG_Reply::SState::FromRequestStatus(status);
+
+            switch (state) {
+                case SPSG_Reply::SState::eSuccess:
+                case SPSG_Reply::SState::eError:
+                    break;
+                default:
+                    // Any message leads to eError when item completes, so need to set other states early
+                    item.state.SetState(state);
+            }
+        }
+
+        if (auto stats = reply->stats.lock()) stats->IncCounter(SPSG_Stats::eMessage, severity);
+
+    } else if (chunk_type.first & SPSG_Args::eData) {
+        if (auto stats = reply->stats.lock()) {
+            if (item_type == SPSG_Args::eBlob) {
+                auto has_blob_id = !args.GetValue<SPSG_Args::eBlobId>().get().empty();
+                stats->AddData(has_blob_id, SPSG_Stats::eReceived, chunk.size());
+            }
+        }
+
+        auto blob_chunk = args.GetValue("blob_chunk");
+        auto index = blob_chunk.empty() ? 0 : stoul(blob_chunk);
+
+        if (item.chunks.size() <= index) item.chunks.resize(index + 1);
+
+        item.chunks[index] = move(chunk);
+        item.state.SetNotEmpty();
+    }
+
+    const bool is_not_reply = item_type != SPSG_Args::eReply;
+
+    if (item.expected.Cmp<less>(item.received)) {
+        item.state.AddError("Protocol error: received more than expected");
+
+        // If item is not a reply itself, add the error to its reply as well
+        if (is_not_reply) {
+            reply->reply_item.GetLock()->state.AddError("Protocol error: received more than expected");
+        }
+
+    // Set item complete if received everything. Reply is set complete when stream closes
+    } else if (is_not_reply && item.expected.Cmp<equal_to>(item.received)) {
+        item.state.SetComplete();
+    }
+}
+
 
 #define HTTP_STATUS_HEADER ":status"
 
@@ -526,21 +886,19 @@ int SPSG_IoSession::OnData(nghttp2_session*, uint8_t, int32_t stream_id, const u
     auto it = m_Requests.find(stream_id);
 
     if (it != m_Requests.end()) {
-        if (!it->second->OnReplyData((const char*)data, len)) {
-            Reset("Failed to parse response data");
-        }
+        it->second->OnReplyData((const char*)data, len);
     }
 
     return 0;
 }
 
-bool SPSG_IoSession::Retry(shared_ptr<SPSG_Request> req, const SUvNgHttp2_Error& error)
+bool SPSG_IoSession::Retry(shared_ptr<SPSG_Request> req, const SUvNgHttp2_Error& error, bool refused_stream)
 {
     SContextSetter setter(req->context);
     server.throttling.AddFailure();
 
     auto& debug_printout = req->reply->debug_printout;
-    auto retries = req->GetRetries();
+    auto retries = req->GetRetries(refused_stream);
 
     if (retries) {
         // Return to queue for a re-send
@@ -554,7 +912,7 @@ bool SPSG_IoSession::Retry(shared_ptr<SPSG_Request> req, const SUvNgHttp2_Error&
     req->reply->SetFailed(error);
     _DEBUG_ARG(const auto& server_name = server.address.AsString());
     PSG_THROTTLING_TRACE("Server '" << server_name << "' failed to process request '" <<
-            debug_printout.id << '\'');
+            debug_printout.id << "', " << error);
     return false;
 }
 
@@ -574,8 +932,8 @@ int SPSG_IoSession::OnStreamClose(nghttp2_session*, int32_t stream_id, uint32_t
         if (error_code) {
             auto error(SUvNgHttp2_Error::FromNgHttp2(error_code, "on close"));
 
-            if (!Retry(req, error)) {
-                ERR_POST("Request failed with " << error);
+            if (!Retry(req, error, error_code == NGHTTP2_REFUSED_STREAM)) {
+                ERR_POST("Request for " << server.address.AsString() << " failed with " << error);
             }
         } else {
             req->reply->SetComplete();
@@ -694,7 +1052,7 @@ void SPSG_IoSession::OnReset(SUvNgHttp2_Error error)
     }
 
     if (some_requests_failed) {
-        ERR_POST("Some requests failed with " << error);
+        ERR_POST("Some requests for " << server.address.AsString() << " failed with " << error);
     }
 
     m_Requests.clear();
@@ -725,13 +1083,8 @@ SPSG_ThrottleParams::SThreshold::SThreshold(string error_rate)
     }
 }
 
-uint64_t s_SecondsToMs(double seconds)
-{
-    return seconds > 0.0 ? static_cast<uint64_t>(seconds * milli::den) : 0;
-}
-
 SPSG_ThrottleParams::SPSG_ThrottleParams() :
-    period(s_SecondsToMs(TPSG_ThrottlePeriod::GetDefault())),
+    period(SecondsToMs(TPSG_ThrottlePeriod::GetDefault())),
     max_failures(TPSG_ThrottleMaxFailures::eGetDefault),
     until_discovery(TPSG_ThrottleUntilDiscovery::eGetDefault),
     threshold(TPSG_ThrottleThreshold::GetDefault())
@@ -822,7 +1175,7 @@ void SPSG_IoImpl::OnShutdown(uv_async_t*)
 
     for (auto& server_sessions : m_Sessions) {
         for (auto& session : server_sessions.first) {
-            session.Reset("Shutdown is in process");
+            session.Reset("Shutdown is in process", SUv_Tcp::eNormalClose);
         }
     }
 }
@@ -835,6 +1188,8 @@ void SPSG_DiscoveryImpl::OnShutdown(uv_async_t*)
     for (auto& server : servers) {
         server.throttling.StartClose();
     }
+
+    if (m_Stats) m_Stats->Stop();
 }
 
 void SPSG_IoImpl::AddNewServers(size_t servers_size, size_t sessions_size, uv_async_t* handle)
@@ -871,13 +1226,14 @@ void SPSG_IoImpl::OnQueue(uv_async_t* handle)
         }
     }
 
+    TPSG_MaxConcurrentSubmits::TValue remaining_submits = m_MaxConcurrentSubmits;
     auto d = m_Random.first;
     auto i = m_Sessions.begin();
     auto next_i = [&]() { if (++i == m_Sessions.end()) i = m_Sessions.begin(); };
 
     // We have to find available session first and then get a request,
     // as we may not be able to put the request back into the queue after (if it's full)
-    while (sessions) {
+    while (sessions && remaining_submits) {
         auto rate = d(m_Random.second);
         _DEBUG_ARG(const auto original_rate = rate);
 
@@ -903,7 +1259,7 @@ void SPSG_IoImpl::OnQueue(uv_async_t* handle)
             for (; (session != server_sessions.end()) && session->IsFull(); ++session);
 
             // All existing sessions are full and no new sessions are allowed
-            if ((session == server_sessions.end()) && (server_sessions.size() >= TPSG_MaxSessions::GetDefault())) {
+            if (session == server_sessions.end()) {
                 PSG_IO_TRACE("Server '" << server_name << "' has no room for a request");
 
             // Session is available or can be created
@@ -925,19 +1281,23 @@ void SPSG_IoImpl::OnQueue(uv_async_t* handle)
                         return;
                     }
 
-                    // All existing sessions are full
-                    if (session == server_sessions.end()) {
-                        server_sessions.emplace_back(server, queue, handle->loop);
-                        session = (server_sessions.rbegin() + 1).base();
-                        PSG_IO_TRACE("Additional session for server '" << server_name << "' was added");
-                    }
-
                     _DEBUG_ARG(const auto req_id = req->reply->debug_printout.id);
                     bool result = session->ProcessRequest(req);
 
                     if (result) {
                         PSG_IO_TRACE("Server '" << server_name << "' will get request '" <<
                                 req_id << "' with rate = " << original_rate);
+                        --remaining_submits;
+                        ++server.stats;
+
+                        // Add new session if allowed to
+                        if (session->IsFull() &&
+                                (distance(session, server_sessions.end()) == 1) &&
+                                (server_sessions.size() < TPSG_MaxSessions::GetDefault())) {
+                            server_sessions.emplace_back(server, queue, handle->loop);
+                            PSG_IO_TRACE("Additional session for server '" << server_name << "' was added");
+                        }
+
                         break;
                     } else {
                         PSG_IO_TRACE("Server '" << server_name << "' failed to process request '" <<
@@ -963,7 +1323,9 @@ void SPSG_IoImpl::OnQueue(uv_async_t* handle)
 
     // Continue processing of requests in the IO thread queue on next UV loop iteration
     queue.Signal();
-    PSG_IO_TRACE("No sessions available [anymore]");
+
+    PSG_IO_TRACE((sessions ? "Max concurrent submits reached" : "No sessions available [anymore]") <<
+            ", submitted: " << m_MaxConcurrentSubmits - remaining_submits);
 }
 
 void SPSG_DiscoveryImpl::OnTimer(uv_timer_t* handle)
@@ -1148,8 +1510,8 @@ void SPSG_IoImpl::AfterExecute()
 }
 
 SPSG_DiscoveryImpl::SNoServers::SNoServers(SPSG_Servers::TTS& servers) :
-    m_RetryDelay(s_SecondsToMs(TPSG_NoServersRetryDelay::GetDefault())),
-    m_Timeout(s_SecondsToMs(TPSG_RequestTimeout::GetDefault() * (1 + TPSG_RequestRetries::GetDefault()))),
+    m_RetryDelay(SecondsToMs(TPSG_NoServersRetryDelay::GetDefault())),
+    m_Timeout(SecondsToMs(TPSG_RequestTimeout::GetDefault() * (1 + TPSG_RequestRetries::GetDefault()))),
     m_FailRequests(const_cast<atomic_bool&>(servers->fail_requests))
 {
 }
@@ -1184,14 +1546,24 @@ bool SPSG_DiscoveryImpl::SNoServers::operator()(bool discovered, SUv_Timer* time
 
 /** SPSG_IoCoordinator */
 
+shared_ptr<SPSG_Stats> s_GetStats(SPSG_Servers::TTS& servers)
+{
+    if (TPSG_Stats::GetDefault()) {
+        return make_shared<SPSG_Stats>(servers);
+    } else {
+        return {};
+    }
+}
+
 uint64_t s_GetDiscoveryRepeat(const CServiceDiscovery& service)
 {
-    return service.IsSingleServer() ? 0 : s_SecondsToMs(TPSG_RebalanceTime::GetDefault());
+    return service.IsSingleServer() ? 0 : SecondsToMs(TPSG_RebalanceTime::GetDefault());
 }
 
 SPSG_IoCoordinator::SPSG_IoCoordinator(CServiceDiscovery service) :
+    stats(s_GetStats(m_Servers)),
     m_Barrier(TPSG_NumIo::GetDefault() + 2),
-    m_Discovery(m_Barrier, 0, s_GetDiscoveryRepeat(service), service, m_Servers),
+    m_Discovery(m_Barrier, 0, s_GetDiscoveryRepeat(service), service, stats, m_Servers),
     m_RequestCounter(0),
     m_RequestId(1)
 {
