@@ -1,4 +1,4 @@
-/*  $Id: objistrxml.cpp 647921 2022-04-05 12:06:40Z ivanov $
+/*  $Id: objistrxml.cpp 653650 2022-08-01 18:52:25Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -128,9 +128,10 @@ bool CObjectIStreamXml::EndOfData(void)
     return false;
 }
 
-string CObjectIStreamXml::GetPosition(void) const
+void CObjectIStreamXml::Location(string& loc_type, size_t& loc) const
 {
-    return "line "+NStr::SizetToString(m_Input.GetLine());
+    loc_type = "line";
+    loc = m_Input.GetLine();
 }
 
 void CObjectIStreamXml::SetEnforcedStdXml(bool set)
@@ -447,16 +448,19 @@ CTempString CObjectIStreamXml::ReadName(char c)
 
     // save beginning of tag name
     const char* ptr = m_Input.GetCurrentPos();
+    m_LastTag = string(ptr+iColon, i-iColon);
+    string ns_prefix;
+    if (iColon > 1) {
+        ns_prefix = string(ptr, iColon-1);
+    }
+    m_Input.SkipChars(i);
 
     // check end of tag name
-    m_Input.SkipChars(i);
     if (c == '\n' || c == '\r') {
         m_Input.SkipChar();
         m_Input.SkipEndOfLine(c);
     }
-    m_LastTag = string(ptr+iColon, i-iColon);
     if (iColon > 1) {
-        string ns_prefix(ptr, iColon-1);
         if (ns_prefix == "xmlns") {
             string value;
             ReadAttributeValue(value, true);
@@ -515,7 +519,7 @@ CTempString CObjectIStreamXml::ReadName(char c)
         char ch = SkipWS();
         return IsEndOfTagChar(ch) ? CTempString() : ReadName(ch);
     }
-    return CTempString(ptr+iColon, i-iColon);
+    return m_LastTag;
 }
 
 CTempString CObjectIStreamXml::RejectedName(void)
@@ -596,6 +600,12 @@ string CObjectIStreamXml::ReadFileHeader(void)
     }
     
     m_Doctype_found = false;
+    if (GetStackDepth() > 0) {
+        const CObjectStack::TFrame& top = TopFrame();
+        if (top.HasTypeInfo()) {
+            m_Doctype_found = top.GetTypeInfo()->GetDataSpec() == EDataSpec::eDTD;
+        }
+    }
     for ( ;; ) {
         switch ( BeginOpeningTag() ) {
         case '?':
@@ -606,7 +616,7 @@ string CObjectIStreamXml::ReadFileHeader(void)
                 m_Input.SkipChar();
                 CTempString tagName = ReadName(m_Input.PeekChar());
                 if ( tagName == "DOCTYPE" ) {
-                    m_Doctype_found = true;
+//                    m_Doctype_found = true;
                     ReadName(SkipWS());
                     // skip the rest of !DOCTYPE
                     for ( ;; ) {
@@ -2231,6 +2241,10 @@ CObjectIStreamXml::BeginClassMember(const CClassTypeInfo* classType,
                 TMemberIndex ind = first+1;
                 if (classType->GetMemberInfo(ind)->GetId().HasNotag()) {
                     TopFrame().SetNotag();
+                    if (x_IsStdXml() && !GetEnforcedStdXml()) {
+                        ETypeFamily type = GetRealTypeFamily(classType->GetMemberInfo(ind)->GetTypeInfo());
+                        m_SkipNextTag = type != eTypeFamilyPrimitive && type != eTypeFamilyContainer;
+                    }
                     return ind;
                 }
                 if ( NextTagIsClosing() )
@@ -2293,7 +2307,7 @@ CObjectIStreamXml::BeginClassMember(const CClassTypeInfo* classType,
 
     TMemberIndex ind = classType->GetMembers().Find(tagName);
     if (ind == kInvalidMember) {
-        ind = classType->GetMembers().FindDeep(tagName);
+        ind = classType->GetMembers().FindDeep(tagName, pos);
         if (ind != kInvalidMember && ind >= pos) {
             TopFrame().SetNotag();
             UndoClassMember();
