@@ -1,4 +1,4 @@
-/*  $Id: vdbread.cpp 629397 2021-04-15 14:17:16Z vasilche $
+/*  $Id: vdbread.cpp 653133 2022-07-21 14:50:05Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -588,9 +588,25 @@ static CKConfig s_InitProxyConfig()
 static DECLARE_TLS_VAR(const CRequestContext*, s_LastRequestContext);
 static DECLARE_TLS_VAR(CRequestContext::TVersion, s_LastRequestContextVersion);
 
+/*
+// Performance collecting code
+static struct SReport {
+    size_t count;
+    double time;
+    SReport() : count(0), time(0) {}
+    ~SReport() {
+        LOG_POST("GetRequestContext() called "<<count<<" times, spent "<<time<<"s");
+    }
+} dummy;
+*/
+
 static void s_UpdateVDBRequestContext(void)
 {
+    // Performance collecting code
+    // CStopWatch sw(CStopWatch::eStart);
     CRequestContext& req_ctx = CDiagContext::GetRequestContext();
+    // Performance collecting code
+    // dummy.count += 1; dummy.time += sw.Elapsed();
     auto req_ctx_version = req_ctx.GetVersion();
     if ( &req_ctx == s_LastRequestContext && req_ctx_version == s_LastRequestContextVersion ) {
         return;
@@ -694,6 +710,67 @@ static void s_VDBInit()
 // end of VDB library initialization code
 /////////////////////////////////////////////////////////////////////////////
         
+
+//#define UPDATE_REQUEST_CONTEXT_FINAL
+
+#ifndef UPDATE_REQUEST_CONTEXT_FINAL
+static DECLARE_TLS_VAR(size_t, s_RequestContextUpdaterRecursion);
+#endif
+
+class CFinalRequestContextUpdater {
+public:
+    CFinalRequestContextUpdater();
+    ~CFinalRequestContextUpdater();
+    CFinalRequestContextUpdater(const CFinalRequestContextUpdater&) = delete;
+};
+
+
+CVDBMgr::CRequestContextUpdater::CRequestContextUpdater()
+{
+#ifndef UPDATE_REQUEST_CONTEXT_FINAL
+    size_t r = s_RequestContextUpdaterRecursion;
+    s_RequestContextUpdaterRecursion = r+1;
+    if ( r != 0 ) {
+        return;
+    }
+    DECLARE_SDK_GUARD();
+    s_UpdateVDBRequestContext();
+#endif
+}
+
+
+inline CFinalRequestContextUpdater::CFinalRequestContextUpdater()
+{
+#ifndef UPDATE_REQUEST_CONTEXT_FINAL
+    size_t r = s_RequestContextUpdaterRecursion;
+    s_RequestContextUpdaterRecursion = r+1;
+    if ( r != 0 ) {
+        return;
+    }
+#endif
+    DECLARE_SDK_GUARD();
+    s_UpdateVDBRequestContext();
+}
+
+
+CVDBMgr::CRequestContextUpdater::~CRequestContextUpdater()
+{
+#ifndef UPDATE_REQUEST_CONTEXT_FINAL
+    size_t r = s_RequestContextUpdaterRecursion;
+    s_RequestContextUpdaterRecursion = r-1;
+#endif
+}
+
+
+inline CFinalRequestContextUpdater::~CFinalRequestContextUpdater()
+{
+#ifndef UPDATE_REQUEST_CONTEXT_FINAL
+    size_t r = s_RequestContextUpdaterRecursion;
+    s_RequestContextUpdaterRecursion = r-1;
+#endif
+}
+
+
 void CVDBMgr::x_Init(void)
 {
     s_VDBInit();
@@ -755,7 +832,7 @@ CVDB::CVDB(const CVDBMgr& mgr, const string& acc_or_path)
     : m_Name(acc_or_path)
 {
     DECLARE_SDK_GUARD();
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     string path = CVPath::ConvertAccOrSysPathToPOSIX(acc_or_path);
     if ( rc_t rc = VDBManagerOpenDBRead(mgr, x_InitPtr(), 0, "%.*s",
                                         int(path.size()), path.data()) ) {
@@ -819,7 +896,7 @@ CVDBTable::CVDBTable(const CVDB& db,
       m_Name(table_name)
 {
     DECLARE_SDK_GUARD();
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     if ( rc_t rc = VDatabaseOpenTableRead(db, x_InitPtr(), table_name) ) {
         *x_InitPtr() = 0;
         RCState rc_state = GetRCState(rc);
@@ -850,7 +927,7 @@ CVDBTable::CVDBTable(const CVDBMgr& mgr,
 {
     *x_InitPtr() = 0;
     DECLARE_SDK_GUARD();
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     string path = CVPath::ConvertAccOrSysPathToPOSIX(acc_or_path);
     if ( rc_t rc = VDBManagerOpenTableRead(mgr, x_InitPtr(), 0, "%.*s",
                                            int(path.size()), path.data()) ) {
@@ -917,7 +994,7 @@ CVDBTableIndex::CVDBTableIndex(const CVDBTable& table,
     : m_Table(table),
       m_Name(index_name)
 {
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     if ( rc_t rc = VTableOpenIndexRead(table, x_InitPtr(), index_name) ) {
         *x_InitPtr() = 0;
         if ( GetRCObject(rc) == RCObject(rcIndex) &&
@@ -981,12 +1058,17 @@ CNcbiOstream& operator<<(CNcbiOstream& out, const CVDBCursor& obj)
 
 void CVDBCursor::Init(const CVDBTable& table)
 {
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     if ( *this ) {
         NCBI_THROW2(CSraException, eInvalidState,
                     "Cannot init VDB cursor again",
                     RC(rcApp, rcCursor, rcConstructing, rcSelf, rcOpen));
     }
+    if ( !table ) { // VTableCreateCursorRead lacks check for null table argument
+        NCBI_THROW2(CSraException, eNullPtr,
+                    "Cannot init VDB cursor",
+                    RC(rcApp, rcCursor, rcConstructing, rcTable, rcNull));
+    }
     if ( rc_t rc = VTableCreateCursorRead(table, x_InitPtr()) ) {
         *x_InitPtr() = 0;
         NCBI_THROW2(CSraException, eInitFailed,
@@ -1020,7 +1102,7 @@ void CVDBCursor::CloseRow(void)
 rc_t CVDBCursor::OpenRowRc(TVDBRowId row_id)
 {
     CloseRow();
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     if ( rc_t rc = VCursorSetRowId(*this, row_id) ) {
         return rc;
     }
@@ -1063,7 +1145,7 @@ TVDBRowId CVDBCursor::GetMaxRowId(void) const
 
 void CVDBCursor::SetParam(const char* name, const CTempString& value) const
 {
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     if ( rc_t rc = VCursorParamsSet
          ((struct VCursorParams *)GetPointer(),
           name, "%.*s", value.size(), value.data()) ) {
@@ -1078,7 +1160,7 @@ uint32_t CVDBCursor::GetElementCount(TVDBRowId row, const CVDBColumn& column,
                                      uint32_t elem_bits) const
 {
     DECLARE_SDK_GET_GUARD();
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     uint32_t read_count, remaining_count;
     if ( rc_t rc = VCursorReadBitsDirect(*this, row, column.GetIndex(),
                                          elem_bits, 0, 0, 0, 0,
@@ -1097,7 +1179,7 @@ void CVDBCursor::ReadElements(TVDBRowId row, const CVDBColumn& column,
                               void* buffer) const
 {
     DECLARE_SDK_GET_GUARD();
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     uint32_t read_count, remaining_count;
     if ( rc_t rc = VCursorReadBitsDirect(*this, row, column.GetIndex(),
                                          elem_bits, start, buffer, 0, count,
@@ -1112,6 +1194,9 @@ void CVDBCursor::ReadElements(TVDBRowId row, const CVDBColumn& column,
                        '['<<row<<"]["<<start<<".."<<(start+count-1)<<
                        "] only "<<read_count<<" elements are read");
     }
+    if ( s_GetDebugLevel() >= 9 ) {
+        LOG_POST(Info<<"VDB "<<*this<<column<<'['<<row<<"]: @"<<start<<" #"<<count);
+    }
 }
 
 
@@ -1210,7 +1295,7 @@ void CVDBColumn::Init(const CVDBCursor& cursor,
                       EMissing missing)
 {
     DECLARE_SDK_GUARD();
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     m_Name = name;
     if ( rc_t rc = VCursorAddColumn(cursor, &m_Index, name) ) {
         if ( backup_name &&
@@ -1247,18 +1332,21 @@ void CVDBColumn::Init(const CVDBCursor& cursor,
 }
 
 
+bool CVDBColumn::IsStatic(const CVDBCursor& cursor) const
+{
+    bool static_value;
+    if ( rc_t rc = VCursorIsStaticColumn(cursor, GetIndex(), &static_value) ) {
+        NCBI_THROW2_FMT(CSraException, eInvalidState,
+                        "Cannot get static status of "<<cursor<<*this,rc);
+    }
+    return static_value;
+}
+
+
 void CVDBColumn::ResetIfAlwaysEmpty(const CVDBCursor& cursor)
 {
-    if ( !*this ) {
-        return;
-    }
-    bool static_value;
-    if ( VCursorIsStaticColumn(cursor, GetIndex(), &static_value) ) {
-        // cannot get 'static' value -> do nothing
-        return;
-    }
     try {
-        if ( static_value && CVDBValue(cursor, 1, *this).empty() ) {
+        if ( IsStatic(cursor) && CVDBValue(cursor, 1, *this).empty() ) {
             // value is always empty
             Reset();
         }
@@ -1296,7 +1384,7 @@ CNcbiOstream& operator<<(CNcbiOstream& out, const CVDBValue& obj)
 void CVDBValue::x_Get(const CVDBCursor& cursor, const CVDBColumn& column)
 {
     DECLARE_SDK_GET_GUARD();
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     uint32_t bit_offset, bit_length;
     if ( rc_t rc = VCursorCellData(cursor, column.GetIndex(),
                                    &bit_length, &m_Data, &bit_offset,
@@ -1310,6 +1398,40 @@ void CVDBValue::x_Get(const CVDBCursor& cursor, const CVDBColumn& column)
                         <<cursor<<column<<": "<<bit_offset,
                         RC(rcApp, rcColumn, rcDecoding, rcOffset, rcUnsupported));
     }
+    if ( s_GetDebugLevel() >= 9 ) {
+        CNcbiOstrstream s;
+        if ( bit_length == 8 ) {
+            s << '"' << NStr::PrintableString(CTempString((const char*)m_Data, m_ElemCount)) << '"';
+        }
+        else if ( bit_length == 16 ) {
+            for ( uint32_t i = 0; i < m_ElemCount; ++i ) {
+                if ( i ) {
+                    s << ", ";
+                }
+                s << ((const int16_t*)m_Data)[i];
+            }
+        }
+        else if ( bit_length == 32 ) {
+            for ( uint32_t i = 0; i < m_ElemCount; ++i ) {
+                if ( i ) {
+                    s << ", ";
+                }
+                s << ((const int32_t*)m_Data)[i];
+            }
+        }
+        else if ( bit_length == 64 ) {
+            for ( uint32_t i = 0; i < m_ElemCount; ++i ) {
+                if ( i ) {
+                    s << ", ";
+                }
+                s << ((const int64_t*)m_Data)[i];
+            }
+        }
+        else {
+            s << "*** bad bit_length="<<bit_length;
+        }
+        LOG_POST(Info<<"VDB "<<cursor<<column<<": "<<CNcbiOstrstreamToString(s));
+    }
     m_Ref.Set(cursor, 0, column);
 }
 
@@ -1320,7 +1442,7 @@ void CVDBValue::x_Get(const CVDBCursor& cursor,
                       EMissing missing)
 {
     DECLARE_SDK_GET_GUARD();
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     uint32_t bit_offset, bit_length;
     if ( rc_t rc = VCursorCellDataDirect(cursor, row, column.GetIndex(),
                                          &bit_length, &m_Data, &bit_offset,
@@ -1434,7 +1556,7 @@ void CVDBValueFor4Bits::x_Get(const CVDBCursor& cursor,
                               const CVDBColumn& column)
 {
     DECLARE_SDK_GET_GUARD();
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     uint32_t bit_offset, bit_length, elem_count;
     const void* data;
     if ( rc_t rc = VCursorCellDataDirect(cursor, row, column.GetIndex(),
@@ -1453,6 +1575,21 @@ void CVDBValueFor4Bits::x_Get(const CVDBCursor& cursor,
     m_RawData = static_cast<const char*>(data);
     m_ElemOffset = bit_offset >> 2;
     m_ElemCount = elem_count;
+    if ( s_GetDebugLevel() >= 9 ) {
+        CNcbiOstrstream s;
+        if ( bit_length == 4 ) {
+            for ( uint32_t i = 0; i < elem_count; ++i ) {
+                if ( i ) {
+                    s << ", ";
+                }
+                s << '?';
+            }
+        }
+        else {
+            s << "*** bad bit_length="<<bit_length;
+        }
+        LOG_POST(Info<<"VDB "<<cursor<<column<<'['<<row<<"]: "<<CNcbiOstrstreamToString(s));
+    }
     m_Ref.Set(cursor, row, column);
 }
 
@@ -1515,7 +1652,7 @@ void CVDBValueFor2Bits::x_Get(const CVDBCursor& cursor,
                               const CVDBColumn& column)
 {
     DECLARE_SDK_GET_GUARD();
-    s_UpdateVDBRequestContext();
+    CFinalRequestContextUpdater ctx_updater;
     uint32_t bit_offset, bit_length, elem_count;
     const void* data;
     if ( rc_t rc = VCursorCellDataDirect(cursor, row, column.GetIndex(),
@@ -1534,6 +1671,21 @@ void CVDBValueFor2Bits::x_Get(const CVDBCursor& cursor,
     m_RawData = static_cast<const char*>(data);
     m_ElemOffset = bit_offset >> 1;
     m_ElemCount = elem_count;
+    if ( s_GetDebugLevel() >= 9 ) {
+        CNcbiOstrstream s;
+        if ( bit_length == 2 ) {
+            for ( uint32_t i = 0; i < elem_count; ++i ) {
+                if ( i ) {
+                    s << ", ";
+                }
+                s << '?';
+            }
+        }
+        else {
+            s << "*** bad bit_length="<<bit_length;
+        }
+        LOG_POST(Info<<"VDB "<<cursor<<column<<'['<<row<<"]: "<<CNcbiOstrstreamToString(s));
+    }
     m_Ref.Set(cursor, row, column);
 }
 
