@@ -1,4 +1,4 @@
-/*  $Id: wgsresolver_impl.cpp 648942 2022-04-25 12:53:18Z ivanov $
+/*  $Id: wgsresolver_impl.cpp 652896 2022-07-18 18:39:07Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,6 +37,7 @@
 #include <corelib/ncbi_param.hpp>
 #include <util/line_reader.hpp>
 #include <sra/error_codes.hpp>
+#include <kns/http.h>
 
 #include <objects/seqloc/Seq_id.hpp>
 #include <objects/general/Dbtag.hpp>
@@ -44,7 +45,7 @@
 BEGIN_NCBI_NAMESPACE;
 
 #define NCBI_USE_ERRCODE_X   WGSResolver
-NCBI_DEFINE_ERR_SUBCODE_X(32);
+NCBI_DEFINE_ERR_SUBCODE_X(36);
 
 BEGIN_NAMESPACE(objects);
 
@@ -353,7 +354,7 @@ CRef<CWGSResolver> CWGSResolver_VDB::CreateResolver(const CVDBMgr& mgr)
     if ( ret2->IsValid() ) {
         ret = ret2;
     }
-    if ( !ret->m_AccIndexIsPrefix ) {
+    if ( !ret->AccIndexIsPrefix() ) {
         CRef<CWGSResolver_VDB> ret3(new CWGSResolver_VDB(mgr, eThirdIndex, ret));
         if ( ret3->IsValid() ) {
             ret = ret3;
@@ -372,13 +373,8 @@ void CWGSResolver_VDB::Close(void)
 
 void CWGSResolver_VDB::x_Close()
 {
+    m_Impl.reset();
     m_Mgr.Close();
-    m_Db.Close();
-    m_GiIdxTable.Close();
-    m_AccIdxTable.Close();
-    m_AccIndex.Close();
-    m_GiIdxCursorCache.Clear();
-    m_AccIdxCursorCache.Clear();
 }
 
 
@@ -418,16 +414,204 @@ static string s_ResolveAccOrPath(const CVDBMgr& mgr, const string& acc_or_path)
 }
 
 
+DECLARE_SRA_REF_TRAITS(KClientHttpRequest, );
+DECLARE_SRA_REF_TRAITS(KClientHttpResult, );
+
+DEFINE_SRA_REF_TRAITS(KClientHttpRequest, );
+DEFINE_SRA_REF_TRAITS(KClientHttpResult, );
+
+class NCBI_SRAREAD_EXPORT CClientHttpRequest
+    : public CSraRef<KClientHttpRequest>
+{
+public:
+    CClientHttpRequest(const CKNSManager& mgr, const string& path)
+        {
+            if ( rc_t rc = KNSManagerMakeClientRequest(mgr, x_InitPtr(), 0x01010000, 0,
+                                                       "%s", path.c_str()) ) {
+                *x_InitPtr() = 0;
+                NCBI_THROW2(CSraException, eInitFailed,
+                            "Cannot create http client request", rc);
+            }
+        }
+
+private:
+};
+
+
+class NCBI_SRAREAD_EXPORT CClientHttpResult
+    : public CSraRef<KClientHttpResult>
+{
+public:
+    enum EHead {
+        eHead
+    };
+    
+    CClientHttpResult(const CClientHttpRequest& request, EHead)
+        {
+            if ( rc_t rc = KClientHttpRequestHEAD(request, x_InitPtr()) ) {
+                *x_InitPtr() = 0;
+                NCBI_THROW2(CSraException, eInitFailed,
+                            "Cannot get http HEAD", rc);
+            }
+        }
+    
+private:
+};
+
+
+static CTime s_GetURLTimestamp(const CVDBMgr& mgr, const string& path)
+{
+    try {
+        CVFSManager vfs(mgr);
+        CKNSManager kns(vfs);
+        CClientHttpRequest request(kns, path);
+        CClientHttpResult result(request, CClientHttpResult::eHead);
+        char buffer[99];
+        size_t size;
+        if ( rc_t rc = KClientHttpResultGetHeader(result, "Last-Modified",
+                                                  buffer, sizeof(buffer), &size) ) {
+            NCBI_THROW2(CSraException, eNotFound,
+                        "No Last-Modified header in HEAD response", rc);
+        }
+        CTempString str(buffer, size);
+        CTimeFormat fmt("w, d b Y h:m:s Z"); // standard Last-Modified HTTP header format
+        return CTime(str, fmt);
+    }
+    catch ( CException& exc ) {
+        if ( CWGSResolver::s_DebugEnabled(CWGSResolver::eDebug_error) ) {
+            ERR_POST_X(36, "CWGSResolver_VDB: cannot get URL timestamp of "<<path<<": "<<exc);
+        }
+        return CTime();
+    }
+}
+
+
+static CTime s_GetTimestamp(const CVDBMgr& mgr, const string& path)
+{
+    CTime timestamp;
+    if ( path[0] == 'h' &&
+         (NStr::StartsWith(path, "http://") ||
+          NStr::StartsWith(path, "https://")) ) {
+        // try http:
+        timestamp = s_GetURLTimestamp(mgr, path);
+    }
+    else {
+        // try direct file access
+        if ( CDirEntry(path).GetTime(&timestamp) ) {
+            timestamp.ToUniversalTime();
+        }
+        else {
+            timestamp = CTime();
+        }
+    }
+    if ( timestamp.IsEmpty() ) {
+        if ( CWGSResolver::s_DebugEnabled(CWGSResolver::eDebug_error) ) {
+            ERR_POST_X(34, "CWGSResolver_VDB("<<path<<"): "
+                       "cannot get timestamp");
+        }
+    }
+    else {
+        _ASSERT(timestamp.IsUniversalTime());
+        if ( CWGSResolver::s_DebugEnabled(CWGSResolver::eDebug_open) ) {
+            LOG_POST_X(35, "CWGSResolver_VDB("<<path<<"): "
+                       "timestamp: "<<CTime(timestamp).ToLocalTime());
+        }
+    }
+    return timestamp;
+}
+
+
 void CWGSResolver_VDB::Open(const CVDBMgr& mgr, const string& acc_or_path)
 {
-    string path = s_ResolveAccOrPath(mgr, acc_or_path);
-
     // open VDB file
     TDBMutex::TWriteLockGuard guard(m_DBMutex);
-    x_Close();
-    m_Mgr = mgr;
+    unique_ptr<SImpl> impl = make_unique<SImpl>(mgr, acc_or_path);
+    if ( impl->m_Db ) {
+        m_WGSIndexPath = acc_or_path;
+        swap(m_Impl, impl);
+    }
+    else if ( IsValid() ) {
+        if ( s_DebugEnabled(eDebug_error) ) {
+            ERR_POST_X(33, "CWGSResolver_VDB("<<acc_or_path<<"): "
+                       "index disappeared from "<<GetWGSIndexResolvedPath());
+        }
+    }
+}
+
+
+inline
+CRef<CWGSResolver_VDB::SGiIdxTableCursor> CWGSResolver_VDB::SImpl::GiIdx(TIntId row)
+{
+    CRef<SGiIdxTableCursor> curs = m_GiIdxCursorCache.Get(row);
+    if ( !curs ) {
+        curs = new SGiIdxTableCursor(GiIdxTable());
+    }
+    return curs;
+}
+
+
+inline
+CRef<CWGSResolver_VDB::SAccIdxTableCursor> CWGSResolver_VDB::SImpl::AccIdx(void)
+{
+    CRef<SAccIdxTableCursor> curs = m_AccIdxCursorCache.Get();
+    if ( !curs ) {
+        curs = new SAccIdxTableCursor(AccIdxTable());
+    }
+    return curs;
+}
+
+
+inline
+void CWGSResolver_VDB::SImpl::Put(CRef<SGiIdxTableCursor>& curs, TIntId row)
+{
+    if ( curs->m_Table == GiIdxTable() ) {
+        m_GiIdxCursorCache.Put(curs, row);
+    }
+}
+
+
+inline
+void CWGSResolver_VDB::SImpl::Put(CRef<SAccIdxTableCursor>& curs)
+{
+    if ( curs->m_Table == AccIdxTable() ) {
+        m_AccIdxCursorCache.Put(curs);
+    }
+}
+
+
+inline
+CRef<CWGSResolver_VDB::SGiIdxTableCursor> CWGSResolver_VDB::GiIdx(TIntId gi)
+{
+    return m_Impl->GiIdx(gi);
+}
+
+
+inline
+CRef<CWGSResolver_VDB::SAccIdxTableCursor> CWGSResolver_VDB::AccIdx(void)
+{
+    return m_Impl->AccIdx();
+}
+
+
+inline
+void CWGSResolver_VDB::Put(CRef<SGiIdxTableCursor>& curs, TIntId gi)
+{
+    m_Impl->Put(curs, gi);
+}
+
+
+inline
+void CWGSResolver_VDB::Put(CRef<SAccIdxTableCursor>& curs)
+{
+    m_Impl->Put(curs);
+}
+
+
+CWGSResolver_VDB::SImpl::SImpl(const CVDBMgr& mgr, const string& acc_or_path)
+{
+    string path = s_ResolveAccOrPath(mgr, acc_or_path);
     try {
-        m_Db = CVDB(m_Mgr, path);
+        m_Db = CVDB(mgr, path);
     }
     catch ( CSraException& exc ) {
         if ( exc.GetErrCode() == exc.eNotFoundDb ) {
@@ -437,16 +621,8 @@ void CWGSResolver_VDB::Open(const CVDBMgr& mgr, const string& acc_or_path)
     }
 
     // save original argument for possible changes in symbolic links
-    m_WGSIndexPath = acc_or_path;
     m_WGSIndexResolvedPath = path;
-    if ( !CDirEntry(path).GetTime(&m_Timestamp) ) {
-        m_Timestamp = CTime();
-    }
-    else {
-        if ( s_DebugEnabled(eDebug_open) ) {
-            LOG_POST_X(30, "CWGSResolver_VDB("<<acc_or_path<<"): index timestamp: "<<m_Timestamp);
-        }
-    }
+    m_Timestamp = s_GetTimestamp(mgr, path);
     m_GiIdxTable = CVDBTable(m_Db, "GI_IDX");
     m_AccIdxTable = CVDBTable(m_Db, "ACC_IDX");
     m_AccIndexIsPrefix = true;
@@ -455,6 +631,15 @@ void CWGSResolver_VDB::Open(const CVDBMgr& mgr, const string& acc_or_path)
         m_AccIndexIsPrefix = false;
         m_AccIndex = CVDBTableIndex(m_AccIdxTable, "accession");
     }
+    if ( s_DebugEnabled(eDebug_open) ) {
+        auto gi_idx = GiIdx();
+        auto acc_idx = AccIdx();
+        LOG_POST_X(33, "CWGSResolver_VDB("<<acc_or_path<<"):"
+                   " gi_rows: "<<gi_idx->m_Cursor.GetMaxRowId()<<
+                   " acc_rows: "<<acc_idx->m_Cursor.GetMaxRowId());
+        Put(acc_idx);
+        Put(gi_idx);
+    }
 }
 
 
@@ -482,86 +667,50 @@ bool CWGSResolver_VDB::x_Update(void)
     string path = s_ResolveAccOrPath(m_Mgr, GetWGSIndexPath());
     if ( path != GetWGSIndexResolvedPath() ) {
         // resolved to a different path -> new index by symbolic link
-        LOG_POST_X(32, "CWGSResolver_VDB: new index path: "<<path);
+        if ( s_DebugEnabled(eDebug_open) ) {
+            LOG_POST_X(32, "CWGSResolver_VDB: new index path: "<<path);
+        }
         Reopen();
         return true;
     }
 
-    CTime timestamp;
-    if ( !CDirEntry(path).GetTime(&timestamp) ) {
-        // cannot get timestamp -> remote reference
-        return false;
+    CTime timestamp = s_GetTimestamp(m_Mgr, path);
+    if ( timestamp.IsEmpty() ) {
+        // cannot get timestamp, always reopen
+        Reopen();
+        return true;
     }
-    if ( timestamp == m_Timestamp ) {
+    if ( timestamp == GetTimestamp() ) {
         // same timestamp
         return false;
     }
-    if ( s_DebugEnabled(eDebug_open) ) {
-        LOG_POST_X(31, "CWGSResolver_VDB: new index timestamp: "<<timestamp);
-    }
     Reopen();
     return true;
 }
 
 
-inline
-CRef<CWGSResolver_VDB::SGiIdxTableCursor> CWGSResolver_VDB::GiIdx(TIntId row)
+CWGSResolver::TWGSPrefixes CWGSResolver_VDB::x_GetGiPrefixes(TGi gi)
 {
-    CRef<SGiIdxTableCursor> curs = m_GiIdxCursorCache.Get(row);
-    if ( !curs ) {
-        curs = new SGiIdxTableCursor(GiIdxTable());
-    }
-    return curs;
-}
-
-
-inline
-CRef<CWGSResolver_VDB::SAccIdxTableCursor> CWGSResolver_VDB::AccIdx(void)
-{
-    CRef<SAccIdxTableCursor> curs = m_AccIdxCursorCache.Get();
-    if ( !curs ) {
-        curs = new SAccIdxTableCursor(AccIdxTable());
-    }
-    return curs;
-}
-
-
-inline
-void CWGSResolver_VDB::Put(CRef<SGiIdxTableCursor>& curs, TIntId row)
-{
-    if ( curs->m_Table == GiIdxTable() ) {
-        m_GiIdxCursorCache.Put(curs, row);
-    }
-}
-
-
-inline
-void CWGSResolver_VDB::Put(CRef<SAccIdxTableCursor>& curs)
-{
-    if ( curs->m_Table == AccIdxTable() ) {
-        m_AccIdxCursorCache.Put(curs);
-    }
-}
-
-
-CWGSResolver::TWGSPrefixes CWGSResolver_VDB::GetPrefixes(TGi gi)
-{
-    TDBMutex::TReadLockGuard guard(m_DBMutex);
     TWGSPrefixes ret;
+    TDBMutex::TReadLockGuard guard(m_DBMutex);
     if ( s_DebugEnabled(eDebug_resolve) ) {
         LOG_POST_X(24, "CWGSResolver_VDB("<<GetWGSIndexPath()<<"): Resolving "<<gi);
     }
-    CRef<SGiIdxTableCursor> cur = GiIdx();
-    CVDBStringValue value = cur->WGS_PREFIX(GI_TO(TVDBRowId, gi), CVDBValue::eMissing_Allow);
-    if ( !value.empty() ) {
-        if ( s_DebugEnabled(eDebug_resolve) ) {
-            LOG_POST_X(25, "CWGSResolver_VDB("<<GetWGSIndexPath()<<"): WGS prefix "<<*value);
+    try {
+        CRef<SGiIdxTableCursor> cur = GiIdx();
+        CVDBStringValue value = cur->WGS_PREFIX(GI_TO(TVDBRowId, gi), CVDBValue::eMissing_Allow);
+        if ( !value.empty() ) {
+            if ( s_DebugEnabled(eDebug_resolve) ) {
+                LOG_POST_X(25, "CWGSResolver_VDB("<<GetWGSIndexPath()<<"): WGS prefix "<<*value);
+            }
+            ret.push_back(*value);
         }
-        ret.push_back(*value);
+        m_Impl->m_FailedAccRequestCount.Set(0);
+        Put(cur);
     }
-    Put(cur);
-    if ( ret.empty() && m_NextResolver ) {
-        ret = m_NextResolver->GetPrefixes(gi);
+    catch ( ... ) {
+        m_Impl->m_FailedAccRequestCount.Add(1);
+        throw;
     }
     return ret;
 }
@@ -588,16 +737,16 @@ static inline bool s_SplitAccIndex(string& uacc, Uint2& key_num)
 }
 
 
-CWGSResolver::TWGSPrefixes CWGSResolver_VDB::GetPrefixes(const string& acc)
+CWGSResolver::TWGSPrefixes CWGSResolver_VDB::x_GetAccPrefixes(const string& acc)
 {
-    TDBMutex::TReadLockGuard guard(m_DBMutex);
     TWGSPrefixes ret;
+    TDBMutex::TReadLockGuard guard(m_DBMutex);
     if ( s_DebugEnabled(eDebug_resolve) ) {
         LOG_POST_X(26, "CWGSResolver_VDB("<<GetWGSIndexPath()<<"): Resolving "<<acc);
     }
     string uacc = acc;
     SAccIdxTableCursor::acc_range_number_t key_num = 0;
-    if ( m_AccIndexIsPrefix ) {
+    if ( AccIndexIsPrefix() ) {
         if ( !s_SplitAccIndex(uacc, key_num) ) {
             if ( s_DebugEnabled(eDebug_resolve) ) {
                 LOG_POST_X(27, "CWGSResolver_VDB("<<GetWGSIndexPath()<<"): invalid accession");
@@ -606,41 +755,67 @@ CWGSResolver::TWGSPrefixes CWGSResolver_VDB::GetPrefixes(const string& acc)
         }
     }
     NStr::ToUpper(uacc);
-    TVDBRowIdRange range;
-    {{
-        PROFILE(sw_AccFind);
-        range = m_AccIndex.Find(uacc);
-    }}
-    if ( s_DebugEnabled(eDebug_resolve) ) {
-        LOG_POST_X(27, "CWGSResolver_VDB("<<GetWGSIndexPath()<<"): "
-                   "range "<<range.first<<"-"<<range.second);
-    }
-    if ( range.second ) {
-        CRef<SAccIdxTableCursor> cur = AccIdx();
-        for ( TVDBRowCount i = 0; i < range.second; ++i ) {
-            TVDBRowId row_id = range.first+i;
-            if ( m_AccIndexIsPrefix ) {
-                PROFILE(sw_AccRange);
-                CVDBValueFor<SAccIdxTableCursor::acc_range_number_t> v =
-                    cur->ACCESSION_RANGE(row_id);
-                if ( v[0] > key_num ) {
-                    // current range is past the requested id, end of scan
-                    break;
-                }
-                if ( v[1] < key_num ) {
-                    // current range is before the requested id, check next range
-                    continue;
-                }
-            }
-            PROFILE(sw_WGSPrefix);
-            CTempString prefix = *cur->WGS_PREFIX(row_id);
-            if ( s_DebugEnabled(eDebug_resolve) ) {
-                LOG_POST_X(27, "CWGSResolver_VDB("<<GetWGSIndexPath()<<"): WGS prefix "<<prefix);
-            }
-            ret.push_back(prefix);
+    try {
+        TVDBRowIdRange range;
+        {{
+            PROFILE(sw_AccFind);
+            range = m_Impl->m_AccIndex.Find(uacc);
+        }}
+        if ( s_DebugEnabled(eDebug_resolve) ) {
+            LOG_POST_X(27, "CWGSResolver_VDB("<<GetWGSIndexPath()<<"): "
+                       "range "<<range.first<<"-"<<range.second);
         }
-        Put(cur);
+        if ( range.second ) {
+            CRef<SAccIdxTableCursor> cur = AccIdx();
+            for ( TVDBRowCount i = 0; i < range.second; ++i ) {
+                TVDBRowId row_id = range.first+i;
+                if ( AccIndexIsPrefix() ) {
+                    PROFILE(sw_AccRange);
+                    CVDBValueFor<SAccIdxTableCursor::acc_range_number_t> v =
+                        cur->ACCESSION_RANGE(row_id);
+                    if ( v[0] > key_num ) {
+                        // current range is past the requested id, end of scan
+                        break;
+                    }
+                    if ( v[1] < key_num ) {
+                        // current range is before the requested id, check next range
+                        continue;
+                    }
+                }
+                PROFILE(sw_WGSPrefix);
+                CTempString prefix = *cur->WGS_PREFIX(row_id);
+                if ( s_DebugEnabled(eDebug_resolve) ) {
+                    LOG_POST_X(27, "CWGSResolver_VDB("<<GetWGSIndexPath()<<"): WGS prefix "<<prefix);
+                }
+                ret.push_back(prefix);
+            }
+            Put(cur);
+        }
+        m_Impl->m_FailedGiRequestCount.Set(0);
     }
+    catch ( ... ) {
+        m_Impl->m_FailedGiRequestCount.Add(1);
+        throw;
+    }
+    return ret;
+}
+
+
+CWGSResolver::TWGSPrefixes CWGSResolver_VDB::GetPrefixes(TGi gi)
+{
+    CVDBMgr::CRequestContextUpdater ctx_updater;
+    TWGSPrefixes ret = x_GetGiPrefixes(gi);
+    if ( ret.empty() && m_NextResolver ) {
+        ret = m_NextResolver->GetPrefixes(gi);
+    }
+    return ret;
+}
+
+
+CWGSResolver::TWGSPrefixes CWGSResolver_VDB::GetPrefixes(const string& acc)
+{
+    CVDBMgr::CRequestContextUpdater ctx_updater;
+    auto ret = x_GetAccPrefixes(acc);
     if ( ret.empty() && m_NextResolver ) {
         ret = m_NextResolver->GetPrefixes(acc);
     }
